/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.algorithms.data;

/**
 * A Bernoulli distribution is defined on a sample space of two events.
 * For example: tossing a coin, did I get heads?; Throwing a die, did I get 
 * die side 5?.
 * This class labels the two items in the sample space as 0 (failure) and 
 * 1 (success). Giving the probability of the success event (1) also 
 * determince the probability of failure. 
 * This class is adapted from <I>Smalltalk-80: The language
 * and Its Implementation, Goldberg & Robson, Addison-Wesley, 1983, chapter 21, 
 * pages 417-438.
 * @version 1.0 10 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class Bernoulli extends DiscreteProbability
	{
	
	protected double successProbability;
	
	/**
	 * Returns a Bernoulli with the propability of success equal to
	 * successProbability.
	 * @exception OutOfBoundsException thrown if successProbability
	 * is not between 0 and 1.
	 */
	public Bernoulli( double successProbability ) throws OutOfBoundsException
		{
		if ( ( successProbability >= 0.0 ) && ( successProbability <= 1.0 ) )
			this.successProbability = successProbability;
		else
			throw new OutOfBoundsException( "Probablility must be between 0 and 1" +
				"Your value was: " + successProbability);
		}
		
	/**
	 * Returns the probability of an item in the sample
	 * space will occur. sampleSpaceItem must be either
	 * 0 or 1.
	 * @param sampleSpaceItem item in the sample space for which
	 * the probability of occuring is computed.
	 * @exception OutOfBoundsException thrown if sampleSpaceItem is not
	 * either 0 or 1.
	 */
	public double density( double sampleSpaceItem ) throws OutOfBoundsException
		{
		int bernoulliItem = (int) Math.round( sampleSpaceItem );
		if ( bernoulliItem == 1 )
			return successProbability;
		else if ( bernoulliItem == 0 ) 
			return ( 1 - successProbability );
		else
			throw new OutOfBoundsException( "Bernoulli trails must result 0 and 1");
		}
		
	/**
	 * Returns the mean of the Bernoulli distribution
	 */	
	public double mean()
		{
		return successProbability;
		}

	/**
	 * Returns the variance of the probability distribution
	 */	
	public double variance()
		{
		return successProbability * ( 1.0 - successProbability);
		}

	/**
	 * Returns 1 or 0. 1 is returned with the probability
	 * of success, set in the constructor.
	 */
	public double nextElement()
		{
		return inverseDistribution(  randomNumber.nextFloat() );
		}
	
	protected double inverseDistribution( double random )
		{
		if ( random <= successProbability )
			return 1;
		else
			return 0;
		}
	}
