/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.algorithms.data;

/**
 * The 80%-20% rule states that 80%
 * of the programs resources are consumed by  
 * 20% of the code. Computing 
 * folklore contains claims that this rule applies to various aspects of computing.
 * To generalize the rule assume we have N consumers (lines of code, etc).
 * Label the consumers 1, 2, ..., N, ordered by percent of resources ( time, 
 * memory, etc) by the consumer. So consumer 1 consumes the most resources,
 * consumer N consumes the least resources.
 * The generalized rule states that alpha per cent of 
 * the resources are consumed by the first (100% - alpha) % of the consumers,
 * and so on recursively, where
 * .5 <= alpha <= 1. 
 * Reference: <I>Handbook of Algorithms and Data Structures</I>, G. H. Gonnet, 
 * Addison-Wesley, 1984, pp 207-208. 
 * @version 1.0 12 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class GeneralizedEightyTwenty extends NoInverseDistribution
	{
	
	private double theta;
	
	/**
	 * Returns a GeneralizedEightyTwenty distribution.
	 * @param percentConsumed Percent of resources consumed by 100 - percentConsumed of consumer.
	 * @exception OutOfBoundsException thrown if percentConsumed is 
	 * not greater in the interval [.5, 1].
	 */
	public GeneralizedEightyTwenty( double percentConsumed, int numberOfConsumers ) 
		{
		if ( ( percentConsumed < .5 ) || ( percentConsumed > 1 ) )
			throw new OutOfBoundsException( 
				"percentConsumed must be in the interval [.5, 1, your value was " + percentConsumed);
		if (  numberOfConsumers < 1 ) 
			throw new OutOfBoundsException( 
				"numberOfConsumers must be greater than 0, your value was " + numberOfConsumers);
		populationSize = numberOfConsumers;
		theta = Math.log( percentConsumed)/ Math.log( 1 - percentConsumed );
		super.computePartialCumulativeDist( Math.min( 20,numberOfConsumers) );
		}
		
	/**
	 * Returns the probability of an item in the sample
	 * space will occur. sampleSpaceItem must be either
	 * 0 or 1.
	 * @param sampleSpaceItem an integer value in the interval
	 * [1, populationSize]. Is cast to int in density.
	 * @exception OutOfBoundsException thrown if sampleSpaceItem is not
	 * either 0 or 1.
	 */
	public double density( double indexOfConsumer ) throws OutOfBoundsException
		{
		int index = (int) indexOfConsumer;
		if ( ( index <= 0 ) || ( index > populationSize ) )
			throw new OutOfBoundsException( 
				"Agrument to density function of GeneralizedEightyTwenty must be value in interval [1, " +
				populationSize + "], your value was " + indexOfConsumer);
		
		return ( Math.pow( index, theta ) - Math.pow( index - 1, theta))/ Math.pow( populationSize, theta);
		}
		
	/**
	 * Returns the aproximate mean of the probability distribution. 
	 * The value is off by O(1/N), N = numberOfConsumers.
	 */	
	public double mean()
		{
		return theta * populationSize / ( theta + 1 )   + .5;
		}

	/**
	 * Returns the variance of the probability distribution.
	 * The value is off by O(N^(1-theta), theta = ln(percentConsumed)/
	 * ln( 1 - percentConsumed), and  N = numberOfConsumers.
	 */	
	public double variance()
		{
		return  theta * populationSize * populationSize / ( (theta + 1 ) * (theta + 1 ) * ( theta + 2 ) );
		}

	}
