/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.algorithms.data;

/**
 * The Geometric distribution deals with Bernoulli like situations: the
 * sample space is divided into success events and failure events. The
 * geometric distributions answers questions about how many trials are 
 * required to get a successful event. The density function maps an
 * integer N to the probability that it requires N trials to get the
 * first success. The nextElement() returns the number of trials needed
 * to get a success in a sequence of random trials.
 * This class is adapted from <I>Smalltalk-80: The language
 * and Its Implementation</I>, Goldberg & Robson, Addison-Wesley, 1983, chapter 21, 
 * pages 417-438.
 * @version 1.0 10 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class Geometric extends Bernoulli
	{
		
	/**
	 * Returns a Geometric.
	 * @param successProbability the probability of a successful event.
	 * @exception OutOfBoundsException thrown if successProbability
	 * is not in the interval [0, 1].
	 */
	public Geometric( double successProbability ) throws OutOfBoundsException
		{
		super( successProbability );
		}
		
	/**
	 * Returns the probability that it takes numberOfTrials
	 * to get the first successful event.
	 * @param numberOfTrials an integer number of trials. Will be converted to 
	 * an int. Must be greater than 0.
	 */
	public double density( double numberOfTrials ) throws OutOfBoundsException
		{
		int trials = (int) numberOfTrials;
		try
			{
			if ( trials > 0 )
				return successProbability * 
						Math.pow( 1.0 - successProbability, trials - 1);
			}
		// The ArithmeticException is thrown when 1.0 - successProbability <= 0
		// and trials - 1 is not an integer. 
		catch ( ArithmeticException canNotHappen )
			{
			}
		return 0.0;

		}
		
	/**
	 * Returns the mean of the distribution
	 */	
	public double mean()
		{
		return 1/ successProbability;
		}

	/**
	 * Returns the variance of the distribution
	 */	
	public double variance()
		{
		return  ( 1.0 - successProbability)/ ( successProbability * successProbability);
		}

	/**
	 * Returns the number of trials has randomProb probability of 
	 *  first success.
	 */
	protected double inverseDistribution( double randomProb )
		{
		// See Art of Computer Programming, Knuth, Vol 2. pp 131
		
		return Math.ceil( Math.log( randomProb)/ Math.log( 1.0 - successProbability ) );
		}
	}
