/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.algorithms.data;

/**
 * An abstract class handling discrete probability distributions with no inverse 
 * or easly computable inverse. The inverse is determined by brute force. Let the sample
 * space contain items s<sub>1</sub>, s<sub>2</sub>, ..., s<sub>n</sub>. Let PD( x) 
 * denote the density function of x. Map the interval [0, 1] to the sample space 
 * by: <br>
 * [0, PD(s<sub>1</sub>)] <-> s<sub>1</sub>, <br>
 * (PD(s<sub>1</sub>), PD(s<sub>1</sub>) + PD(s<sub>2</sub>) ] <-> s<sub>2</sub>, <br>
 * (PD(s<sub>1</sub>) + PD(s<sub>2</sub>), PD(s<sub>1</sub>) + PD(s<sub>2</sub>)  + PD(s<sub>3</sub>)] <-> s<sub>3</sub>, <br>
 * etc. In this way the cumulative distribution of s<sub>1</sub>, s<sub>2</sub>, ..., s<sub>n</sub>
 * gives the inverse. This class computes cumulative distribution.
 * @version 1.0 12 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public abstract class  NoInverseDistribution extends DiscreteProbability
	{

	/**
	 * Number of items in the sample space
	 */
	int populationSize = 1;
	
	/**
	 * Contains inital part of cumulative distribution for this density
	 * function
	 */
	double[] partialCumulativeDist;

	/**
	 * Return the next element in a random selection of 
	 * items in the sample space, using distribution of concrete
	 * subclass. The next element is an integer value from 1 to 
	 * size of the sample space.
	 */
	public double nextElement()
		{
		return inverseDistribution(  randomNumber.nextDouble() );
		}
	
	/**
	 * Maps the interval [0, 1] to the sample space, so that probability of an
	 * item in sampe space occuring is given by the distribution of the 
	 * concrete subclass. Concrete sublcass must insure computePartialCumulativeDist
	 * is called before nextElement or inverseDistribution is called.
	 */
	protected double inverseDistribution( double random )
		{
		// Since no closed form for the inverse is available, compute inverse
		// by brute force: compute all the cumulative distribution for  
		// sample space with item 1 maped to interval [0, x), where
		// x is density function of item 1.
		
		//To save time the cumulative dist. of first few items has been 
		// precomputed.
		
		double cumulativeDist = partialCumulativeDist[partialCumulativeDist.length-1];

		if ( 	cumulativeDist < random )
			{
			// Brute force 
			for ( int k = partialCumulativeDist.length; k <= populationSize; k++ )
				{
				cumulativeDist += density( k );
				if ( random <= cumulativeDist ) return k;
				}
			return populationSize; 
			}
		else
			// Search table
			for ( int k = 1; k < partialCumulativeDist.length; k++ )
				if ( random <= partialCumulativeDist[k] ) return k;
			
		return populationSize; 
		}
		
	/**
	 * Precompute the cumulative distributions for the 
	 * high propb. elements in the space.  The will
	 * reduce the time to compute the nextElement.
	 * @param cacheLength number of items to compute
	 * the cumulative distributions for
	 */
	protected void computePartialCumulativeDist( int cacheLength )
		{
		double cumulativeDist = 0.0;
		
		partialCumulativeDist = new double[cacheLength];
		for ( int k = 1; k < cacheLength; k++ )
			{
			cumulativeDist += density( k );
			partialCumulativeDist[k] = cumulativeDist;
			}
		
		}
	}