/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.algorithms.data;

/**
 * This class generates a sequence of numbers. The sequence has a starting number
 * and an increment. The sequence is generated by adding the increment to the last
 * number. If increment is positive get an increasing sequence, if negative get a
 * decreasing sequence. Returns inf (or -inf) in unlikey case when reach the end
 * of the double value ( about 1.79769e+308 on the upper range).
 *
 * @see		sdsu.data.NumberGenerator
 * @version 1.0 18 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class  OrderedNumberGenerator implements NumberGenerator
	{
	double startElement;
	double nextElement;
	double increment;
	
	/**
	 * Creates a sequence of numbers starting at 0 and 
	 * increment 1.
	 */
	public OrderedNumberGenerator()
		{
		this( 0 );
		}
	
	/**
	 * Creates a sequence of numbers starting at startNumber
	 * and increment 1.
	 */
	public OrderedNumberGenerator( double startNumber )
		{
		this( startNumber,  1);
		}

	/**
	 * Creates an sequence of numbers starting at startNumber
	 * and increment increment.
	 * @exception  IllegalArgumentException if increment == 0 (zero)
	 */
	public OrderedNumberGenerator( double startNumber, double increment )
		throws IllegalArgumentException
		{
		if ( increment == 0.0 )
			throw new IllegalArgumentException( "increment in the constructor " +
				"OrderedNumberGenerator can not have zero value" );
			
		startElement = startNumber;
		nextElement = startElement;
		this.increment = increment;
		}
	
	/**
	 * Returns the next number in the ordered sequence.
	 * If reach the end of the range of double, will return inf.
	 * As the max value of a float is about 1.79769e+308 any reasonable
	 * starting value and increment will not reach that value.
	 */
	public final double nextElement()
		{
		double elementToReturn = nextElement;
		nextElement = nextElement + increment;
		
		return elementToReturn;
		}
	}