/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.algorithms.data;

/**
 * The Poisson distrubution addresses the question of how many events
 * occur in a unit of time or space. 
 * This class is adapted from <I>Smalltalk-80: The language
 * and Its Implementation, Goldberg & Robson, Addison-Wesley, 1983, chapter 21, 
 * pages 417-438.
 * @version 1.0 10 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class Poisson extends DiscreteProbability
	{
	
	protected double mu;
	
	/**
	 * Returns a Poisson distribution object.
	 * @param averageEvents the averate number of events happening per unit (time, space).
	 * @exception OutOfBoundsException thrown if averageEvents is not greater than 0.0.
	 */
	public Poisson( double averageEvents ) throws OutOfBoundsException
		{
		if ( averageEvents <= 0.0 )
			throw new OutOfBoundsException( 
				"averageEvents must be greater than 0.0, your value was " + averageEvents);
		this.mu = averageEvents;
		}
		
	/**
	 * Returns the probability that numberOfEvents will occur
	 * in one unit (time, space).
	 * @param numberOfEvents does not make sense unless is integer value.
	 */
	public double density( double numberOfEvents )
		{
		int events = (int) numberOfEvents;
		if ( numberOfEvents <= 0 )
			return 0;
		
		double probability = 0.0;
		try
			{
			probability = (Math.pow( mu, events ) * Math.exp( - mu )) / factorial( events );
			}
		// The ArithmeticException is thrown when mu <= 0
		// and events is not an integer. 
		catch ( ArithmeticException canNotHappen )
			{
			}

		return probability;	
		}
		
	/**
	 * Returns the mean of the distribution
	 */	
	public double mean()
		{
		return mu;
		}

	/**
	 * Returns the variance of the probability distribution
	 */	
	public double variance()
		{
		return mu;
		}

	/**
	 * Returns the number events that occur in simualtion of 
	 * one unit (of time or space).
	 */
	public double nextElement()
		{
		double triger = Math.exp( - mu );
		int numberOfEvents = 0;
		double eventValue = 1.0;
		while ( eventValue >= triger )
			{
			eventValue = eventValue * randomNumber.nextDouble();
			numberOfEvents++;
			}
		return numberOfEvents;
		}
	
	/**
	 * Returns the facatorial of n
	 */
	private long factorial( long n )
		{
		long factorial = 1;
		for ( long k = 2; k <= n; k++ )
			factorial *= k;
		return factorial;
		}

	}
