/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.algorithms.data;

import java.util.Random;
import java.util.Vector;

/**
 * An abstract class defining an interface for probability distributions.
 * A <b>sample space</b> is the set of events or items of interest. A
 * <b> probability function</b> on a sample space is map from the sample
 * space to the interval [0, 1], such that sum of the probabilities of all items
 * in the space is one. A <b> random variable</b> is a real-valued function defined
 * over the events in a sample space. The probability function of a random varabiel is 
 * a <b>probability function</b>. The <b>density</b> is a function that assigns 
 * probabilities to allowed ranges of the random variable.
 * This class and many of its subclasses are adapted from <I>Smalltalk-80: The language
 * and Its Implementation</I>, Goldberg & Robson, Addison-Wesley, 1983, chapter 21, 
 * pages 417-438.
 * @version 1.0 10 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public abstract class ProbabilityDistribution implements NumberGenerator
	{
	/**
	 * Random number generator used in selecting an item
	 * from the sample space.
	 */
	protected Random randomNumber = new Random();
	
	/**
	 * Returns the probability of an item in the sample
	 * space will occur.
	 * @param sampleSpaceItem item in the sample space for which
	 * the probability of occuring is computed.
	 * @exception OutOfBoundsException if sampleSpaceItem is not in the
	 * sample space of the distribution.
	 */
	public abstract double density( double sampleSpaceItem ) throws OutOfBoundsException;
	
	public double density( Number sampleSpaceItem )
		{
		return density( sampleSpaceItem.doubleValue() );
		}
	
	/**
	 * Returns the mean of the probability distribution
	 */	
	public abstract double mean();

	/**
	 * Returns the variance of the probability distribution
	 */	
	public abstract double variance();
	
	/**
	 * Computes the density of each item in the vector.
	 * Returns a vector of the densities.
	 */
	public abstract Vector distribution( Vector sampleSpaceCollection );

	/**
	 * Returns a ramdon element in the sample space. The probability of
	 * a given item being returned is determined by the probability
	 * distribution on the sample space.
	 */
	public abstract double nextElement();
	}