/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.algorithms.data;

/**
 * The Zipfian distribution models the frequency of the word usage in written 
 * English and the population of U.S. cities (amoung other things). When ranked 
 * by frequency, the frequency
 * of the i'th is proportional to the frquency of the first item divided by i.  
 * Reference: <I>Handbook of Algorithms and Data Structures</I>, G. H. Gonnet, 
 * Addison-Wesley, 1984, pp 203. 
 * @version 1.0 11 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class Zipf extends NoInverseDistribution
	{
		
	private double harmonic;
	
	/**
	 * Returns a Zipf distribution.
	 * @param populationSize number of elements in the sample space.
	 * @exception OutOfBoundsException thrown if populationSize is 
	 * not greater than zero.
	 */
	public Zipf( int populationSize ) 
		{
		if ( populationSize <= 0 )
			throw new OutOfBoundsException( 
				"populationSize must be greater than 0, your value was " + populationSize);
		this.populationSize = populationSize;
		harmonic = computeHarmonic( populationSize );
		computePartialCumulativeDist( Math.min( 25, populationSize ) );
		}
		
	/**
	 * Returns the probability of an item in the sample
	 * space will occur. sampleSpaceItem must be either
	 * 0 or 1.
	 * @param sampleSpaceItem an integer value in the interval
	 * [1, populationSize]. Is cast to int in density.
	 * @exception OutOfBoundsException thrown if sampleSpaceItem is not
	 * either 0 or 1.
	 */
	public double density( double sampleSpaceItem ) throws OutOfBoundsException
		{
		int listIndex = (int) sampleSpaceItem;
		if ( ( listIndex <= 0 ) || ( listIndex > populationSize ) )
			throw new OutOfBoundsException( 
				"Agrument to density function of Zipf must be value in interval [1, " +
				populationSize + "], your value was " + sampleSpaceItem);
		
		return 1.0 / ( listIndex * harmonic);
		}
		
	/**
	 * Returns the mean of the Bernoulli distribution
	 */	
	public double mean()
		{
		return populationSize/ harmonic;
		}

	/**
	 * Returns the variance of the probability distribution
	 */	
	public double variance()
		{
		return (  ( (populationSize + 1.0)/2) - (populationSize/harmonic) ) * ( populationSize / harmonic);
		}

	
	double computeHarmonic( int n )
		{
		double sum = 0.0;
		for ( int k = 1 ; k <= n; k++ )
			sum = sum + (1.0/ (double) k );
		return sum;
			
		}
	}
