/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.compare;

import java.util.Hashtable;

/**
 * A concrete class for comparing Letter grades
 * The following shows the order of the grades:
 * <pre>
 *		A, A-, B+, B, B-, C+, C, P, CR, C-, D+, D, D-, F, NC, W

 * </pre>
 * That is A > B, etc. 
 * All other input is considered larger than W.
 *
 * @version 1.0 9 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class LetterGradeComparer extends Comparer
	{
	private static final String[] GRADES =
		{ "a", "a-", "b+", "b", "b-", "c+", "c", "p", "cr", "c-",  
			"d+", "d", "d-", "f", "nc", "w" };
	
	private static Hashtable GRADE_ORDER = new Hashtable();
	private static int MAX_LEVEL = GRADES.length + 1;
	
	static
		{
		for ( int k = 0; k < GRADES.length; k++ )
			GRADE_ORDER.put( GRADES[k], new Integer( k) );
		}
			
	private static Comparer singleInstance;
	
	/**
	 * Force use of instance method to get object instance.
	 */  
	private LetterGradeComparer() {};	
	
	/**
	 * Returns a LetterGradeComparer object.
	 */
	public static Comparer getInstance()
		{
		if ( singleInstance == null)
			singleInstance = new LetterGradeComparer();
		
		return singleInstance;
		}
		
	
	/**
	 * Returns true if the leftOperand is less than the rightOperand.
	 * @exception ClassCastException If operand objects are not strings.
	 *  ClassCastException is a RuntimeException, so compiler does not 
	 *  force you to catch this exception.
	 */
	public final boolean lessThan( Object leftOperand, Object rightOperand ) 
		throws ClassCastException
		{		
		int left =  toGradeLevel( leftOperand );
		int right =  toGradeLevel( rightOperand );

		return left < right;
		}
	
	/**
	 * Returns true if the leftOperand is greater than the rightOperand.
	 * @exception ClassCastException If operand objects are not strings objects.
	 *  ClassCastException is a RuntimeException, so compiler does not force you
	 *  to catch this exception.
	 */
	public final boolean greaterThan( Object leftOperand, Object rightOperand ) 
		throws ClassCastException
		{
		int left =  toGradeLevel( leftOperand );
		int right =  toGradeLevel( rightOperand );;

		return left > right;
		}


	/**
	 * Returns true if the leftOperand is equal to the rightOperand.
	 * @exception ClassCastException If operand objects are not Number objects.
	 *  ClassCastException is a RuntimeException, so compiler does not force you
	 *  to catch this exception.
	 */
	public final boolean equals( Object leftOperand, Object rightOperand ) 
		throws ClassCastException
		{
		int left =  toGradeLevel( leftOperand );
		int right =  toGradeLevel( rightOperand );;

		return left == right;
		}

	private static final int toGradeLevel( Object grade )
		{
		String letterGrade =  ((String) grade).toLowerCase();
		if ( !GRADE_ORDER.containsKey( letterGrade ) )
			return MAX_LEVEL;
		else
			{
			Integer level = (Integer) GRADE_ORDER.get( letterGrade );
		
			return level.intValue();
			}
		}
		

	}