/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.compare;

/**
 * A concrete class for comparing strings and strings that are in number format. 
 * Assumes that the string representation does not contain leading zeros.
 * Longer strings are larger than shorter strings. If a string contains a
 * decimal point then the number of characters (or digits) to the left of the 
 * decimal point is used as the length of the string. If two strings are of 
 * equal length than normal strign compare is used.
 * The following comparisons are all true using this comparer
 * <pre>
 *		lessThan( "2", "3" )
		lessThan( "22", "3" )
		lessThan( "234", "236" )
		lessThan( "123a", "123b" )
		lessThan( "12.3", "9.8" )
		lessThan( "a", "b" )
		lessThan( "XX12", "aaaab" )
		lessThan( "22.457", "22.6" )
		lessThan( "22.4", "22.6333" )

 * </pre>
 * @version 1.0 5 December 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
 
public class NumericStringComparer extends Comparer
	{

	private static Comparer singleInstance;
	private static String decimalChar = ".";
	
	/**
	 * Force use of instance method to get object instance.
	 */  
	private NumericStringComparer() {};	
	
	/**
	 * Returns a NumericStringComparer object.
	 */
	public static Comparer getInstance()
		{
		if ( singleInstance == null)
			singleInstance = new NumericStringComparer();
		
		return singleInstance;
		}
		
	
	/**
	 * Returns true if the leftOperand is less than the rightOperand.
	 * @exception ClassCastException If operand objects are not strings.
	 *  ClassCastException is a RuntimeException, so compiler does not force you
	 *  to catch this exception.
	 */
	public final boolean lessThan( Object leftOperand, Object rightOperand ) 
		throws ClassCastException
		{		
		String leftStringOperand =  (String) leftOperand;
		String rightStringOperand =  (String) rightOperand;		

		int leftLength = numericStringLength(leftStringOperand);
		int rightLength = numericStringLength(rightStringOperand);
		
					
		if ( leftLength < rightLength )
			return true;
		else if ( leftLength > rightLength )
			return false;
		else if (leftStringOperand.compareTo( rightStringOperand) < 0)
			return true;
		else
			return false;
		}
	
	private final int numericStringLength( String numberOrString )
		{
		int decimalLocation = numberOrString.indexOf( decimalChar );
		if ( decimalLocation > -1 )
			return decimalLocation;
		else
			return numberOrString.length();
		}
		
	/**
	 * Returns true if the leftOperand is greater than the rightOperand.
	 * @exception ClassCastException If operand objects are not strings objects.
	 *  ClassCastException is a RuntimeException, so compiler does not force you
	 *  to catch this exception.
	 */
	public final boolean greaterThan( Object leftOperand, Object rightOperand ) 
		throws ClassCastException
		{
		String leftStringOperand =  (String) leftOperand;
		String rightStringOperand =  (String) rightOperand;		

		int leftLength = numericStringLength(leftStringOperand);
		int rightLength = numericStringLength(rightStringOperand);
		
					
		if ( leftLength > rightLength )
			return true;
		else if ( leftLength < rightLength )
			return false;
		else if (leftStringOperand.compareTo( rightStringOperand) > 0)
			return true;
		else
			return false;
		}


	/**
	 * Returns true if the leftOperand is equal to the rightOperand.
	 * @exception ClassCastException If operand objects are not Number objects.
	 *  ClassCastException is a RuntimeException, so compiler does not force you
	 *  to catch this exception.
	 */
	public final boolean equals( Object leftOperand, Object rightOperand ) 
		throws ClassCastException
		{
		String leftStringOperand =  (String) leftOperand;
		String rightStringOperand =  (String) rightOperand;		

		if ( leftStringOperand.equals( rightStringOperand) )
			return true;
		else
			return false;
		}

	}