/* 
 * Copyright (C) 1998 Peter Proud-Madruga and
 *  Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.doc;

import java.util.*;
import java.io.*;
import sdsu.io.*;
import sdsu.util.ProgramProperties;

/**
 * This class fixes a problem with javadoc. Javadoc assumes that
 * all images are in a subdirectory of your current directory.
 * If is not true your javadoc pages will contain broken images.
 * This class changes the standard references of the images to
 * a valid reference. The way to use this program on a unix
 * machine is with the command line:
 * <pre>
 *		java sdsu.doc.FixJavadoc
 * </pre>
 * This will change the images references in all files in the
 * current directory that end in ".html". The "-h=directory"
 * (or "-h directory") 
 * flag will cause the program to process files in the given
 * directory instead of the current directoy. The 
 * "-i=imagesReference" (or "-i imagesReference"
 * will use the "imageReferece" as the 
 * location for the standard javadoc images. If the "-i" option
 * is not used, a http reference to a machine at SDSU is used.
 * If the flag "-s" is used, then the program is run in the 
 * silent mode. The default is the verbose mode, which prints 
 * the name of each file processed to standard out. For example:
 * <pre>
 *		java sdsu.doc.FixJavadoc -h=whitney/program1 -s
 * </pre>
 * will processes all ".html" files in the 
 * directory "whitney/program1" in the silent mode.
 *
 * @author Peter Proud-Madruga
 * @author Roger Whitney
 * @version 1.0 15 January 1998
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class FixJavadoc
	{
	static final String imageReference = "src=\"images";
	static Random randomNumber = new Random();
	
	File htmlDirectory;
	String imagesURL;
	PrintWriter progressOut;
	
	public static void main( String args[] ) throws IOException
		{
		ProgramProperties options = new ProgramProperties( args);
		
		String htmlFiles = options.getString( "h", "." );
		String imagesLocation = 
				options.getString( "i",
										"http://www.eli.sdsu.edu/doc/images");
		
		FixJavadoc fixer;
		
		if ( options.containsKey( "s" ))
			fixer = new FixJavadoc( imagesLocation, 
											new File( htmlFiles) );
		else
			fixer = new FixJavadoc( imagesLocation, 
											new File( htmlFiles),
											new PrintWriter( System.out ) );

		fixer.processFiles();
		}
	
	/**
	 * Create a new FixJavadoc object. Use imagesURL as the location
	 * for standard Javadoc images. Process all files in the 
	 * current directory. Run in silent mode.
	 */
	protected FixJavadoc( String imagesURL )
		{
		this( imagesURL, new File( "." ), null );
		}

	/**
	 * Create a new FixJavadoc object. Use imagesURL as the location
	 * for standard Javadoc images. Process all files in the 
	 *  directory htmlLocation. Run in silent mode.
	 */
	protected FixJavadoc( String imagesURL, File htmlLocation )
		{
		this( imagesURL, htmlLocation, null );
		}
	
	/**
	 * Create a new FixJavadoc object. Use imagesURL as the location
	 * for standard Javadoc images. Process all files in the 
	 *  directory htmlLocation. List files processed on the 
	 * given Writer status.
	 */
	protected FixJavadoc( String imagesURL, 
								File htmlLocation, 
								PrintWriter status )
		{
		htmlDirectory =  htmlLocation;
		this.imagesURL = "src=\"" + imagesURL.trim();
		progressOut = status;
		}
	
	/**
	 * Process the html files.
	 */
	protected void processFiles()
		{
		String[] listing = 
			htmlDirectory.list( new FileExtensionFilter("html"));
		
		for ( int k = 0; k < listing.length; k++ )
			{
			if ( progressOut != null )
				{
				progressOut.println( "Processing file: " + listing[k]);
				progressOut.flush();
				}
			correctImageReference( listing[k] );
			}
		}
		
	/**
	 * Replaces all occurances of the string imageReference with 
	 * the string imageURL in the file "fileName"
	 */
	protected void correctImageReference( String fileName)
		{
		File originalFile = new File( htmlDirectory, fileName );
		File scratchFile =  scratchFile();
		PrintWriter cout = null;
		StringReplaceReader fixed = null;
		try
			{
			BufferedReader bufferedFile = 
				new BufferedReader( new FileReader( originalFile ) );
			
			//	StringReplaceReader will replace all occurrances
			// of imageReference with imageURL
			fixed = new StringReplaceReader( 
						bufferedFile, imageReference, imagesURL);
			
			cout = new PrintWriter
				( new BufferedWriter( new FileWriter( scratchFile )));
			
			// fixed.contents returns file contents after making the 
			// changes
			cout.print( fixed.contents() );
			
			// can't delete or rename file that has an open stream 
			cout.close();
			fixed.close();
			
			originalFile.delete();
			scratchFile.renameTo( originalFile);
			}
			
		// only files that exist are opened for reading
		catch (FileNotFoundException shouldNotHappen)
			{
			System.err.println( "FileNotFoundException on existing file");
			shouldNotHappen.printStackTrace();
			}
			
		catch (IOException ioError )
			{
			System.err.println( "IO error while processing file: " + 
					fileName);
			ioError.printStackTrace();
			}
		finally
			{
			if (scratchFile.exists() )
				{
				cout.close();
				scratchFile.delete();
				}
			}
		}
	
	/**
	 * Return a file which has a name
	 * that does not exist in the current directory
	 */
	protected File scratchFile()
		{
		String newFileName;
		File newFile;
		do
			{
			newFileName = "" + randomNumber.nextInt();
			newFile = new File( htmlDirectory, newFileName);
			}
		while ( newFile.exists() );
		return newFile;
		}
	}