/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.html;

/**
 * Creates input buttons for use in an HTML form. Documentation for button types comes 
 * in part from
 * Web pages by Arnoud "Galactus" Engelfriet galactus@html.com. See 
 * <A HREF="http://www.htmlhelp.com/"> http://www.htmlhelp.com/</A>
 * Use static methods Button.checkBox(...), Button.radio(...), Button.submit(...)
 * Button.reset(...), Button.hidden(...) to create instances of this class. 
 * @version 0.8 8 April 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class Button implements HTML
	{
	/**
	 * String for indicated a checkbox in button constructor.
	 */
	private static final String CHECKBOX_TYPE = "checkbox";

	/**
	 * String for indicated a radio button in button constructor.
	 */
	private static final String RADIO_TYPE = "radio";

	/**
	 * String for indicated a submit button in button constructor.
	 */
	private static final String SUBMIT_TYPE = "submit";

	/**
	 * String for indicated a reset button in button constructor.
	 */
	private static final String RESET_TYPE = "reset";

	/**
	 * String for indicated a file button in button constructor.
	 */
	private static final String FILE_TYPE = "file";

	/**
	 * String for indicated a hidden button in button constructor.
	 */
	private static final String HIDDEN_TYPE = "hidden";
	
	private String name = null;
	private String value = null;
	private String type;
	private boolean checked = false;
	
	/**
	 * Create a button.
	 * @param buttonType type of button to create
	 * @param name name of button created
	 * @param value value submitted with form if button is checked
	 */
	private Button( String buttonType, String name, String value )
		{
		this.name = name;
		this.value = value;
		this.type = buttonType;
		}
	
	/**
	 * Returns a checkbox. Here is an example:<br>
	 *<FORM ACTION="http://www.nowwhere.edu/crap" METHOD="POST">
	 *<INPUT TYPE="checkbox" NAME="top" VALUE="on">A Checkbox<br>
	 * @param name name used to identify button when form is sent,
	 * plain text only.
	 * @param value value submitted with form if button is checked,
	 * plain text only.
	 */
	public static Button checkBox( String name, String value )
		{
		return new Button(  CHECKBOX_TYPE, name, value );
		}

	/**
	 * Produces a radio button. A radio button always exists in a group. 
	 * All members of this group should have the
    * same NAME attribute, and different VALUEs. The VALUE of the selected
    * radio button will be sent to the
    *  server. You can specify CHECKED on one radio button,
    * which then will come up selected initially. 
    * An example: <br>
	 * <FORM ACTION="http://www.nowwhere.edu/crap" METHOD="POST">
	 * <INPUT TYPE="radio" NAME="top" VALUE="on">A Radio button<br>
	 * @param name name of button created, plain text only.
	 * @param value value submitted with form if button is selected,
	 * plain text only.
	 */
	public static Button radio( String name, String value)
		{
		return new Button(  RADIO_TYPE, name, value );
		}

	/**
	 * Produces a button, which when pressed sends the contents of 
	 * the form to the server. You can have more than
    *  one submit button in the form. Each should have a different NAME. 
    * The name and value of the pressed button
    *   will be sent to the server as well. The value of the VALUE attribute is 
    * typically used as text on the submit button. 
    * An example: <br>
	 * <FORM ACTION="http://www.nowwhere.edu/crap" METHOD="POST">
	 * <INPUT TYPE="submit" VALUE="Don't Submit">A Submit button<br>
	 * @param value name displayed in the button. Plain text only.
	 */
	public static Button submit( String value )
		{
		return new Button(  SUBMIT_TYPE, null, value );
		}

	/**
	 * Produces a button, which will restore the form to
	 * its original state if pressed. The value of the VALUE
    *   attribute is typically used as text on the reset button. 	 
    * An example: <br>
	 * <FORM ACTION="http://www.nowwhere.edu/crap" METHOD="POST">
	 * <INPUT TYPE="checkbox" NAME="top" VALUE="on">Select Me<br>
	 * <INPUT TYPE="reset" VALUE="Clear">A reset button<br>
	 * @param value Text shown in button on the form, plain text only
	 */
	public static Button reset( String value )
		{
		return new Button(  RESET_TYPE, null, value );
		}

	/**
	 * Allows you to embed information in the form which you do 
	 * not want changed. This can be useful if the
    *  document is generated by a script and you need to store 
    * state information. NAME and VALUE of this input
    *   field will be sent to the server without modifications. 
    * @param name name of button created, plain text only
	 * @param value value submitted with form, plain text only
	 */
	public static Button hidden( String name, String value )
		{
		return new Button(  HIDDEN_TYPE, name, value );
		}

	/**
	 * Use on radio buttons and check boxes. This will make the button (box) appear
	 * selected when form is displayed. Only one radio button can be selected
	 */
	public void makeSelected()
		{
		checked = true;
		}
		
	/**
	 * Returns the html button tags in a string for use in a html page
	 */
	public String toString()
		{
		StringBuffer buttonTag = new StringBuffer( 12 );
		buttonTag.append( "\n<INPUT" );
		buttonTag.append( " TYPE=\"" );
		buttonTag.append( type );
		buttonTag.append( "\"" );
		if ( name != null )		//submit and reset buttons don't have names
			{
			buttonTag.append( " NAME=\"" );
			buttonTag.append( name );
			buttonTag.append( "\"" );
			}
		buttonTag.append( " VALUE=\"" );
		buttonTag.append( value );
		buttonTag.append( "\"" );
		if ( checked )
			buttonTag.append( " CHECKED " );
			
		buttonTag.append( ">" );

		return buttonTag.toString();
		}
	}