/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.html;

/**
 * HTML formats text, which contains special tags to indicate various
 * text features (bold, italic, etc). HTML has three special characters (<, >, &)
 * which must be encoded if they are to displayed in a browser. Use the class  
 * to format items in HTML. This class escapes the
 * special HTML characters. It is used by other class in sdsu.html to process strings
 * to html. If you wish to generate HTML tags directly use appendHTMLTags().
 * This will not escape the special characters. The resulting
 * Formatter object can be used correctly by other classes in sdsu.html.  
 * @version 0.9 6 April 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class Formatter implements HTML
	{
	static final String NEW_LINE = "\n<BR>";
	
	private String containerTypeBeginingTags = "";
	private String containerTypeEndingTags = "";
	private String alignmentBeginingTags = "";
	private String alignmentEndingTags = "";
	
	private StringBuffer contents = new StringBuffer();
	
	/**
	 * Create a new Formatter object on the string
	 * All special html characters in string <B>asciiText</B> are escaped
	 */
	public Formatter( String asciiText )
		{
		append( asciiText );
		}

	/**
	 * Create a new Formatter
	 */
	public Formatter()
		{
		}
	
	/**
	 * Centers the contents of object.
	 */	
	public void alignCenter()
		{
		setAlignment( "\n<CENTER>", "\n</CENTER>" );
		}

	/**
	 * Right justifies the contents of object.
	 */	
	public void alignRight()
		{
		setAlignment( "\n<DIV ALIGN=right>", "\n</DIV>");
		}

	/**
	 * Left justifies contents of object.
	 */	
	public void alignLeft()
		{
		setAlignment( "\n<DIV ALIGN=left>", "\n</DIV>");
		}

	/**
	 * Sets the current text to preformatted. 
	 * HTML treats tabs and newlines as a space. HTML ignores multiple spaces. 
	 * In preformated text tabs, newlines and multiple spaces are treated as 
	 * tabs, newlines and multiple spaces. 
	 * Here is an example of preformatted text: <br>
	 * <PRE>This   has    some extra spaces in it</PRE><br>
	 */	
	public void makePreformatted()
		{
		setContainerType( "\n<PRE>", "\n</PRE>");
		}

	/**
	 * Sets the current object to be a 
	 * HTML paragraph. 
	 */	
	public void makeParagraph()
		{
		setContainerType( "\n<P>", "\n</P>");
		}

	/**
	 * Sets the current object to be a 
	 * HTML block quote. 
	 */	
	public void makeBlockQuote()
		{
		setContainerType( "\n<BLOCKQUOTE>", "\n</BLOCKQUOTE>");
		}
	
	/**
	 * Converts the text to an string with proper html tags
	 */
	public String toString()
		{
		return alignmentBeginingTags + containerTypeBeginingTags +
				contents.toString() +
				containerTypeEndingTags + alignmentEndingTags;
		}	
	
	/**
	 * Append the string 'asciiText' to the current end of the text
	 */
	public void append( String asciiText)
		{
	 	// '<', '>', '&' are special html characters and must be escaped
	 	// Those are the only characters that are escaped in html
		contents.append( encodeText( asciiText ));
		}

	/**
	 * Append the an existing HTML object to the current end of the text.
	 * Object is converted to html tags immediately, so any changes made to the
	 * object later will not be reflected in web page.
	 */
	public void append( HTML htmlObject)
		{
		contents.append( htmlObject.toString() );
		}


	/**
	 * Append the string 'asciiText' to the current end of the text
	 * as bold text
	 */
	public void appendBold( String asciiText)
		{
		contents.append( "\n<B>");		
		contents.append( encodeText( asciiText ));
		contents.append( "\n</B>");		
		}

	/**
	 * Append the string 'asciiText' to the current end of the text
	 * as bold text
	 */
	public void appendItalic( String asciiText)
		{
		contents.append( "\n<I>");		
		contents.append( encodeText( asciiText ));
		contents.append( "\n</I>");		
		}

	/**
	 * Append the string heading to the current end of the text
	 * as a heading.
	 * Browser differ on how they render headings. Here are examples to 
	 * demonstrate how your current renders headings:<br>
	 * <H1>Heading 1</H1> <H2>Heading 2</H2> <H3>Heading 3</H3>
	 * <H4>Heading 4</H4> <H5>Heading 5</H5> <H6>Heading 6</H6> <br>
	 * @param heading string added as a heading.
	 * @param headingLevel heading level text is set to.
	 */
	public void appendHeading( String heading, int headingLevel)
		{
		String startTag = "";
		String endTag = "";
		switch ( headingLevel )
			{
			case 1: startTag = "\n<H1>"; endTag = "\n</H1>"; break;
			case 2: startTag = "\n<H2>"; endTag = "\n</H2>"; break;
			case 3: startTag = "\n<H3>"; endTag = "\n</H3>"; break;
			case 4: startTag = "\n<H4>"; endTag = "\n</H4>"; break;
			case 5: startTag = "\n<H5>"; endTag = "\n</H5>"; break;
			case 6: startTag = "\n<H6>"; endTag = "\n</H6>"; break;
			}
		contents.append( startTag );
		contents.append( encodeText( heading ));
		contents.append( endTag );
		}
		
	/**
	 * Appends the string withHTMLTags without escaping special
	 * characters. Use to add strings that contain HTML tags
	 */
	public void appendHTMLTags( String withHTMLTags)
		{
		contents.append( withHTMLTags);
		}

	/**
	 * Append the string 'asciiText' to the current end of the text
	 * as linked text.
	 * An example of a url string for a relative link is <BOLD>notes/index.html</BOLD>
	 * An example of a url string for a absolute link is <BOLD>
	 * http://www.eli.sdsu.edu/courses/index.html</BOLD>.
	 * @param asciiText the string appended
	 * @param url the url the string is a link to
	 */
	public void appendLink( String asciiText, String url)
		{
		contents.append( "<A HREF=\"");		
		contents.append( url );		
		contents.append( "\">" );		
		contents.append( encodeText( asciiText ));
		contents.append( "</A>");	
		}	

	/**
	 * Appends a mailTo reference.
	 * The provided mail address is linked and displayed. If 
	 * mailAddress is: whitney@cs.sdsu.edu, then the result will be: 
	 * <a href="mailto:whitney@cs.sdsu.edu">:whitney@cs.sdsu.edu</a>
	 * @param mailAddress string containing a well formed and valid
	 * mail address 
	 */
	public void appendMailLink( String mailAddress)
		{
		contents.append( "<A HREF=\"mailto:");		
		contents.append( mailAddress);
		contents.append( "\">");		
		contents.append( mailAddress);
		contents.append( "</A>");		
		}

	/**
	 * Appends one HTML line break (or carriage returns) 
	 * to the current end of the text.
	 */
	public void appendLineBreak( )
		{
		contents.append( NEW_LINE );
		}


	/**
	 * Converts characters ('<', '>', '&') with special html meaning to their
	 * html encoded form.
	 * @param text string that needs to be encoded
	 * @return text with all occurances of '<', '>', '&' in html encoded form
	 */
	static final String encodeText( String text)
		{
		StringBuffer encodedText = new StringBuffer( text.length() + 16 );
		
		char[] textChars = text.toCharArray();
		for ( int k = 0; k < textChars.length; k ++ )
			switch ( textChars[k] )
				{
				case '>': encodedText.append( "&gt;" ); 	break;
				case '<': encodedText.append( "&lt;" ); 	break;
				case '&': encodedText.append( "&amp;" ); 	break;
				default : encodedText.append( textChars[k] );
				}
		return encodedText.toString();
		}
		

	private void setContainerType( String beginTag, String endTag)
		{
		containerTypeBeginingTags = beginTag;
		containerTypeEndingTags = endTag;
		}

	private void setAlignment( String beginTag, String endTag)
		{
		alignmentBeginingTags = beginTag;
		alignmentEndingTags = endTag;
		}
		
	}