/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.html;
import sdsu.util.Table;


/**
 * Implements an HTML table. This allows displaying of items in rows and columns.
 * Here is an example of an HTML table with: border size 2, a Caption, centered, 
 * width at 70%, the first row and column set as headers. <br>
 * <CENTER>
 * <TABLE  BORDER=2 WIDTH="70%" ALIGN=center>
 * <CAPTION>The Caption</CAPTION>
 * <TR><TH>&nbsp;</TH><TH>Row 0 (zero) is declared </TH><TH> to be </TH>
 * <TH>a header</TH></TR>
 * <TR><TH>Column 0 is a </TH><TD>Cell 1,1</TD><TD>Cell 1, 2</TD>
 * <TD>Cell 1, 2</TD></TR>
 * <TR><TH> header</TH><TD>Cell 1,1</TD><TD>&nbsp;</TD><TD>&nbsp;</TD></TR>
 * </TABLE>
 * </CENTER><br><br>
 * @version 0.8 6 April 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class HTMLTable implements HTML
	{
	private String[][] tableElements;
	private String caption = null;
	private int borderWidth = 1; 			// in pixels
	private String alignment = LEFT;
	private String width = null;
	private int labelRow = -1;
	private int labelColumn = -1;
	
	private static final String LEFT = "left";
	private static final String CENTER = "center";
	private static final String RIGHT = "right";
	 
	private static final String  emptyString = "&nbsp;";
	
	/**
	 * Create a new table with given number of rows and columns. All elements are 
	 * set to the empty string.
	 * @param numberOfRows number of rows in the table
	 * @param numberOfColumns number of colums in the table
	 */
	public HTMLTable( int  numberOfRows, int numberOfColumns )
		{
		tableElements = new String[numberOfRows][numberOfColumns];
		
		for ( int row = 0; row < numberOfRows; row++ )
			for ( int column = 0; column < numberOfColumns; column++)
				tableElements[row][column] = emptyString;
		}
		
	/**
	 * Create a new html table with element given by the sdsu.util.Table
	 * @param initialTableElements table containing the initial elements for
	 * the html table
	 */
	public HTMLTable( sdsu.util.Table initialTableElements )
		{
		int numberOfRows = initialTableElements.numberOfRows();
		int numberOfColumns = initialTableElements.numberOfColumns();
		tableElements = new String[numberOfRows][numberOfColumns];
		
		try
			{
			for ( int row = 0; row < numberOfRows; row++ )
				for ( int column = 0; column < numberOfColumns; column++)
					{
					Object element = initialTableElements.elementAt( row, column);
					if ( element == null )
						element = emptyString;
					tableElements[row][column] = element.toString();
					}
			}
		catch ( ArrayIndexOutOfBoundsException impossible )
			{ // New table is same size as old, so this should not occur
			}
		}
	
	/**
	 * Centers the table in the page. 
	 * Does not affect the alignment of the items
	 * in each table location.
	 */	
	public void alignCenter()
		{
		alignment = CENTER;
		}

	/**
	 * Right justifies the table. The right edge of the table will start on the right edge of
	 * the page.
	 * Does not affect the alignment of the items
	 * in each table location.
	 */	
	public void alignRight()
		{
		alignment = RIGHT;
		}

	/**
	 * Left justifies the table. The Left edge of the table will start on the left edge of
	 * the page. This is the default setting.
	 * Does not affect the alignment of the items
	 * in each table location.
	 */	
	public void alignLeft()
		{
		alignment = LEFT;
		}


	/**
	 * Sets the width of the border drawn around table and indivitual cells.
	 * default setting is border of one pixel. Pixel size of zero gives no border.
	 * @param pixelWidth width of border in pixels
	 */
	public void setBorderWidth( int pixelWidth )
		{
		borderWidth = pixelWidth;
		}

	/**
	 * Sets text caption displayed above the table.
	 * @param caption displayed above table
	 */
	public void setCaption( Formatter caption )
		{
		this.caption = caption.toString();
		}

	/**
	 * Makes the given <STRONG>row</STRONG> a header for the columns of table. 
	 * Netscape will center and make bold all items on the given row.
	 * @param row index of the row to be the header
	 */
	public void makeRowAHeader( int row )
		{
		labelRow = row;
		}

	/**
	 * Makes the given <STRONG>column</STRONG> a header for the rows of table. 
	 * Netscape will center and make bold all items on the given column.
	 * @param column index of the column to be the header
	 */
	public void makeColumnAHeader( int column )
		{
		labelColumn = column;
		}
		
		
	/**
	 * Places string representation of data in table at given row and column
	 * @exception java.lang.ArrayIndexOutOfBoundsException thrown if 
	 * row or column is not a valid index
	 * @param data HTML object whose string representation is added to the table
	 */
	public void setElementAt( HTML data, int row,  int column ) throws ArrayIndexOutOfBoundsException
		{
		tableElements[ row ][ column ] = data.toString();
		}

	/**
	 * Places string representation of data in table at given row and column
	 * @exception java.lang.ArrayIndexOutOfBoundsException thrown if 
	 * row or column is not a valid index
	 * @param data Item whose string representation is added to the table, 
	 * special characters are escaped for you
	 */
	public void setElementAt( Object data, int row,  int column ) throws ArrayIndexOutOfBoundsException
		{
		setElementAt( new Formatter( data.toString() ), row, column);
		}
	
	/**
 	 * Suggests a width in pixels of the table. Browser will try to use this width.
 	 * @param pixelWidth suggested width of the table in pixels.
 	 */
	public void setWidth(int pixelWidth )
		{
		width= String.valueOf( pixelWidth );
		}

	/**
 	 * Suggests a width for the table in percent of window width. 
 	 * Browser will try to use this width.
	 * @param percentOfWindowWidth int between 0 and 100
	 */
	public void setWidthPercent(int percentOfWindowWidth )
		{
		width= "\"" + String.valueOf( percentOfWindowWidth ) +"%\"";
		}


	/**
	 * Returns the string version of the object in table at given row and column. 
	 * @exception java.lang.ArrayIndexOutOfBoundsException thrown if 
	 * row or column is not a valid index
	 */
	public String  elementAt( int row,  int column ) throws ArrayIndexOutOfBoundsException 
		{
		return  tableElements[ row ][ column ];
		}

	public String toString()
		{
		int tableSize = tableElements.length * tableElements[0].length;
		StringBuffer tableHTML = new StringBuffer( 3*tableSize + 10 );
		addTableHeader( tableHTML );
		addTableCells( tableHTML );
		addTableEnding(  tableHTML );
		return  tableHTML.toString() ;
		}
	
	/**
	 * Add the table tag and all its attributes to the string buffer
	 */
	private void addTableHeader( StringBuffer tableTags )
		{
		// center alingmemt tag inside table is not supported by Netscape yet
		if ( alignment.equals( CENTER) )
			tableTags.append( "\n<CENTER>" );
		
		tableTags.append( "\n<TABLE " );
		
		if ( borderWidth > 0 )
			{
		 	tableTags.append( " BORDER=\"" );
		 	tableTags.append(  borderWidth );
		 	tableTags.append( "\"" );
		 	}
		 
		if ( !alignment.equals( LEFT) )
			{
		 	tableTags.append( " ALIGN=\"" );
		 	tableTags.append(  alignment );
		 	tableTags.append( "\"" );
		 	}

		if ( width != null )
			{
		 	tableTags.append( " WIDTH=\""  );
		 	tableTags.append( width );
		 	tableTags.append( "\"" );
		 	}

		tableTags.append( ">" );

		if ( caption != null )
		 	tableTags.append( "\n<CAPTION>" + caption + "</CAPTION>" );		
		}

	private  void addTableCells( StringBuffer tableTags )
		{
		String[][] tableElements = this.tableElements;	// local access is faster
		int numberOfRows = tableElements.length;
		int numberOfColumns = tableElements[0].length;
		String cellTag;
		
		for ( int row = 0; row < numberOfRows; row++ )
			{
			tableTags.append( "\n  <TR>" ); // space for allignment
			for ( int column = 0; column < numberOfColumns; column++)
				{
				cellTag = getCellTag( row, column);
				tableTags.append( "<" + cellTag + ">" ); 		// begin tag
				tableTags.append( tableElements[row][column] );
				tableTags.append( "</" + cellTag + ">" );		// end tag
				}
			tableTags.append( "</TR>" );
			}
		}

	private String getCellTag( int row, int column )
		{
		if (( row == labelRow) || ( column == labelColumn) )
			return "TH";
		else
			return "TD";
		}

	/**
	 * Add the tags to close the table to the string buffer
	 */
	private void addTableEnding( StringBuffer tableTags )
		{
		tableTags.append( "\n</TABLE>" );
		if ( alignment.equals( CENTER ) )
			tableTags.append( "\n</CENTER>" );

		}

	}
