/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.html;

/**
 * Represents an inline image that can be part of a web page. 
 * An image is constructed with a URL to the actual image.
 * Supported formats depend on the browser 
 * used to render the html. GIF 87a (.gif) is very common. 
 * @version 0.8 1 April 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class Image implements HTML
	{
	String alignment = null;
	String alternativeText = "";
	String mapURL = null;
	String imageURL = null;
	
	String borderSize = null;					// in pixels
	String height = null;						// in pixels or percent
	String width = null;							// in pixels
	String verticalGutter = null;				// in pixels
	String horizontalGutter = null;			// in pixels 
	
	String linkURL = null;
	
	boolean centerImage = false;
	boolean putTextOnNextLine = false;
	boolean isMap = false;
	

	/**
	 * Returns a image that the red SDSU logo. 
	 * Image is from SDSU Graphics Library.
	 * The image looks like:<br>
	 * <IMG SRC="http://www.sdsu.edu/graphics/sdsu_logo_3.gif"> 
	 */
	public static Image redSDSU()
		{
		Image logo = new Image( "http://www.sdsu.edu/graphics/sdsu_logo_3.gif" );
		return logo;		
		}

	/**
	 * Returns a image that the black SDSU logo. 
	 * Image is from SDSU Graphics Library.
	 * The image looks like:<br>
	 * <IMG SRC="http://www.sdsu.edu/graphics/sdsu_logo_9.gif"> 
	 */
	public static Image blackSDSU()
		{
		Image logo = new Image( "http://www.sdsu.edu/graphics/sdsu_logo_9.gif" );
		return logo;		
		}

	/**
	 * Returns a image that is a red divider bar that will span the entire window. 
	 * Image is from SDSU Graphics Library.
	 * The bar looks like:<br>
	 * <IMG SRC="http://www.sdsu.edu/graphics/divider_red.gif" WIDTH="100%" HEIGHT=6>
	 */
	public static Image redDividerBar()
		{
		return dividerBar( "red" );
		}

	/**
	 * Returns a image that is a divider bar of a given color. 
	 * Valid colors are: red, blue, green, teal, yellow, rainbow
	 * Images are from SDSU Graphics Library.
	 * The bar looks like:<br>
	 * <IMG SRC="http://www.sdsu.edu/graphics/divider_red.gif" WIDTH="100%" HEIGHT=6>
	 */
	public static Image dividerBar( String color )
		{
		Image bar = new Image( "http://www.sdsu.edu/graphics/divider_" + color.trim() + ".gif",
			"----------------------------------------------------------------------" );
		bar.setWidthPercent( 100 );
		bar.setHeight( 6 );
		return bar;
		}
		
	/**
	 * Create an html reference to an image to be displayed in a web page.
	 * URL gives the relative or absolute location for the actual image.
	 * An example of a url string for a relative relative is
	 *  <BOLD>notes/happy.gif</BOLD>
	 * An example of a url string for a absolute location is <BOLD>
	 * http://www.sdsu.edu/graphics/divider_red.gif</BOLD>. That is the url string should 
	 * <BOLD> not </BOLD> start with '&lt;A HREF=', end with '&lt;/A&gt;', nor quote the url 
	 */	
	public Image( String URL )
		{
		this( URL, "");
		}
		

	/**
	 * Create an html reference to an image to be displayed in a web page with alternative text.
	 * URL gives the relative or absolute location for the actual image.
	 * An example of a url string for a relative relative is
	 *  <BOLD>notes/happy.gif</BOLD>
	 * An example of a url string for a absolute location is <BOLD>
	 * http://www.sdsu.edu/graphics/divider_red.gif</BOLD>. That is the url string should 
	 * <BOLD> not </BOLD> start with '&lt;A HREF=', end with '&lt;/A&gt;', nor quote the url 
	 * @param	URL	location of the image
	 * @param	textVersion	text to be displayed in text-only browsers
	 */	
	public Image( String URL, String textVersion )
		{
		imageURL = "\"" + URL.trim() +"\"";
		alternativeText = "\"" + textVersion  +"\"";
		}
	
	/**
	 * Make the image line up against the left margin, and text will flow around it.
	 * Only one of alignLeft, alignCenter, alignRight, alignTextTop, alignTextMiddle,
	 * alignTextBottom can be used. If more than one are called, the last one called is used. 
	 */
	public void alignLeft()
		{
		alignment = "LEFT";
		}

	/**
	 * Center the image in the page. Text will <B>not</B> flow around it.
	 * Text will appear on the next line after image.
	 * Only one of alignLeft, alignCenter, alignRight, alignTextTop, alignTextMiddle,
	 * alignTextBottom can be used. If more than one are called, the last one called is used. 
	 */
	public void alignCenter()
		{
		centerImage = true; 
		// IMG tag does not support center, so fake it by using <center> ,</center> tags
		}

	/**
	 * Make the image line up against the right margin, and text will flow around it.
	 * Only one of alignLeft, alignCenter, alignRight, alignTextTop, alignTextMiddle,
	 * alignTextBottom can be used. If more than one are called, the last one called is used. 
	 */
	public void alignRight()
		{
		alignment="RIGHT";
		}

	/**
	 * Alight text following the image at top of the image.
	 * Only one of alignLeft, alignCenter, alignRight, alignTextTop, alignTextMiddle,
	 * alignTextBottom can be used. If more than one are called, the last one called is used. 
	 */
	public void alignTextTop()
		{
		alignment="TOP";
		}

	/**
	 * Alight text following the image at middle of the image.
	 * Only one of alignLeft, alignCenter, alignRight, alignTextTop, alignTextMiddle,
	 * alignTextBottom can be used. If more than one are called, the last one called is used. 
	 */
	public void alignTextMiddle()
		{
		alignment="MIDDLE";
		}

	/**
	 * Alight text following the image at bottom of the image.
	 * Only one of alignLeft, alignCenter, alignRight, alignTextTop, alignTextMiddle,
	 * alignTextBottom can be used. If more than one are called, the last one called is used. 
	 */
	public void alignTextBottom()
		{
		alignment="BOTTOM";
		}

	/**
	 * Place text following the image on the line below the image.
	 * Text is not aligned wiht image, just put on the next line.
	 * Can be used with other align methods.
	 */
	public void alignTextBelow()
		{
		putTextOnNextLine = true;
		}

	/**
	 * Width of the image in pixels. Width & Height allow browsers to lay out the page in advance
	 * of downloading the image
	 */
	public void setWidth(int pixelWidth )
		{
		width= String.valueOf( pixelWidth );
		}

	/**
	 * Width of the image in percent of the window width.  The image will be 
	 * stretched or compressed to the given size. Should only be used for decorative
	 * items like color divider bars.
	 * @param percentOfWindowWidth number between 0 and 100
	 */
	public void setWidthPercent(int percentOfWindowWidth )
		{
		width= "\"" + String.valueOf( percentOfWindowWidth ) +"%\"";
		}

	/**
	 * Width of the image in pixels. Width & Height allow browsers to lay out the page in advance
	 * of downloading the image
	 */
	public void setHeight(int pixelHeight )
		{
		height= String.valueOf( pixelHeight );
		}

	/**
	 * Size of the border drawn around an image that is a link
	 */
	public void setBorderSize(int pixelSize )
		{
		borderSize= String.valueOf( pixelSize );
		}

	/**
	 * Makes image a link to the given url
	 * An example of a url string for a relative link is <BOLD>notes/diagram.gif</BOLD>
	 * An example of a url string for a absolute link is <BOLD>
	 * http://www.eli.sdsu.edu/courses/brokenPipe.gif</BOLD>. That is the url string should 
	 * <BOLD> not </BOLD> start with '&lt;A HREF=', end with '&lt;/A&gt;', nor quote the url 
	 */	
	public void link( String url)
		{
		linkURL = url;
		
		// Remove ugly border around a link image if the client has not set a border
		if (  borderSize == null )
			borderSize = "0";
		}

		
	/**
	 * Returns the html string represented by this object.
	 */
	public String toString()
		{
		StringBuffer imageTag = new StringBuffer( 10 );
		
		//First build the IMG tag
		imageTag.append( "<IMG " );
		imageTag.append( "SRC=" + imageURL + " ");
		imageTag.append( "ALT=" + alternativeText + " ");
		
		if ( alignment != null )
			imageTag.append( "ALIGN=" + alignment + " ");
		if ( borderSize != null )
			imageTag.append( "BORDER=" + borderSize + " ");
		if ( height != null )
			imageTag.append( "HEIGHT=" + height + " ");
		if ( width != null )
			imageTag.append( "WIDTH=" + width + " ");
		if ( verticalGutter != null )
			imageTag.append( "VSPACE=" + verticalGutter + " ");
		if ( horizontalGutter != null )
			imageTag.append( "HSPACE=" + horizontalGutter + " ");
		imageTag.append( ">");
		
		// Now all the stuff that must be outside the tag
		if ( putTextOnNextLine ) 
			imageTag.append( "<BR CLEAR=ALL>");
		
		String imageTagString = imageTag.toString();
		
		if ( linkURL != null )
			{
			imageTag.insert( 0, "\n<A HREF=\"" + linkURL +"\">\n" );
			imageTag.append( "\n</A>");
			}
		
		if ( centerImage )
			{
			imageTag.insert( 0, "\n<CENTER>" );
			imageTag.append( "\n</CENTER>");
			}
			
		return imageTag.toString();
		}
	}
