/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.html;

import java.util.Enumeration;

/**
 * Create a dropdown menu or a scrollable list in an html form.
 * Examples of each: <br>
 * <FORM ACTION="http://www.eli.sdsu.edu/crap" METHOD="POST">
 * <SELECT NAME="Roger" SIZE=3>
 * <OPTION VALUE="a">Larry</OPTION>
 * <OPTION VALUE="b">Currly</OPTION> 
 * <OPTION VALUE="c">Moe</OPTION>
 * </SELECT>Scrollable list<P>
 * <SELECT NAME="Roger" SIZE=1>
 * <OPTION VALUE="a">Larry</OPTION>
 * <OPTION VALUE="b">Currly</OPTION> 
 * <OPTION VALUE="c">Moe</OPTION>
 * </SELECT>Dropdown menu<P>
 * Must be used in a form. Item user selects in reported as 
 * <B>ListName=valueOfSelectedItem</B>. Where ListName is the name given to the list,
 * and valueOfSelectedItem is the value given to the selected item, or 
 * the display string of item if no value was given.<P>
 *  Use static methods SelectionList.scrolling(...) and SelectionList.dropDownMenu(...)
 * to create instances of this class.
 * @version 0.8 6 April 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
 
public class SelectionList implements HTML
	{
	private String name;
	private String size;
	private boolean multipleSelection = false;
	private StringBuffer listItems = new StringBuffer();
	
	/**
	 * Returns a SelectionList for use in a form.
	 * @param listName name in name-value cgi pair sent to cgi program when an item
	 * is selected in this list
	 * @param size number of items, one per line, that can be seen by the user at one time,
	 */
	protected SelectionList( String listName, int size )
		{
		name = listName;
		this.size = String.valueOf( size);
		}
		
	/**
	 * Returns a scrolling list for use in a form.
	 * @param listName name in naem-value cgi pair sent to cgi program when an item
	 * is selected in this list
	 * @param size number of items, one per line, that can be seen by the user at one time,
	 * must be greater than one
	 */
	public static SelectionList scrolling( String listName, int size )
		{
		if (size < 2) size = 2;
		return new SelectionList(listName, size );
		}

	/**
	 * Returns a drop down menu for use in a form.
	 * @param listName name in naem-value cgi pair sent to cgi program when an item
	 * is selected in this list
	 */
	public static SelectionList dropDownMenu( String listName )
		{
		return new SelectionList(listName, 1 );
		}

	/**
	 * Append an item to the end of the list for display
	 * @param itemDisplay string that user will see in the list and the value
	 * return if this item is selected by user
	 */
	public void append( String itemDisplay )
		{
		append( itemDisplay, null, false);
		}


	/**
	 * Append an item to the end of the list for display
	 * @param itemDisplay string that user will see in the list
	 * @param value the value returned if this item is selected by user
	 */
	public void append( String itemDisplay, String value)
		{
		append( itemDisplay, value, false);
		}

	/**
	 * Append an item to the end of the list. This item will be displayed
	 * as selected when user first sees the form
	 * @param itemDisplay string that user will see in the list, special characters
	 * in itemDisplay are escaped for you
	 * @param value the value returned if this item is selected by user,
	 * if value is null or an empty string then itemDisplay is used as return value
	 * @param selected true indicates the item will appear as selected
	 */
	public void append( String itemDisplay, String value, boolean selected)
		{
		listItems.append("\n  <OPTION");
		if ( ( value != null ) && ( value.length() > 0 ))
			{
			listItems.append(" VALUE=\"");
			listItems.append( value);
			listItems.append("\"");
			}
		if ( selected )
			listItems.append( " SELECTED ");
		listItems.append( ">");
		listItems.append(  new Formatter(itemDisplay).toString() );
		listItems.append( "</OPTION>");
		}

	/**
	 * Append an items to the end of the list for display
	 * @param itemsToDisplay Enumeration containing items
	 * that user will see in the list and the values
	 * returned
	 */
	public void append( Enumeration itemsToDisplay )
		{
		while ( itemsToDisplay.hasMoreElements() )
			append( itemsToDisplay.nextElement().toString(), null, false);
		}

	public String toString()
		{
		return  "\n<SELECT NAME=\"" + name + "\" SIZE=\"" + size + "\">" +
				listItems.toString() + "\n</SELECT>";
		
		}	
	
	
	}