/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.html;

/**
 * There are three types of Text inputs: scrolling (TEXTAREA tag),
 * single-line (INPUT tag) and password (INPUT tag).
 * Examples of each:<br>
 * <FORM ACTION="http://www.eli.sdsu.edu/crap" METHOD="POST">
 *<TEXTAREA NAME="smith" ROWS=3 COLS=50>Hi Mom</TextArea>Scrolling<br>
 *<INPUT TYPE="text" NAME="userText" VALUE="Hi Dad" SIZE= 30>Single-line<br>
 *<INPUT TYPE="password" NAME="userText" VALUE="secret" SIZE= 30>Password
 * </FORM><br><br>
 * All must be used in a form. Text in input area is reported as: 
 * <B>TextInputName=TextInInputArea</B>. Where TextInputName is the name given
 * to the TextInput object, and TextInInputArea is 
 * the text in the input area when the form
 * is sent.<P>
 * Use static methods TextInput.scrolling( ), 
 * TextInput.singleLine(...), TextInput.password(...) to create
 *  instance of this class.
 * @version 0.8 6 April 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class TextInput implements HTML
	{
	private static final int NOT_SET = -1;
	private static final int TEXT_AREA = -1;
	private static final int INPUT_FIELD = 0;
	private static final int PASSWORD = 1;
	
	private String name;
	private String intialText = null;
	private int numberOfColumns;		// in characters
	private int numberOfRows;
	private int maxTextLength = NOT_SET;
	private int inputType;
	
	/**
	 * Create a scrolling text input area.
	 * @param name name used to report the contents of text area to a cgi program.
	 * The value reported will be the text in the text input.
	 * @param numberOfRows number of rows of text displayable in the text area,
	 * should be at least three, so the scrollbars look reasonable.
	 * @param numberOfColumns the width of the text input in number of characters.
	 */
	public static TextInput scrolling( String name, int numberOfRows, int numberOfColumns)
		{
		TextInput scrolling = new TextInput();
		scrolling.name = name;
		scrolling.numberOfRows = numberOfRows;
		scrolling.numberOfColumns = numberOfColumns;
		scrolling.inputType = TEXT_AREA;
		return scrolling;
		}

	/**
	 * Create a single-line text input area.
	 * @param name name used to report the contents of text area to a cgi program.
	 * The value reported will be the text in the text input.
	 * @param numberOfColumns the width of the text input in number of characters.
	 */
	public static TextInput singleLine( String name, int numberOfColumns)
		{
		TextInput multiLine = new TextInput();
		multiLine.name = name;
		multiLine.numberOfColumns = numberOfColumns;
		multiLine.inputType = INPUT_FIELD;
		return multiLine;
		}

	/**
	 * Create a single-line text input area.
	 * @param name name used to report the contents of text area to a cgi program.
	 * The value reported will be the text in the text input.
	 * @param numberOfColumns the width of the text input in number of characters.
	 * @param maxTextLength maximum number of characters a user is allowed to enter.
	 */
	public static TextInput singleLine( String name, 
												int numberOfColumns, 
												int maxTextLength)
		{
		TextInput singleLine = new TextInput();
		singleLine.name = name;
		singleLine.numberOfColumns = numberOfColumns;
		singleLine.inputType = INPUT_FIELD;
		singleLine.maxTextLength = maxTextLength;
		return singleLine;
		}

	/**
	 * Create a text input area for entering passwords. Input is echoed 
	 * as * or some other single character.
	 * @param name name used to report the contents of text area to a cgi program.
	 * The value reported will be the text in the text input.
	 * @param numberOfColumns the width of the text input in number of characters.
	 */
	public static TextInput password( String name, int numberOfColumns )
		{
		TextInput singleLine = new TextInput();
		singleLine.name = name;
		singleLine.numberOfColumns = numberOfColumns;
		singleLine.inputType = PASSWORD;
		return singleLine;
		}
			
	/**
	 * Sets the inital text seen in the text input. Must be plain text.
	 * No html marked text allowed. Special characters in text are escaped for you.
	 */
	public void setInitalText( String text )
		{
		intialText = ( new Formatter( text)).toString();
		}
	
	public String toString()
		{
		switch ( inputType )
			{
			case TEXT_AREA: 	return textAreaTags();
			case INPUT_FIELD: return inputFieldTags();
			case PASSWORD: 	return passwordTags();
			}
		// Compiler requires this
		return "";
		}
		
	private String passwordTags()
		{
		StringBuffer tags = new StringBuffer( 10 );
		tags.append( "\n<INPUT TYPE=\"password\" " );
		tags.append( " NAME=\"" );
		tags.append( name );
		tags.append( "\" SIZE=\"" );
		tags.append( numberOfColumns );
		tags.append( "\"" );
	
		if ( intialText != null )
			{
			tags.append( " VALUE=\"" );
			tags.append( intialText );
			tags.append( "\"" );
			}
		tags.append( ">" );
		
		return tags.toString();
		}
		
	private String inputFieldTags()
		{
		StringBuffer tags = new StringBuffer( 10 );
		tags.append( "\n<INPUT TYPE=\"text\" " );
		tags.append( " NAME=\"" );
		tags.append( name );
		tags.append( "\" SIZE=\"" );
		tags.append( numberOfColumns );
		tags.append( "\"" );
		
		if ( intialText != null )
			{
			tags.append( " VALUE=\"" );
			tags.append( intialText );
			tags.append( "\"" );
			}

		if ( maxTextLength != NOT_SET )
			{
			tags.append( " MAXLENGTH=\"" );
			tags.append( maxTextLength );
			tags.append( "\"" );
			}
		tags.append( ">" );
		
		return tags.toString();
		}
		
	private String textAreaTags()
		{
		StringBuffer tags = new StringBuffer( 9 );
		tags.append( "\n<TEXTAREA NAME=\"" );
		tags.append( name );
		tags.append( "\" COLS=\"" );
		tags.append( numberOfColumns );
		tags.append( "\" ROWS=\"" );
		tags.append( numberOfRows );
		tags.append( "\">" );
		
		if ( intialText != null )
			tags.append( intialText );

		tags.append( "\n</TEXTAREA>" );
		
		return tags.toString();
		}

	/**
	 * Force clients to use static methods
	 */
	private TextInput() {}	
	}