/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.html;

/**
 * An HTML document has a minimal required structure ( HEAD, BODY, etc). 
 * This class 
 * generates the basic structure for a HTML document. Append HTML 
 * objects and/or HTML tags a Webpage object, then use the toString() 
 * method to get the HTML document in ascii format. 
 * 
 * @version 0.8 6 April 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class WebPage
	{
	//Tag invariant: all tags start on new line, no newline at end of tag
	
	private String title = "";
	private StringBuffer body = new StringBuffer();
	private String backgroundColor = null;
	private String textColor = null;
	private String backgroundImageURL = null;
	
	static String versionNumber = "0.8";
	
	/**
	 * A color for use in a web page
	 */
	public static final String AQUA = "#00FFFF";
	public static final String BLACK = "#000000";
	public static final String BLUE = "#0000FF";
	public static final String GRAY = "#808080";
	public static final String GREEN = "#008000";
	public static final String OLIVE = "#808000";
	public static final String RED = "#FF0000";
	public static final String TEAL = "#008080";
	public static final String WHITE = "#FFFFFF";
	
	public WebPage() {}
	
	/**
	 * Create a new page with the given title. 
    */
 	public WebPage( String title )
		{
		this.title = ( new Formatter( title ) ).toString();
		}

	/**
	 * Appends the string representation of item to the page. 
	 */	
	public void append( Object item )
		{
		body.append( item.toString());
		}


	/**
	 * Appends a line break to the page. 
	 */	
	public void appendLineBreak( )
		{
		body.append( "\n<BR>" );
		}

	/**
	 * Removes all elements from the body of the page. 
	 */	
	public void clearPageBody( )
		{
		body = new StringBuffer();;
		}

	/**
	 * Sets the background image of the page. 
	 * @param anImageURL a URL to an image suitable for a 
	 * background
	 */
	public void setBackgroundImage( String anImageURL )
		{
		backgroundImageURL = anImageURL;
		}
		
	/**
	 * Sets the background color of the page. 
	 * Use either the predefined colors in static variables of the class or
	 * the format: #XXYYZZ where XX = hex value of red,
	 * YY = hex value of green, ZZ = hex value of blue
	 * So WHITE = #FFFFFF and BLACK = #000000
	 */
	public void setBackgroundColor( String aColor )
		{
		backgroundColor = aColor;
		}

	/**
	 * Sets the color of the text on the page. 
	 * Use either the predefined colors in static variables of the class or
	 * the format: #XXYYZZ where XX = hex value of red,
	 * YY = hex value of green, ZZ = hex value of blue
	 * So WHITE = #FFFFFF and BLACK = #000000
	 */
	public void setTextColor( String aColor )
		{
		textColor = aColor;
		}
		
	/**
	 * Returns in HTML format the header information of the page.
	 * Uses information set by setBackgroundColor and
	 * setBackgroundImage
	 */
	public String getPageHeader()
		{
		StringBuffer page = new StringBuffer( 15 );
		page.append( "<!DOCTYPE HTML SYSTEM \"html.dtd\">" );	
		page.append( "\n<!-- Document generated via code written using java library sdsu.html version " );	
		page.append( versionNumber );	
		page.append( ">" );	
		page.append( "\n<HTML>\n<HEAD>\n<TITLE>" );	
		page.append( title );	
		page.append( "\n</TITLE>\n</HEAD>" );
		page.append( "\n<BODY" );
		
		
		if ( 	textColor != null )
			{
			page.append( " TEXT=\"" );	
			page.append( textColor );	
			page.append( "\"" );	
			}

		if ( 	backgroundColor != null )
			{
			page.append( " BGCOLOR=\"" );	
			page.append( backgroundColor );	
			page.append( "\"" );	
			}
		if ( 	backgroundImageURL != null )
			{
			page.append( " BACKGROUND=\"" );	
			page.append( backgroundImageURL );	
			page.append( "\"" );	
			}

		page.append( ">" );
		return page.toString();
		}	
	
	/**
	 * Returns in HTML format items append to page.
	 */
	public String getPageBody()
		{
		return body.toString();
		}

	
	/**
	 * Returns in HTML format tags to end the page
	 */
	public String getPageEnding()
		{
		return  "\n</BODY>\n</HTML>";
		}
			
	/**
	 * Return the WebPage as html in a string.
	 */	
	public String toString()
		{
		return getPageHeader() + getPageBody() + getPageEnding();
		}
	
		
		
	}
