/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
import sdsu.io.FilePrefixFilter;

/**
 * A Repository is a hashtable like interface to a persistent storage
 * medium. Items are placed into the repository via the put method, 
 * retrieved via the get method. The items in the repository are 
 * preserved between invocations of the programs that access the
 * repository. Each item in the repository has a key (or label) 
 * which identifies the item, and an optional comment. 
 *
 * The ASCIIFileRepository stores each item in a separate ASCII file.
 * The name of the file is in the format "key.class", where key =
 * key for the item, class is the short name of the class of the item.
 * Items in an ASCIIFileRepository can be instances of: String, 
 * java.util.Properties, sdsu.util.Stringizable (which includes 
 * sdsu.util.List, sdsu.util.LabeledData, sdsu.util.Table, 
 * sdsu.util.LabeledTable) and any class that implements 
 * java.io.Serializable. When created by the ASCIIFileRepository 
 * a file contains comments and metadata. Comments are lines starting with
 * '#' and end at the end of the line. Metadata are on lines starting with
 * "#!". Metadata includes the key and class of the item in the file.
 * The file starts with comment lines, then metadata lines, then 
 * comment lines, then a string version of the item. 
 *
 * If the file is created by hand the comments and metadata are not 
 * required.

 * @version 0.9  June 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class ASCIIFileRepository implements Repository
	{
	static final int EOFIndicator = -1;
	File repositoryDirectory;
	
	Hashtable cachedObjects = new Hashtable();
		
	/**
	 * Open a ASCIIFileRepository.
	 * @param directoryName The directory containing the repository files.
	 */
	public ASCIIFileRepository( String directoryName )
		{
		this( new File( directoryName ) );
		}

	/**
	 * Open a ASCIIFileRepository.
	 * @param repositoryDirectory The directory containing the 
	 * repository files.
	 */
	public ASCIIFileRepository( File repositoryDirectory )
		{
		this.repositoryDirectory = repositoryDirectory;
		}

		
	/**
	 * Returns true if the repository exists
	 */
	public boolean exists()
		{
		return repositoryDirectory.exists();
		}
		
	/**
	 * Create the required storage structures for the
	 * repository. Returns true if create was successful, false otherwise.
	 */ 
	public boolean create()
		{
		if ( repositoryDirectory.exists() )
			return true;
		else
			return repositoryDirectory.mkdirs();
		}

	/**
	 * 
	 * Returns false as does not support subrepositories.
	 */ 
	public boolean createSubrepository( String name )
		{
		return false;
		}	
		
	/**
	 * Add the object "value" to the repository with the 
	 * given key.
	 */
	public void put( String key, Object value) throws IOException
		{
		put( key, value, null );
		}


	public void put( String key, Object value, String comment ) 
		throws IOException
		{
		LabeledObject data = new LabeledObject( value, key, comment );
		cachedObjects.put( key, data );
					
		saveToFile( data.asString(), key, data.shortClassName() );		
		}

	/**
	 * Add the object "value" to the repository with the 
	 * given key. Value is stored using java's serialization
 	 */
	public void putSerializable( String key, java.io.Serializable value) throws IOException
		{
		put( key, value, null );
		}


	public void putSerializable( String key, java.io.Serializable value, String comment ) 
		throws IOException
		{
		LabeledObject data = new LabeledObject( value, key, comment );
		cachedObjects.put( key, data );
					
		saveToFile( data.serializableToString(), key, data.serializableShortName() );		
		}
		
	
	/**
	 * Retreive the item in the repository stored under the given key.
	 */
	public Object get( String  key) throws IOException
		{
		if (cachedObjects.containsKey( key) )
			{
			LabeledObject labeledValue = 
					(LabeledObject) cachedObjects.get( key);
			return labeledValue.getObject();
			}
		else
			{
			LabeledObject labeledValue = getFromFile( key);
			if (labeledValue == null )
				return null;
				
			cachedObjects.put( key, labeledValue);	
			return labeledValue.getObject();
			}
		}
		
	/**
	 * Returns a null as subrepository are not supported.
	 */ 
	public Repository getSubrepository( String name )
		{
		return null;
		}	

	/**
	 * Returns true is the repostitory contains data at the given key
	 */
	public boolean containsKey( String key )
		{
		String[] fileList = 
			repositoryDirectory.list( new FilePrefixFilter( key));
		if (fileList.length == 0 )
			return false;
		else
			return true;
		}

	/**
	 * Returns false as does not support subrepositories
	 */
	public boolean containsSubrepository( String name )
		{
		return false;
		}

	/**
	 * The name separator is the char used to separate the name of a 
	 * Repository and a subrepository or key. The default char used is a
	 * '.'	
	 */
	 public void setNameSeparator( char newSeparator )
	 	{
	 	}
	 	
	 	
	/**
	 * Returns null as does not support subrepositories
	 */ 
	public Enumeration getSubrepositories( )
		{
		return null;
		}	

	
	/**
	 * Returns an enumeration of the keys in the repository
	 */
	public Enumeration keys()
		{
		String[] fileList = repositoryDirectory.list();
		
		Vector keys = new Vector( fileList.length );
		
		for ( int k = 0; k < fileList.length; k++ )
			{
			int separatorIndex = fileList[k].indexOf( "." );
			 
			if ( separatorIndex > 0 )
				keys.addElement( fileList[k].substring(0, separatorIndex ));
			}
		
		return keys.elements();
		}
		
	/**
	 * Returns true if the repository contains no keys or values
	 */
	public boolean isEmpty()
		{
		// this is not efficent, but quick to implement
		Enumeration theKeys = keys();
		
		if ( theKeys.hasMoreElements() )
			return true;
		else
			return false;
		}
	
	/**
	 * Returns the number of keys in the repository.
	 * Same as length
	 */
	public int size()
		{
		Enumeration theKeys = keys();
		int size = 0;
		
		while ( theKeys.hasMoreElements() )
			{
			size++;
			theKeys.nextElement();
			}
		return size;
		}
	
	/**
	 * Returns the number of keys in the repository.
	 * Same as size.
	 */
	public int length()
		{
		return size();
		}
	
	/**
	 * Removes the given key and corresponding value from the repository.
	 */		
	public boolean remove( String key)
		{
		String[] fileList = 
			repositoryDirectory.list( new FilePrefixFilter( key));

		boolean result = false;
		for ( int k = 0; k < fileList.length; k++ )
			{
			File removeMe = new File( repositoryDirectory, fileList[k] );
			result = result || removeMe.delete();
			}
		return result;
		}
	
	
	private LabeledObject getFromFile( String dataLabel ) throws IOException
		{
		String[] fileList = 
			repositoryDirectory.list( new FilePrefixFilter( dataLabel));
			
		if (fileList.length == 0 )
			return null;
		else
			{
			File dataFile = new File(repositoryDirectory, fileList[0] );
			String fileContents = readFile( dataFile);
			
			int dotIndex = fileList[0].indexOf( "." );
			String objectClass =  fileList[0].substring( dotIndex + 1 );
			return LabeledObject.fromString(fileContents, objectClass);
			}
		} 
	
	private String readFile( File readMe ) throws IOException
		{
		BufferedReader cin = new BufferedReader( new FileReader( readMe ));
		StringBuffer contents = new StringBuffer( 1024 );
		int readSize = 1024;
		
		char[] fileChars = new char[readSize];
		int charsRead = cin.read( fileChars, 0, readSize);
		
		while ( charsRead != EOFIndicator)
			{
			contents.append( fileChars, 0, charsRead );
			charsRead = cin.read( fileChars, 0, readSize);
			}
	
		return contents.toString();
		}
		
	private void saveToFile( String data, String filePrefix, String fileSuffix) 
		throws IOException
		{
		String fileName = filePrefix + "." + fileSuffix;
		File dataFile = new File( repositoryDirectory, fileName);
		
		BufferedWriter out = new BufferedWriter ( new FileWriter( dataFile));
		out.write( data, 0, data.length() );
		out.close();
		}
	}