/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.FilterReader;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.Reader;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.NoSuchElementException;

/**
 * A chunk is any sequence of characters you wish. A chunk could be a
 * single line, a table, or a complete document. The ChunkWriter
 * separates chunks with a string separator, called a chunk seperator.
 * The separator can be any string. The separator is not part of a chunk.
 * The separator should be selected to insure that it does not occur in 
 * the input as a
 * part of a chunk, as it will be interpreted as the end of the chunk
 * not as
 * part of the chunk. ChunkWriter will return each chunk one at a time.
 *
 * @version 0.8 7 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class ChunkReader implements Enumeration
	{
	protected Reader input;
	protected char[] inputBuffer;
	protected int[] shiftTable;
	protected int bufferCharCount;
	
	protected char[] dataSeparator = null;
	
	protected boolean reachedEOF = false;
	protected static int EOFIndicator = -1;
	protected static int DEFAULT_BUFFER_SIZE = 256;
	
	/**
	 * Create a ChunkReader object with a Reader as an input and the 
	 * given separator. This class does not buffer the reader for you.
	 */
	public ChunkReader( Reader in, String separator )
		{
		input = in;
		setSeparator( separator);
		}

	/**
	 * Create a ChunkReader object with a String as an input and the 
	 * given separator.
	 */
	public ChunkReader( String input, String separator )
		{
		this( new StringReader( input) , separator );
		}
	
	/**
	 * Create a ChunkReader object with a file as an input and the 
	 * given separator. File IO will be buffered for you.
	 * @exception FileNotFoundException thrown if fileName is not 
	 * valid file name. 
	 */
	public static ChunkReader fromFile( String fileName, String separator )
		throws FileNotFoundException
		{
		FileReader inputFile = new FileReader(fileName);
		BufferedReader bufferedInput = new BufferedReader( inputFile, 8000 );
		return new ChunkReader( bufferedInput, separator );
		}
	
	public void setSeparator( String newSeparator )
		{
		dataSeparator = newSeparator.toCharArray();		
		initializeQuickSearch();
		}

    /**
	 * Returns true if there are more blocks of data in the input.
     */
    public boolean eof()
    	{
    	return hasMoreElements();
    	}
		
    /**
	 * Returns true if there are more blocks of data in the input.
     */
    public boolean hasMoreElements()
    	{
    	// there two sources of input, reachedEOF contians the status
    	// of the original input source, bufferCharCount the number of chars
    	// in the bufferedInput 
    	if (reachedEOF && (bufferCharCount == 0 ))
    		return false;
    	else 
    		return true;
    	}

	/**
	 * Returns the next block of data.
	 * @exception NoSuchElementException if there are no blocks of 
	 * data left.
	 */
	public Object readChunk() 
		{
		return nextElement();
		}
		
	/**
	 * Returns the next block of data.
	 * @exception NoSuchElementException if there are no blocks of 
	 * data left.
	 */
	public Object nextElement() 
		{
		StringBuffer outputBuffer = new StringBuffer(126);
		try
			{
			boolean foundDataSeparator = false;
			while ( !foundDataSeparator)
				{
				fillInputBuffer();
				foundDataSeparator = searchInputBuffer( outputBuffer );
				}

			}
		catch (IOException error )
			{
			throw new NoSuchElementException(
				"IO error in tring to find next element: " +
				error.getMessage());
			}
			
		return outputBuffer.toString();
		}
		
	/**
	 * Search buffer for the data separator.
	 * @parameter outputBuffer all characters in the inputbuffer 
	 *    that are
	 * 	before the next separator are added to the outputBuffer. 
	 */
	protected boolean searchInputBuffer(StringBuffer outputBuffer)
		{
		// Use quick-search to find pattern. Fill inputBuffer with text. 
		// Process all text in inputBuffer. Place processed text in 
		//  outputBuffer.

		int searchStart = 0;
		int windowStart = 0;
		int separatorLength = dataSeparator.length;

		// Search until pattern extends past end of inputBuffer
		while ( searchStart < bufferCharCount - separatorLength + 1 )
			{
			boolean foundSeparator = true;
			
			// The search
			for ( int index = 0; index < separatorLength; index++ )
				if ( dataSeparator[index] != inputBuffer[index + searchStart] )
					{
					foundSeparator = false;
					break; // for loop
					}
		
			if ( foundSeparator )
				{
				// move text before pattern
				outputBuffer.append( inputBuffer,
										   windowStart, 
										   searchStart - windowStart);
				
				int newWindowStart = searchStart + separatorLength;
				System.arraycopy( inputBuffer, newWindowStart, 
									inputBuffer,0, bufferCharCount - newWindowStart);
				bufferCharCount = bufferCharCount - newWindowStart;
				return true;	//exit loop to shift window and exit
				}
			else
				{
				// look farther along in inputBuffer
				int charLocationAfterPattern = searchStart+ separatorLength;
				
				if ( charLocationAfterPattern >= bufferCharCount )
					searchStart +=  1;
				else
					{
					searchStart += 
						getShift( inputBuffer[ charLocationAfterPattern] );
						
					if (searchStart > bufferCharCount) 
							searchStart = bufferCharCount;

					}
				}		
			}
		//reached end of window and not found pattern
		//move chars already searched
		if ( reachedEOF  )
			{
			outputBuffer.append( inputBuffer, 
							         windowStart, 
							         bufferCharCount - windowStart);
			bufferCharCount = 0;
			return true; //EOF is a separator
			}
		else
			{
			outputBuffer.append( inputBuffer, 
			                     windowStart, 
			                     searchStart - windowStart);
			                     
			System.arraycopy( inputBuffer, searchStart, 
								inputBuffer,0, bufferCharCount - searchStart);
			bufferCharCount = bufferCharCount - searchStart;
			return false;
			}

		}


	/**
	 * Fill  input buffer with chars from input
	 * Read until buffer is full or reach EOF
	 * On reaching EOF close input reader
	 */
	final protected void fillInputBuffer() throws IOException
		{
		int charsToRead = inputBuffer.length - bufferCharCount;
		
		while ( (charsToRead > 0 ) && ( !reachedEOF )  )
			{
			int firstEmptySlotInWindow = bufferCharCount;
			
			int charsRead = input.read( inputBuffer, 
			                            firstEmptySlotInWindow,
			                            charsToRead );
			
			if (charsRead == EOFIndicator)
				{
				reachedEOF = true;
				input.close();
				}
			else if (charsRead > 0 )
				{
				bufferCharCount = bufferCharCount + charsRead;
				charsToRead = charsToRead - charsRead;
				}
			else
				throw new IOException(
					"Read attempted. Did not reach EOF and " +
					" no chars were read");
			}
		}			
	
	/**
	 * Return the number of positions we can shift pattern when
	 * findMyShift is character in inputBuffer after the pattern
	 */
	protected int getShift( char findMyShift )
		{
		if ( findMyShift >= shiftTable.length )
			return 1;
		else
			return shiftTable[ findMyShift];
		}


	private void initializeQuickSearch()
		{
		// quicksearch assumes access to character after the separator.
		// Insure that input buffer has at least than size. The larger 
		// the buffer
		// the few reads, which improves performance.
		inputBuffer = new char[ Math.max( dataSeparator.length + 1, 
		                                  DEFAULT_BUFFER_SIZE )];

		bufferCharCount = 0;

		// allocate for most ascii characters
		// for chars not in table will use brute force search
		shiftTable = new int[126];	
		
		// build shiftTable for quick search
		// Entry for character X contains how far to shift
		// pattern when pattern does not match text and  
		// character X is the character in text after end of
		// pattern
		
		// Default for characters not in pattern
		for (int k = 0; k < shiftTable.length; k++)
			shiftTable[k] = dataSeparator.length + 1;
		
		for (int k = 0; k < dataSeparator.length; k++ )
			if (dataSeparator[k] < shiftTable.length )
				shiftTable[ dataSeparator[k]] = dataSeparator.length - k;
		}
		
	} // end class
