/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.io;

import java.io.*;

/**
 * A chunk is any sequence of characters you wish. A chunk could be a
 * single line, a table, or a complete document. The ChunkWriter
 * separates chunks with a string separator, called a chunk seperator.
 * This allows other classes (ChunkReader for example) to determine
 * where one chunk ends and another begins. Currently the ChunkWriter
 * does <B>not</B> check chunks for occurences of the chunk separator,
 * so select seperator strings that will not occur in your chunks.
 * Future versions of the ChunkWriter will escape occurances of the 
 * seperator in the chunk.<P>
 *
 * The writeChunk methods will end a chunk. All other writes methods
 * (inherited from FilterWriter) add to the current chunk.
 *
 * @version 0.5 7 January 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class ChunkWriter extends FilterWriter
	{
	private String chunkSeparator;
	
	/**
	 * Create a new ChunkWriter
	 * @param out Writer the ChunkWriter will writer to
	 * @param chunkSeparator The string that will placed between chunks.
	 *		chunkSeparator should be at least two characters. Select a string
	 *		that is not likely to occur in your chunks.
	 */
	public ChunkWriter( Writer out, String chunkSeparator)
		{
		super( out);
		this.chunkSeparator = chunkSeparator;
		}
		
	public void setChunkSeparator( String newSeparator )
		{
		chunkSeparator = newSeparator;
		}
	
	/**
	 * Writes the string chunk and a chunkSeperator to end 
	 * the current chunk
	 */
	public void writeChunk(String chunk) throws IOException 
		{
		super.write(chunk + chunkSeparator);
		}

	/**
	 * Writes the chunkSeperator to end the current chunk
	 */
	public void writeChunk() throws IOException 
		{
		super.write( chunkSeparator);
		}
	
	}