/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;
  
import java.io.PushbackReader;
import java.io.IOException;
import sdsu.util.LabeledData;

/**
 * Parses two types of comments: normal and data. A <B>normal comment</B> 
 * starts with a comment character on a new line. (Actually a comment in indicated
 * when the first non-whitespace character on a new line is the comment char.)
 * A <B> data comment</B> is a comment with the special comment character 
 * occuring immediatley after the comment character in a comment and the 
 * text in the comment is formated in name-value pairs. These pairs can be 
 * used to create a LabeledData object.
 *
 * @version 1.0 12 June 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
//12345678901234567890123456789012345678901234567890123456789012345678
public class CommentParser
	{
	private static final int EOF = -1;
	private static final String lineSeparator = 
		System.getProperty("line.separator");
	
	private static final char RETURN = '\n';
	private static final char NEWLINE = '\r';
	private  String whiteSpace = "\t\r\n ";
	private char commentChar;
	private char specialCommentChar;
	private PushbackReader input;
	
	public CommentParser(  PushbackReader input,
											char commentChar, 
											char specialCommentChar )
		{
		this(input, commentChar, specialCommentChar, "\t\n\r ");
		}
	
	public CommentParser(  PushbackReader input, 
											char commentChar, 
											char specialCommentChar, 
											String whiteSpace )
		{
		this.input = input;
		this.commentChar = commentChar;
		this.specialCommentChar = specialCommentChar;
		this.whiteSpace = whiteSpace;
		}
	
	/**
	 * Insures that the input stream starts with non-whitespace char
	 */
	public  void removeWhiteSpace( ) throws IOException
		{
		int c = input.read();
		while ( whiteSpace.indexOf( (char) c ) > -1 )
			c =    input.read();
		
		input.unread( c );
		}

	/**
	 * Returns rest of the input in a string
	 */
	public String readAll( ) throws IOException
		{
		StringBuffer data = new StringBuffer();
		removeWhiteSpace(  );
		
		int c = input.read();
		
		while ( c != EOF )
			{
			data.append( (char) c );
			c = input.read();
			}
		return data.toString();
		}
		
	/**
	 * Returns comment from the input
	 * If input does not start with a comment return a string
	 * of length zero
	 */
	public  String readComment(  ) throws IOException
		{
		StringBuffer comment = new StringBuffer();
		
		boolean isEndOfComment = false;
		
		while ( ! isEndOfComment )
			{
			removeWhiteSpace(  );

			int char1 = input.read();
			int char2 = input.read();
			input.unread( char2 );	
					
			if ( ( commentChar == (char) char1 ) && 
				 ( specialCommentChar != (char) char2 ))
				{
				comment.append( readLine(  ));
				comment.append( lineSeparator );
				}
			else
				{
				input.unread( char1 );
				isEndOfComment = true;
				}
			}
		return comment.toString();
		}

	/**
	 * Returns special comment from the input
	 * If input does not start with a special comment return a string
	 * of length zero
	 */
	public LabeledData readDataComment( ) throws IOException
		{
		StringBuffer comment = new StringBuffer();
		
		boolean isEndOfComment = false;
		
		while ( ! isEndOfComment )
			{
			removeWhiteSpace(  );

			int char1 = input.read();
			int char2 = input.read();
					
			if ( (commentChar == (char) char1 ) && 
				 ( specialCommentChar == (char) char2 ))
				{
				comment.append( readLine(  ));
				}
			else
				{
				input.unread( char2 );
				input.unread( char1 );
				isEndOfComment = true;
				}
			}
		LabeledData metaData = new LabeledData();
		if ( comment.length() > 0 )
			metaData.fromString( comment.toString() );
		return metaData;
		}

	/**
	 * Returns the next line of text. Does not return the newline char
	 */
	private  String readLine( ) throws IOException
		{
		StringBuffer line = new StringBuffer();
		int c = input.read();
		while ( ( RETURN != (char) c) && ( NEWLINE != (char) c ) )
			{
			line.append( (char) c );
			c = input.read();
			}
		return line.toString();
		}
	} //End CommentParser
