/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.FilenameFilter;
import java.io.File;

/**
 * FileExtensionFilter can be used by java.io.File and java.awt.FileDialog
 * to select files with a given file extension. A file extension is the 
 * text after a "." in the file name. So foo.bar.txt has the
 * extension txt, it also has the extension .bar.txt. Given either extension
 * the filter will return the file foo.bar.txt. 
 * The comparison of the file extension is not case sensitive. So .txt
 * is considered the same as .TXT
 * A sample use: <pre>
 *		File currentDirectory = new File( "." );
 *		String[] listing = currentDirectory.list( new FileExtensionFilter("html"));
 * </pre> gives all the file ending in .html in the current directory.
 *
 * @version 1.0 11 November 1997
 * @author Roger Whitney 
 *   (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */ 
 
 
public class FileExtensionFilter implements FilenameFilter
	{
	String fileExtension;
	
	/**
	 * @param extension the file extension to be filtered for.
	 *		The strings .txt and txt will both filter for files ending in .txt
	 */ 
	public FileExtensionFilter( String extension )
		{
		if ( extension.startsWith( "." ))
			fileExtension = extension.toLowerCase();
		else
			fileExtension = "." + extension.toLowerCase();
		}

	 /**
	  * Called by File and FileDialog to select files.
	  * Returns true on file names that File and FileDialog should accept
	  */
	 public  boolean accept(File directory, String fileName)
		{
		String lowerCaseName = fileName.toLowerCase();
		if ( lowerCaseName.endsWith( fileExtension ) )
			return true;
		else
			return false;
		}
	}
