/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.FilenameFilter;
import java.io.File;

/**
 * FilePrefixFilter can be used by java.io.File and java.awt.FileDialog
 * to select files with a given file prefix. A file prefix is the 
 * text before a "." in the file name. So foo.bar.txt has the
 * prefix foo, and foo.bar. Given either prefix
 * the filter will accept the file foo.bar.txt. That is when  
 * The comparison of the file extension is not case sensitive. So lost
 * is considered the same as LOST
 * A sample use: <pre>
 *		File currentDirectory = new File( "." );
 *		String[] listing = currentDirectory.list( new FilePrefixFilter("lost"));
 * </pre> gives all the file starting with "lost" in the current directory.
 *
 * @version 1.0 11 November 1997
 * @author Roger Whitney 
 *   (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */ 
 
 
public class FilePrefixFilter implements FilenameFilter
	{
	String filePrefix;
	
	/**
	 * @param extension the file extension to be filtered for.
	 *		The strings .txt and txt will both filter for files ending in .txt
	 */ 
	public FilePrefixFilter( String extension )
		{
		if ( extension.endsWith( "." ) )
			filePrefix = extension.toLowerCase();
		else
			filePrefix = extension.toLowerCase() + ".";
		}

	 /**
	  * Called by File and FileDialog to select files.
	  */
	 public  boolean accept(File directory, String fileName)
		{
		String lowerCaseName = fileName.toLowerCase();
		if ( lowerCaseName.startsWith( filePrefix ) )
			return true;
		else
			return false;
		}
	}
