/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.Serializable;
import java.io.IOException;
import java.io.StringReader;
import java.io.PushbackReader;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.util.NoSuchElementException;
import java.util.Properties;
import sdsu.util.*;
import sdsu.io.StringReplaceReader;
import sdsu.io.StringizableReader;
import sdsu.io.StringizableWriter;

/**
 * LabeledObject is used to store an object (which can be either an
 * instance of String, java.util.Properties, sdsu.util.Stringizable, or
 * java.io.Serializable) with an optional label
 * and comment. The label is to be used as a key in a hashtable.
 * The comment is for human usaged.
 *
 * @version 1.0 6 June 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 * @see sdsu.io.StringizableReader, sdsu.io.StringizableWriter
 */
class LabeledObject implements Serializable
	{
	protected static final char COMMENT_CHAR = 
		StringizableReader.DEFAULT_COMMENT_CHAR;

	protected static final char METADATA_CHAR =
		StringizableReader.DEFAULT_METADATA_CHAR;

	protected static final String lineSeparator = 
		System.getProperty("line.separator");

	protected static final String SERIALIZE_SHORT_NAME = "Serializable";

	Object object;
	String metaDataComment;
	String comment;
	String label;
	LabeledData metaData;
	
	protected LabeledObject()
		{
		this(null, null, null );
		}
		
	/**
	 * Create a new LabeledObject on item with null comment and null
	 * label
	 */
	public LabeledObject( Object item)
		{
		this( item, null, null);
		}

	/**
	 * Create a new LabeledObject on item with the given label and
	 * null comment
	 */
	public LabeledObject( Object item, String label)
		{
		this( item, label, null );
		}
	
	
	/**
	 * Create a new LabeledObject on item with the given label and
	 * comment
	 */
	public LabeledObject( Object item, 
									String label, 
									String comment)
		{
		object = item;
		this.label = label;
		this.comment = comment; 
		}

	/**
	 * Recreates a LabeledObject from a string created by the 
	 * LabeledObject.toString() method.
	 */
	public static LabeledObject fromString( String objectString ) 
		throws IOException
		{
		return fromString( objectString, "String" );
		}

	/**
	 * Recreates a LabeledObject from a string created by the 
	 * LabeledObject.asString() method. If class of the object
	 * is not recorded in the objectString use defaultClass as
	 * the class.
	 */	
	public static LabeledObject fromString( String objectString, 
														 String defaultClass ) 
														 throws IOException
		{
		LabeledObject recreated = new LabeledObject();
		recreated.fromObjectString( objectString, defaultClass);
		return recreated;
		}


	public String getComment()
		{
		return comment;
		}

	public String getLabel()
		{
		return label;
		}

	public Object getObject()
		{
		return object;
		}
	
	public void setObject( Object newData )
		{
		object = newData;
		}
		
	public void setComment( String comment )
		{
		this.comment = comment;
		}
	
	/**
	 * Return a short name for the class of the Object stored.
	 */
	public String shortClassName()
		{
		String fullClassName = object.getClass().getName();
		int packageSeparatorIndex = fullClassName.lastIndexOf( "." );
		String shortClassName = 
			fullClassName.substring( packageSeparatorIndex + 1);

		if ( ( object instanceof Stringizable ) ||
			 ( object instanceof java.util.Properties ) ||
			 ( object instanceof java.lang.String )
		   )
			return shortClassName;
		else if ( object instanceof java.io.Serializable)
			return serializableShortName();
		else				// Not stringizeble, produce a generic string
			return "String";
		}
	
	/**
	 * Return a short name for the class of the serializable
	 * object stored.
	 */
	public String serializableShortName()
		{
		return SERIALIZE_SHORT_NAME;
		}
		
	/**
	 * Return a string representation of the LabeledObject.
	 * Can't use toString() due IOException
	 * @exception IOException when there is some error in converting
	 *		to an object.
	 */ 	
	public String asString() throws IOException
		{
		if ( object instanceof Stringizable )
			return stringizableToString();
		else if ( object instanceof java.util.Properties )
			return propertiesToString();
		else if ( object instanceof java.lang.String )
			return stringToString();
		else if ( object instanceof java.io.Serializable)
			return serializableToString();
		else
			{
			// Not stringizeble, produce a generic string
			return stringToString();
			}			
		}

	private LabeledData createMetaDataObject( String className )
		{
		LabeledData metaData = new LabeledData();
		metaData.put( "class", className );
		if (label != null )
			metaData.put( "label", label );
		return metaData;
		}
		
	/**
	 * Requires that object be an instance of java.io.Serializable
	 * Returns a encoded String containing metaData( label and class)
	 * and string version of the object. The string can be 
	 * used by this class to recreate the
	 *  object.
	 */
	public String serializableToString() throws IOException
		{
		LabeledData metaData = createMetaDataObject( "Serializable" );
		
		// Serialze Object
		ByteArrayOutputStream serializedObject = 
			new ByteArrayOutputStream();
		ObjectOutputStream cout = 
         new ObjectOutputStream( serializedObject );
  		cout.writeObject( object );
		cout.close();

		StringBuffer stringVersion = new StringBuffer();
		appendMetaData( stringVersion, metaData );
		appendComment( stringVersion, comment );

		stringVersion.append( serializedObject.toString() );
		return stringVersion.toString();
		}

	/**
	 * Requires that object be an instance of java.lang.String
	 * Returns a encoded String containing metaData( label and class)
	 * and string version of the String object. The string can be 
	 * used by this class to recreate the
	 * String object.
	 */
	private String stringToString()
		{
		LabeledData metaData = createMetaDataObject( "java.lang.String" );
		
		StringBuffer stringVersion = new StringBuffer();
		appendMetaData( stringVersion, metaData );
		appendComment( stringVersion, comment );

		stringVersion.append( object.toString() );
		return stringVersion.toString();
		}
	
	/**
	 * Requires that object be an instance of java.util.Properties
	 * Returns a encoded String containing metaData( label and class)
	 * and string version of the Properties object. The string can be 
	 * used by this class and the Properties class to recreate the
	 * Properties object.
	 */
	private String propertiesToString()
		{
		LabeledData metaData = createMetaDataObject( object.getClass().getName() );
		
		StringBuffer stringVersion = new StringBuffer();
		appendMetaData( stringVersion, metaData );
		appendComment( stringVersion, comment );

		ByteArrayOutputStream out = new ByteArrayOutputStream();
		((Properties) object).save( out , null );
			// Note Properties add a time stamp as first line. 
			// The time stamp becomes part of the comment,
			// I leave it in to avoid worring about the format of string 
			// created by save
			
			
		stringVersion.append( out.toString() );
		return stringVersion.toString();
		}
		
		
	/**
	 * Create an string representation of the LabeledObject, that
	 * can be used by fromString() to recreate the LabeledObject.
	 * Assumes that the object being labeled is Stringizable.
	 */	
	private String stringizableToString()
		{		
		StringBuffer stringVersion = new StringBuffer();
		
		LabeledData metaData = ((Stringizable) object).getMetaData();
		
		// label is stored in metadata
		if (label != null )
			metaData.put( "label", label );

		appendMetaData( stringVersion, metaData );
		appendComment( stringVersion, comment );

		// Add actual object data	
		stringVersion.append( object );
		stringVersion.append(lineSeparator);
		stringVersion.append(lineSeparator);
		return stringVersion.toString();
		}

	/**
	 * Recreates a LabeledObject from a string created by the 
	 * asString() method. Default class is the class to recreate
	 * when no class is listed in the metaData.
	 */
	private void fromObjectString( String objectString, String defaultClass ) 
		throws IOException
		{
		try
			{
			PushbackReader dataStream = new PushbackReader( 
													new StringReader( objectString), 2);
			CommentParser parser = 
				new CommentParser( dataStream, COMMENT_CHAR, METADATA_CHAR);
			metaDataComment = parser.readComment(  );
			metaData = parser.readDataComment(  );
			comment = parser.readComment(  );
			String dataString = parser.readAll(  );
			
			// If no metadata and no comment, then metaDataComment is 
			// really the comment.
			if (	( comment == null) && 
					(metaDataComment != null) && 
					( metaData.size() == 0)
				)
				{
				comment = metaDataComment;
				metaDataComment = null;
				}
				
			String className = metaData.getData( "class", defaultClass);
			String lowerCaseClassName = className.toLowerCase();
			if ( lowerCaseClassName.endsWith("string") )
				{
				object = dataString;
				}
			else if ( lowerCaseClassName.endsWith("properties") )
				{
				Properties dataObject = new Properties();
				ByteArrayInputStream in = 
						new ByteArrayInputStream ( dataString.getBytes() );
						
				dataObject.load( in );
				object = dataObject;
				}
			
				// Explicit listing of standard Stringizable classes
				//  allows for the 
				// use of just short name of class(in repository class) 
			else if ( lowerCaseClassName.endsWith("labeleddata") )
				{
				object = makeStringizedObject( "sdsu.util.LabeledData", 
				                                metaData, dataString);
				}
			else if ( lowerCaseClassName.endsWith("labeledtable") )
				{
				object = makeStringizedObject( "sdsu.util.LabeledTable", 
				                                 metaData, dataString);
				}
			else if ( lowerCaseClassName.endsWith("table") )
				{
				object = makeStringizedObject( "sdsu.util.Table", 
				                                 metaData, dataString);
				}
			else if ( lowerCaseClassName.endsWith("list") )
				{
				object = makeStringizedObject( "sdsu.util.List", 
				                                 metaData, dataString);
				}
			else if ( lowerCaseClassName.endsWith( 
								SERIALIZE_SHORT_NAME.toLowerCase()) )
				{
				ByteArrayInputStream  input = 
					new ByteArrayInputStream ( dataString.getBytes());
				ObjectInputStream in = new ObjectInputStream( input );
				object =  in.readObject();
				}
			else // Assume Strinizable and full class name is given
				object = makeStringizedObject( className, metaData, dataString);		
			}
		catch ( Exception error )
			{
			throw new NoSuchElementException( "Error in reading input: " +
				error.getMessage());
			}
		}

	private Object makeStringizedObject( String className, 
	                                     LabeledData metaData, 
	                                     String objectData)
		throws ConversionException, 
		       ClassNotFoundException, 
		       IllegalAccessException, 
		       InstantiationException
		{
		Stringizable data = 
			(Stringizable) Class.forName( className ).newInstance();
			
		data.setMetaData( metaData );
		data.fromString( objectData );
		return data;
		}
		
	/**
	 * Appends the metaData to the buffer
	 */
	private void appendMetaData(StringBuffer buffer, LabeledData metaData)
		{
		// Add metaData Line
		// Format \n#$ metadataKey=value \n
		buffer.append(lineSeparator);
		buffer.append(COMMENT_CHAR);
		buffer.append(METADATA_CHAR);
		buffer.append( " ");
		buffer.append(metaData);
		buffer.append(lineSeparator);
		}

	/**
	 * Appends the comment to the buffer
	 */
	private void appendComment(StringBuffer buffer, String comment)
		{
		// Add comment, needs comment char at start of any newline
		// in the comment
		if ( comment != null )
			{
			try
				{
				StringReplaceReader addCommentChar =
					new StringReplaceReader( comment, 
													lineSeparator, 	
													"" + COMMENT_CHAR + lineSeparator);
				buffer.append(COMMENT_CHAR);
				buffer.append(addCommentChar.contents());
				buffer.append(lineSeparator);
				}	
			catch ( Exception skipCommentWhenError )
				{
				}
			}
		}
	}

