/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.IOException;
import java.io.Serializable;
import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * A Repository is a hashtable like interface to a persistent storage
 * medium. Items are placed into the repository via the put method, 
 * retrieved via the get method. The items in the repository are 
 * preserved between invocations of the programs that access the
 * repository. Each item in the repository has a key (or label) 
 * which identifies the item, and an optional comment. 
 *
 * LocalRepository stores each item in a separate file.
 * The name of the file is in the format "key.class", where key =
 * key for the item, class is the short name of the class of the item.
 * This version of LocalRepository supports instances of: String, 
 * java.util.Properties, sdsu.util.Stringizable (which includes 
 * sdsu.util.List, sdsu.util.LabeledData, sdsu.util.Table, 
 * sdsu.util.LabeledTable) and any class that implements 
 * java.io.Serializable. If a class supports Serializable and
 * another supported type (string for example) the non-serializable
 * format is selected over serializable. If you wish to force such an object
 * to be saved via serialization, use putSerializable.
 *  See sdsu.io.ASCIIFileRepository for
 * file formats of these items. ASCIIFileRepository is designed to 
 * support
 * files created by hand for configuration files. These files can contain 
 * persistant comments. 
 *
 * @see sdsu.io.Repository
 * @version 0.9 8 June 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class LocalRepository implements Repository
	{
	static final char DEFAULT_NAME_SEPARATOR = '.';
	
	private char nameSeparator = DEFAULT_NAME_SEPARATOR;
	private File repositoryDirectory;
	private Repository elements;
	private Hashtable subrepositories;
			
	/**
	 * Open a ASCIIFileRepository.
	 * @param directoryName The directory containing the repository files.
	 *  If the directory does not exist, it is not created. In this case
	 *  You must call create() to create the underlying storage before using
	 * the Repository. 
	 */
	public LocalRepository( String directoryName )
		{
		this( new File( directoryName ) );
		}
	
	/**
	 * Open a ASCIIFileRepository.
	 * @param directoryName The directory containing the repository files.
	 *  If the directory does not exist, it is not created. In this case
	 *  You must call create() to create the underlying storage before using
	 * the Repository. 
	 */
	public LocalRepository( File repositoryLocation )
		{
		repositoryDirectory = repositoryLocation;
		elements = new ASCIIFileRepository( repositoryDirectory );
		if ( exists() )
			subrepositories = openSubrepositories( repositoryLocation );
		else
			subrepositories = new Hashtable(); 
		}
	
	/**
	 * Returns true if the repository exists. If repository does 
	 * not exist, use create to create it.
	 */
	public boolean exists()
		{
		return repositoryDirectory.exists();
		}
	
	
	public boolean create()
		{
		return elements.create();
		}

	/**
	 * Create a subrepository. 
	 * Returns true if create was successful, false otherwise.
	 */ 
	public boolean createSubrepository( String name )
		{
		// handle recursive base case and null
		if ( name == null )
			return true;
		else if ( name.equals( "") )
			return true;
		
		// create repositories
		String localKey = prefixKey( name );
		String subkey = suffixKey( name );
		if ( subrepositories.containsKey( localKey ) )
			{
			Repository target = (Repository) subrepositories.get( localKey );
			return target.createSubrepository( subkey );
			}
		else 
			{
			File repositoryLocation = 
					new File(repositoryDirectory, localKey );
			Repository target = 
					new LocalRepository( repositoryLocation);
					
			target.create();
			subrepositories.put( localKey, target);
			return target.createSubrepository( subkey );
			}
		}	

		
	/**
	 * Add the object "value" to the repository or a subrepository with 
	 * the given key.
	 * @param value Currently limited to String, java.util.Properties, 
	 * sdsu.util.Stringizable (which includes 
	 * sdsu.util.List, sdsu.util.LabeledData, sdsu.util.Table, 
 	 * sdsu.util.LabeledTable) Will handle all other types, but not
 	 * very intelligently.
	 * @exception IOException on any IO error in saving the object
	 */
	public void put( String key, Object value) throws IOException
		{
		put( key, value, null );
		}


	/**
	 * Add the object "value" to the repository or a subrepository with 
	 * the given key.
	 * @param value Currently limited to String, java.util.Properties, 
	 * sdsu.util.Stringizable (which includes 
	 * sdsu.util.List, sdsu.util.LabeledData, sdsu.util.Table, 
 	 * sdsu.util.LabeledTable) Will handle all other types, but not
 	 * very intelligently.
 	 * @param key can contain reference to subrepositories. For example
 	 *  "Roger.Sam.key"
	 * @exception IOException on any IO error in saving the object
	 */
	public void put( String key, Object value, String comment ) 
		throws IOException
		{
		put( key, value, comment, false );
		}

	/**
	 * Add the object "value" to the repository with the 
	 * given key. Value is stored using Java's serialization process.
	 */
	public void putSerializable( String key, Serializable value) 
		throws IOException
		{
		putSerializable( key, value, null );
		}


	/**
	 * Add the object "value" to the repository with the 
	 * given key. The object is stored with the given comment.
	 * Value is stored using Java's serialization process.
	 */
	public void putSerializable( String key, Serializable value, String comment ) 
		throws IOException
		{
		put( key, value, comment, true );
		}	

	private void put( String key, Object value, String comment, boolean forceSerialized ) 
		throws IOException
		{
		// handle degenerate cases
		if ( key == null )
			return;
		else if ( key.equals( "") )
			return;


		if ( isBaseKey( key ) )
			{
			if ( forceSerialized )
				elements.putSerializable(  key, (Serializable)value, comment );
			else
				elements.put( key, value, comment);
			}
		else if ( subrepositories.containsKey( prefixKey( key )))
			{
			Repository target; 
			target = (Repository) subrepositories.get( prefixKey( key ));
			if ( forceSerialized )
				target.putSerializable( suffixKey( key ), (Serializable)value, comment );
			else
				target.put( suffixKey( key ), value, comment );
			}
		else // create subrepository to hold the object
			{
			createSubrepository( prefixKey( key ) );
			Repository target; 
			target = (Repository) subrepositories.get( prefixKey( key ));
			target.put( suffixKey( key ), value, comment );
			}
		}
	
	/**
	 * Retreive the item in the repository or a subrepository
	 * stored under the given key.
 	 * @param key can contain reference to subrepositories. For example
 	 *  "Roger.Sam.key"
	 */
	public Object get( String  key) throws IOException
		{
		// handle degenerate cases
		if ( key == null )
			return null;
		else if ( key.equals( "") )
			return null;
			
			
		if ( isBaseKey( key ) )
			return elements.get( key);
		else if ( subrepositories.containsKey( prefixKey( key )))
			{
			Repository target;
			target = (Repository) subrepositories.get( prefixKey( key ));
			return target.get( suffixKey( key ));
			}
		else
			{
			return null;
			}
		}
	
	/**
	 * Return a subrepository. Returns null if repository does
	 * not exist.
 	 * @param name can contain reference to subrepositories. For example
 	 *  "Roger.Sam.Pete"
	 */ 
	public Repository getSubrepository( String name )
		{
		// handle degenerate cases
		if ( name == null )
			return null;
		else if ( name.equals( "") )
			return null;
		
		if ( isBaseKey( name ) )
			return (Repository) subrepositories.get( name );
		else
			{
			String localKey = prefixKey( name );
			String subkey = suffixKey( name );

			Repository target;
			target = (Repository) subrepositories.get( localKey );
			return target.getSubrepository( subkey );
			}
		}	

	/**
	 * Return an enumeration of the subrepositories. The enumeration
	 * does not directly contain sub-subrepositories.
	 */ 
	public Enumeration getSubrepositories( )
		{
		return subrepositories.elements();
		}

	/**
	 * Returns true is the repostitory or a subrepository
	 * contains data at the given key
 	 * @param key can contain reference to subrepositories. For example
 	 *  "Roger.Sam.key"
	 */
	public boolean containsKey( String key )
		{
		// handle degenerate cases
		if ( key == null )
			return false;
		else if ( key.equals( "") )
			return false;
			
		if ( isBaseKey( key ) )
			return elements.containsKey( key);
		else if ( subrepositories.containsKey( prefixKey( key )))
			{
			Repository target;
			target = (Repository) subrepositories.get( prefixKey( key ));
			return target.containsKey( suffixKey( key ));
			}
		else
			{
			return false;
			}
		}
	
	/**
	 * Returns true if the repostitory contains the given subrepository
 	 * @param name can contain reference to subrepositories. For example
 	 *  "Roger.Sam"
	 */
	public boolean containsSubrepository( String name )
		{
		// handle degenerate cases
		if ( name == null )
			return false;
		else if ( name.equals( "") )
			return true;
		
		if ( isBaseKey( name ) )
			return subrepositories.containsKey( name );
		else if ( subrepositories.containsKey( prefixKey( name )))
			{
			Repository target;
			target = (Repository) subrepositories.get( prefixKey( name ));
			return target.containsSubrepository( suffixKey( name ) );
			}
		else
			return false;
		}

	
	/**
	 * Returns an enumeration of the keys in the top level repository
	 */
	public Enumeration keys()
		{
		return elements.keys();
		}
		
	/**
	 * Returns true if the repository contains no keys or subrepositories
	 */
	public boolean isEmpty()
		{
		if ( size() == 0 )
			return true;
		else
			return false;
		}
	
	/**
	 * Returns the number of keys and subrepositories in the repository.
	 * Same as length.
	 */
	public int size()
		{
		return elements.size() +  subrepositories.size();
		}
	
	/**
	 * Returns the number of keys and subrepositories in the repository.
	 * Same as size.
	 */
	public int length()
		{
		return size();
		}
	
	/**
	 * Removes the given key and corresponding value from the repository
	 * or subrepository.
	 */		
	public boolean remove( String key)
		{
		if ( key == null )
			return false;
		else if ( key.equals( "") )
			return false;
			
		if ( isBaseKey( key ) )
			return elements.remove( key);
		else if ( subrepositories.containsKey( prefixKey( key )))
			{
			Repository target;
			target = (Repository) subrepositories.get( prefixKey( key ));
			return target.remove( suffixKey( key ) );
			}
		else
			return true; // it does not exists so answer true
		}

	/**
	 * The name separator is the char used to separate the name of a 
	 * Repository and a subrepository or key. The default char used is a
	 * '.'	
	 */
	 public void setNameSeparator( char newSeparator )
	 	{
	 	nameSeparator = newSeparator;
	 	}


	/**
	 * returns true if key for this repository, false if
	 * it is for a subrepository
	 */
	private boolean isBaseKey( String key )
		{
		if ( key.indexOf( nameSeparator ) > -1 )
			return false;
		else 
			return true;
		}	
	
	/**
	 * Returns all chars in key upto the first occurence of
	 * nameSeparator
	 */
	private String prefixKey( String key )
		{
		int separator = key.indexOf( nameSeparator );
		if ( separator == -1 )
			return key;
		else
			return key.substring( 0, separator );
		}

	/**
	 * Returns all chars in key after the first occurence of
	 * nameSeparator
	 */
	private String suffixKey( String key )
		{
		int separator = key.indexOf( nameSeparator );
		if ( separator == -1 )
			return "";
		else
			return key.substring( separator + 1 );
		}

	private Hashtable openSubrepositories( File parentRepository )
		{
		Hashtable childRepositories = new Hashtable();
		String[] childFiles = parentRepository.list();

		for ( int k = 0; k< childFiles.length; k++ )
			{
			File subfile = new File( parentRepository, childFiles[k] );
			if ( subfile.isDirectory() )
				childRepositories.put( childFiles[k], 
												new LocalRepository( subfile ));
			}
		return childRepositories;
		}
		
	
	}