/* 
 * Copyright (C) 1999 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package sdsu.io;

import java.io.FilterInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.InterruptedIOException;

/**
 * This class input stream can be interrupted and be set to time-out if 
 * a read takes too long. Inputstreams in the JDK 1.1, 1.2 block. If you
 * perform a read on a stream
 * the method will not return until input data is available, 
 * the end of the stream is detected, or an exception is thrown.
 * There are times when you would like to "cancel" a read request.
 * All read requests on this stream can be "canceled" in two different
 * ways. First, you can set a time-out duration. The time is measured
 * in wall clock time. If a read request times out wating for any bytes to read
 * the exception InterruptedIOException is thrown. Second, you can send
 * the method Thread.interrupt() to the thread that requested the read. The read
 * will then throw an InterruptedIOException. Regardless of how the read
 * is "canceled" further read requests can be made on the stream. 
 *
 * @version 1.0 6 January 1999 
 * @author Roger Whitney 
 *	(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class NonBlockingInputStream extends FilterInputStream
	{
	private int pollDuration;
	private long timeout;
	
	/**
	 * Create a NonBlockingInputStream on the input stream with poll delay
	 * of 50 milliseconds and time-out length of about 200,000 centuries.
	 */
	public NonBlockingInputStream( InputStream input )
		{
		this( input, 50, Long.MAX_VALUE );
		}
	
	/**
	 * Create a NonBlockingInputStream on the input stream the given poll delay
	 * time-out.
	 * @param pollDelay time in milliseconds
     * @exception  IllegalArgumentException  if pollDelay < 0 or timeout < pollDelay.
	 */
	public NonBlockingInputStream( InputStream input, int pollDelay, long timeout )
		{
		super( input );
		setPollDelay( pollDelay);
		setTimeout( timeout);
		}

    /**
	 * Set the poll delay. This is the time the thread
	 * will sleep between polling the input for a byte of data.
	 *
	 * @param	  delay	 time in milleseconds a read method will
	 * 		sleep between ckecking the input for a byte of data.
	 * @exception  IllegalArgumentException  if duration < 0.
	 */
	public void setPollDelay( int delay ) throws IllegalArgumentException
		{
		if ( delay < 0 )
			throw new IllegalArgumentException( "Poll duration must be >= 0. Value is " +
				delay );
		pollDuration = delay;
		}

    /**
	 * Set the time-out period. This is the time a read() will wait
	 * for a byte of data to read. If the time is exceeded a the read will
	 * throw an exception.
	 * @param	  timeout	 time in milleseconds. If a byte of data does not
	 * 		become available for reading in this time (wall time), a read method
	 * 		will throw an InterruptedIOException.
	 * @exception  IllegalArgumentException  if timeout < 0.
	 * @see read()
	 */
	public void setTimeout( long timeout ) throws IllegalArgumentException
		{
		if ( timeout <= 0 )
			throw new IllegalArgumentException( "Timeout must be > 0. Value is " +
				timeout );
		this.timeout = timeout;
		}
		
    /**
	 * Reads the next byte of data from the input stream. The value 
	 * byte is returned as an <code>int</code> in the range 
	 * <code>0</code> to <code>255</code>. If no byte is available 
	 * because the end of the stream has been reached, the value 
	 * <code>-1</code> is returned.	 
	 * This method polls the input for data. Polling continues
	 * until input data is available,
	 * end of file is detected,  the time-out period is exceeded,
	 * the current threads is interrupted via Thread.interrupt(), or an exception
	 * is thrown.
	 *
	 * @return	 the next byte of data, or <code>-1</code> if the end of the
	 *	 	    stream is reached.
	 * @exception  IOException  if an I/O error occurs.
	 * @exception  InterruptedIOException  if the time-out period is 
	 * 	exceeded in waiting for a byte to read or the thread is interrupted..
	 */
	public int read() throws IOException, InterruptedIOException 
		{
		pollInput();
		return in.read();
		}

    /**
	 * Reads up to <code>length</code> bytes of data from the input stream 
	 * into an array of bytes. 
	 * An attempt is  made to read as many as <code>length</code>
	 * bytes, but a smaller number may be read,
	 * possibly  zero. The number of bytes actually
	 * read is returned as an integer.
	 * <p>
	 * This method polls the input for data. Polling continues
	 * until input data is available,
	 * end of file is detected,  the time-out period is exceeded,
	 * the current threads is interrupted via Thread.interrupt(), or an exception
	 * is thrown.
	 * <p>
	 * If no byte is
	 * available because the stream is at end of
	 * file, the value <code>-1</code> is returned.
	 * <p>
	 * The first
	 * byte read is stored into element <code>inputBuffer[offset]</code>,
	 * the next one into <code>inputBuffer[offset+1]</code>,
	 * and so on. The number of bytes read is,
	 * at most, equal to <code>length</code>.
	 * <p>
	 * If the first byte cannot
	 * be read for any reason other than end of
	 * file, then an <code>IOException</code> is
	 * thrown. In particular, an <code>IOException</code>
	 * is thrown if the input stream has been closed.
	 * <p>
	 *
	 * @param	  inputBuffer	 the buffer into which the data is read.
	 * @param	  offset   the start offset of the data.
	 * @param	  length   the maximum number of bytes read.
	 * @return	 the total number of bytes read into the buffer, or
	 *	 	    <code>-1</code> if there is no more data because the end of
	 *	 	    the stream has been reached.
	 * @exception  IOException  if an I/O error occurs.
	 * @exception  InterruptedIOException  if the time-out period is 
	 * 	exceeded in waiting for a byte to read or the thread is interrupted..
	 * @see	    java.io.InputStream#read()
	 */
	public int read(byte inputBuffer[], int offset, int length) 
			throws IOException, InterruptedIOException
		{
		pollInput();
		int bytesToRead = Math.min( length, in.available() );
		return in.read( inputBuffer, offset, bytesToRead );
		}

	private void pollInput() throws IOException
		{
		long startTime = System.currentTimeMillis();
		while (  in.available() <= 0 ) 
			try 
				{
				if (  System.currentTimeMillis() - startTime >= timeout )
					throw new InterruptedIOException( "Timeout on read after " + 
						timeout + " milliseconds");
				Thread.sleep( pollDuration );
				}
			catch ( InterruptedException stopReading )
				{
				throw new InterruptedIOException( "read() interrupted" );
				}
		}
	}