/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.Serializable;
import java.io.IOException;
import java.util.Enumeration;

/**
 * A Repository is a hashtable like interface to a persistent storage
 * medium. Items are placed into the repository via the put method, 
 * retrieved via the get method. The items in the repository are 
 * preserved between invocations of the programs that access the
 * repository. Each item in the repository has a key (or label) 
 * which identifies the item, and an optional comment. The comment
 * is stored with the object. In future releases the storage meduim 
 * could be local files, files on remote machines, or a database. In
 * this release only local files are supported.<P>
 *
 * A nested repository can contain 
 * other (subrepository) repositories. Different implementations
 * may organize nested repositories in different ways. <P>
 
 * A top level repository can access keys of a subrepository.
 * For example let Roger
 * and Sam be repositories, let Roger contain the keys <I>left</I> 
 * and <I>right</I> and the repository Sam, let Sam contain the keys 
 * <I>right</I> and <I>wrong</I>. Also let the variable roger be a 
 * reference to the repository Roger. Then: <pre>
 * 	roger.get( "right" ); returns object in Roger stored at "right"
 * 	roger.get( "Sam.right"); returns object in Sam stored at "right"
 * </pre>
 * The char '.' is used as the default name separator.
 *
 * @see sdsu.io.LocalRepository
 * @version 0.8 5 June 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
//12345678901234567890123456789012345678901234567890123456789012345678
public interface Repository
	{
		
	/**
	 * Returns true if the repository exists
	 */
	public boolean exists();
	
	/**
	 * Create the required storage structures for the
	 * repository. Returns true if create was successful, false otherwise.
	 */ 
	public boolean create();
	
	/**
	 * Add the object "value" to the repository with the 
	 * given key.
	 */
	public void put( String key, Object value) throws IOException;


	/**
	 * Add the object "value" to the repository with the 
	 * given key. The object is stored with the given comment.
	 */
	public void put( String key, Object value, String comment ) 
		throws IOException;
		
	/**
	 * Add the object "value" to the repository with the 
	 * given key. Value is stored using Java's serialization process.
	 */
	public void putSerializable( String key, Serializable value) 
		throws IOException;


	/**
	 * Add the object "value" to the repository with the 
	 * given key. The object is stored with the given comment.
	 * Value is stored using Java's serialization process.
	 */
	public void putSerializable( String key, Serializable value, String comment ) 
		throws IOException;
	
	/**
	 * Retreive the item in the repository stored under the given key.
	 * Returns null if no object is mapped to the key.
	 */
	public Object get( String  key) throws IOException;

	/**
	 * Returns true if the repostitory contains data at the given key
	 */
	public boolean containsKey( String key );

	/**
	 * Returns an enumeration of the keys in the repository
	 */
	public Enumeration keys();
		
	/**
	 * Returns true if the repository contains no keys or values
	 */
	public boolean isEmpty();
		
	/**
	 * Returns the number of keys in the repository.
	 * Same as length
	 */
	public int size();
	
	/**
	 * Returns the number of keys in the repository.
	 * Same as size.
	 */
	public int length();
	
	/**
	 * Removes the given key and corresponding value from the repository.
	 */		
	public boolean remove( String key);

	/**
	 * Create a subrepository. 
	 * Returns true if create was successful, false otherwise.
	 */ 
	public boolean createSubrepository( String name );	
	

	/**
	 * Return a subrepository. Returns null if repository does
	 * not exist.
	 */ 
	public Repository getSubrepository( String name );	

	/**
	 * Return an enumeration of the subrepositories. The enumeration
	 * does not directly contain sub-subrepositories.
	 */ 
	public Enumeration getSubrepositories( );	
	

	/**
	 * Returns true if the repostitory contains the given subrepository
	 */
	public boolean containsSubrepository( String name );
	
	/**
	 * The name separator is the char used to separate the name of a 
	 * Repository and a subrepository or key. The default char used is a
	 * '.'	
	 */
	 public void setNameSeparator( char newSeparator );

	
	}
	
