/* 
 * Copyright (C) 1999 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;
import java.io.*;

/**
 * This class adds a few useful methods to java.io.File. This class
 * provides methods to backup a file, to read the contents of a file,
 * and write to a file. These methods are very simple, but are done so 
 * often it becomes annoying to implement each time they are needed.
 *
 * @version 1.0 5 January 1999 
 * @author Roger Whitney 
 *	(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class SimpleFile extends java.io.File
	{
	
    /**
     * Creates a <code>File</code> instance that represents the file 
     * whose pathname is the given path argument. 
     *
     * @param      path   the file pathname.
     * @exception  NullPointerException  if the file path is equal to
     *               <code>null</code>.
     * @see        java.io.File#getPath()
     */
    public SimpleFile(String path) 
    	{
    	super( path );
    	}
    	
	 /**
     * Creates a <code>File</code> instance whose pathname is the 
     * pathname of the specified directory, followed by the separator 
     * character, followed by the <code>name</code> argument. 
     *
     * @param   path   the directory pathname.
     * @param   name   the file pathname.
     * @see     java.io.File#getPath()
     * @see     java.io.File#separator
     */
    public SimpleFile(String path, String name) 
    	{
    	super( path, name );
		}
		
    /**
     * Creates a <code>File</code> instance that represents the file 
     * with the specified name in the specified directory. 
     * <p>
     * If the directory argument is <code>null</code>, the resulting 
     * <code>File</code> instance represents a file in the 
     * (system-dependent) current directory whose pathname is the 
     * <code>name</code> argument. Otherwise, the <code>File</code> 
     * instance represents a file whose pathname is the pathname of the 
     * directory, followed by the separator character, followed by the 
     * <code>name</code> argument. 
     *
     * @param   dir   the directory.
     * @param   name   the file pathname.
     * @see     java.io.File#getPath()
     * @see     java.io.File#separator
     */
    public SimpleFile(File dir, String name) 
    	{
		super(dir, name);
		}

    /**
     * Returns the contents of the file. A Buffered Reader is
     * opened on the file, contents of the file is read,
     * and the file Reader is closed before returning the contents.
     * <P>This is a very simple function, but is done often enough
     * to have done for you. It also allows beginners to read files
     * before having to master Readers/Writers.
     * <p>
     * @exception   FileNotFoundException thrown if file name given 
     *	constuctor does not exist.
     * @exception   IOException  Thrown if there is an IOException when
     *	reading the file.
     */
	public String getContents() throws FileNotFoundException, IOException
		{
		SimpleReader buffered = new SimpleReader(
													new BufferedReader( 
														new  FileReader( this )
											    					));
		String contents = buffered.contents();
		buffered.close();
		return contents;
		}
	
    /**
     * Appends the string <B>tailContents</B> to the end of the 
     * file. File is created if it does not exist.
     * @param tailContents	String added to the end of the file.
     * @exception   IOException  Thrown if there is an IOException when
     *	writing to the file.
     */
	public void append( String tailContents ) throws IOException
		{
		write( tailContents, true );
		}
	
	private void write( String contents, boolean append ) throws IOException
		{
		Writer appender = new BufferedWriter(
										new FileWriter( this.getAbsolutePath(), append ));
		appender.write( contents );
		appender.close();
		}
		
    /**
     * Set the contents of the file to be <B>newContents</B> to the end of the 
     * file. Current contents of the file are erased. File is created if it does not exist.
     * @param newContents	String used to replace current contents of the file.
     * @exception   IOException  Thrown if there is an IOException when
     *	writing to the file.
     */
	public void setContents( String newContents ) throws IOException
		{
		write( newContents, false );
		}
		
    /**
     * Create a backup copy of the current file. The name of the backup file 
     * is formed by appending <B>.bak</B> to the current file name. If a 
     * file by that name already exists, it is deleted.
     * @exception   IOException  Thrown if there is an IOException when
     *	creating the backup file.
     */
	public void backup()  throws IOException
		{
		backup( ".bak" );
		}
	
    /**
     * Create a backup copy of the current file. The name of the backup file 
     * is formed by appending <B>fileExtension</B> to the current file name. 
     * If <B>fileExtension</B> does not begin with a period, a period is added
     * between the file name and the fileExtension. If a 
     * file by that name already exists, it is deleted.
     * @exception   IOException  Thrown if there is an IOException when
     *	creating the backup file.
     */
	public void backup( String fileExtension )  throws IOException
		{
		if ( !this.exists() )
			return;
			
		if ( !fileExtension.startsWith( "." ) )
			fileExtension = "." + fileExtension;
		SimpleFile backUp = new SimpleFile(getAbsolutePath() + fileExtension );
		
		if ( backUp.exists() )
			backUp.delete();
			
		backUp.setContents( getContents() );
		}
	}