/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.io;

import java.io.Writer;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Vector;
import java.util.Enumeration;

/**
 * This class acts like an N way split in a writer stream.
 * N different writers can be connected to the end of 
 * a SplitWriter. These N writers are not chained together.
 * What ever is writen on the SplitWriter is copied and written
 * to each of the N writers. The following shows a SplitWriter
 * with three writers. <P>
 * <pre>
 *          |-->----
 *  ---->---|-->----
 *          |-->----
 * </pre>
 *
 * @version 1.0 30 December 1997 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
 
public class SplitWriter extends Writer
	{
	Vector spliter = new Vector();
	
	/**
	 * Create SplitWriter on the given Writer
	 */
	public SplitWriter( Writer output )
		{
		add( output );
		}
		
	/**
	 * Create an empty SplitWriter. Any output on an empty
	 * SplitWriter is ignored.
	 */
	public SplitWriter( )
		{
		}
	
	/**
	 * Add a writer to the end of the SplitWriter.
	 */
	public void add( Writer output )
		{
		spliter.addElement( output );
		}	

	/**
	 * Add an outputStream to the end of the SplitWriter.
	 */
	public void add( OutputStream output )
		{
		add( new PrintWriter( output ));
		}
	
	/**
	 * Removes a writer from the SplitWriter.
	 */
	public void remove( Writer output )
		{
		spliter.removeElement( output );
		}
		
    /**
     * Flush the stream.  If the stream has saved any characters from the
     * various write() methods in a buffer, write them immediately to their
     * intended destination.  Then, if that destination is another character or
     * byte stream, flush it.  Thus one flush() invocation will flush all the
     * buffers in a chain of Writers and OutputStreams.
     *
     * @exception  IOException  If an I/O error occurs
     */
    public void flush() throws IOException
    	{
    	Enumeration writers = spliter.elements();
    	while (writers.hasMoreElements() )
    		{
    		Writer out = (Writer) writers.nextElement();
    		out.flush();
    		}
    	}

    /**
     * Close the stream, flushing it first.  Once a stream has been closed,
     * further write() or flush() invocations will cause an IOException to be
     * thrown.  Closing a previously-closed stream, however, has no effect.
     *
     * @exception  IOException  If an I/O error occurs
     */
    public void close() throws IOException
    	{
    	Enumeration writers = spliter.elements();
    	while (writers.hasMoreElements() )
    		{
    		Writer out = (Writer) writers.nextElement();
    		out.close();
    		}
    	}

    /**
     * Write a portion of an array of characters.
     *
     * @param  charBuffer  Array of characters
     * @param  offset   Offset from which to start writing characters
     * @param  length   Number of characters to write
     *
     * @exception  IOException  If an I/O error occurs
     */
    public void write(char charBuffer[], int offset, int length) throws IOException
    	{
    	Enumeration writers = spliter.elements();
    	while (writers.hasMoreElements() )
    		{
    		Writer out = (Writer) writers.nextElement();
    		out.write( charBuffer, offset, length);
    		}
    	}

	}