/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.PushbackReader;
import java.io.FilterReader;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.Reader;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import sdsu.util.*;
import sdsu.io.ChunkReader;

//34567890123456789012345678901234567890123456789012345678901234567890
/**
 * Input (String, file, or reader) consists of:
 * optional whitespace followed by optional file metadata line
 * followed by one or more data blocks. A data block consists of 
 * optional whitespace
 * followed by optional comment block,  optional whitespace, followed 
 * by optional metadata block, 
 *  optional whitespace, followed by optional comment block, followed 
 * by data.
 * <BR>
 * The file metadata line must be started with the two characters 
 * <B>##</B>.
 * It can contain any of the following three name-value pairs:<BR>
 * DataSeparator=AString;CommentCharacter=A;MetaDataCharacter=B <BR>
 * The <B>DataSeparator</B> value is the string that separates 
 * data blocks. It must be selected
 * not to clash with other strings in the file. The 
 * <B>CommentCharacter</B> value,
 * which must be a single character, is used to indicate the start of 
 * a line in a comment
 * block. The <B>MetaDataCharacter</B> value, which must be a single 
 * character, is used
 * to indicate a meta data block (see below).
 * <BR>
 * A <B>comment block</B> connsists of consecutive lines each starting 
 * with a comment character, 
 * which defaults to '#'. Comments are ignored. The comment block before 
 * the metadata
 * block is called the <B>metadata comment</B>. The comment block before 
 * the data is called the 
 * <B>data comment</B>.
 * <BR><BR>
 * A <B>metadata block</B> consists of consecutive lines each starting with 
 * a comment character 
 * then the metadata character, which defaults to '$'. The metadata block 
 * contains 
 * name-value pairs used to read the data. The format of the name-value 
 * pair is the
 * default format for the class sdsu.io.LabeledData, which is used to 
 * interpret the 
 * name-value pairs. The format is <B>name=value;</B>. That is the name 
 * and value are
 * separated by the character '=', pairs are separated by the character
 *  ';'. White space
 * can be used. The last pair does not need to be followed by ';'. 
 * If the value contains
 * either '=', ';' or whitespace it must be quoted by the single quote 
 * character. For example:
 * <BR>message='hi mom'<BR>
 * 
 * There are two special name-value pairs. The
 * first is "class=AFullyQuallifiedClassName". If this name-value pair 
 * does not exist
 * the data is read as a string. If this name-value pair does exist, 
 * the class <B>must 
 * implement Stringizable</B>. The indicated class, called the <B>data 
 * class</B>.,
 * is used to interpret the data. The other
 * special name-value pair is "label=ALabelForTheData". This name-value 
 * pair is used to label
 * the data block. Other name-values are used as required to help set 
 * the state for the 
 * data class. The name-value pairs can be on multiple lines, all pairs 
 * in the block must be
 * followed by the ';' character, except the last one in the block. 
 *<BR>
 * If a data class is given in the meta data block, the meta data and 
 * the data are used to
 * recreate an object that is an instance of the data class. The meta 
 * data is used as the 
 * argument to the objects setMetaData method. The data is used as the 
 * arument to the 
 * objects fromString method.
 *<BR>
 * The nextElement method of this class returns the data for the next 
 * data block. The 
 * object returned is either a string (if there was no data class) or 
 * is of type data
 * class.
 *
 * @version 0.8 7 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class StringizableReader implements Enumeration
	{
	/**
	 * Name used in file meta data line to indicate the data 
    * separator string.
	 */
	public static final String META_SEPARATOR_KEY = "DataSeparator";


	/**
	 * Name used in file meta data line to indicate the comment character.
	 */
	public static final String META_COMMENT_CHAR_KEY = "CommentCharacter";


	/**
	 * Name used in file meta data line to indicate the meta data character.
	 */
	public static final String META_DATA_CHAR_KEY = "MetaDataCharacter";

	/**
	 * Default character used at begining of a line to indicate 
	 * a comment, which ends at the end of line.
	 */
	public static final char DEFAULT_COMMENT_CHAR = '#';

	/**
	 * Default character that indicates
	 * a line of metaData when it is the second character of a
	 * comment line.
	 */
	public static final char DEFAULT_METADATA_CHAR = '$';

	/**
	 * Default string that separates data block.
	 */
	public static final String DEFAULT_DATA_BLOCK_SEPARATOR = 
														"@DataBlockSeparator@";
	
	
	protected ChunkReader rawData;
	protected LabeledObject theData;
	
	private char commentChar = DEFAULT_COMMENT_CHAR;
	private char metaDataChar = DEFAULT_METADATA_CHAR;
	 
	private static final String lineSeparator = 
											System.getProperty("line.separator");
	private static final int EOF = -1;
	
	
	/**
	 * Create a StringizableReader object with a Reader as an input 
	 * and the 
	 * given separator. 
	 */
	public StringizableReader( Reader in  )   throws IOException
		{
		String separator = DEFAULT_DATA_BLOCK_SEPARATOR;
		PushbackReader dataStream = new PushbackReader( in, 2);
		
		CommentParser fileData = 
				new CommentParser( dataStream, commentChar, metaDataChar);

		fileData.readComment(  );
		LabeledData fileMetaData = fileData.readDataComment(  );
		
		separator = fileMetaData.getData( META_SEPARATOR_KEY , 
													DEFAULT_DATA_BLOCK_SEPARATOR);
		
		if ( fileMetaData.containsKey( META_COMMENT_CHAR_KEY ) )
			commentChar = fileMetaData.getData( META_COMMENT_CHAR_KEY ).charAt(0);
		if ( fileMetaData.containsKey( META_DATA_CHAR_KEY ) )
			metaDataChar = fileMetaData.getData( META_DATA_CHAR_KEY).charAt(0);


		BufferedReader markableIn = new BufferedReader( dataStream );
		rawData = new ChunkReader( markableIn, separator );
		}

	/**
	 * Create a StringizableReader object with a String as an input 
	 * and the 
	 * given separator.
	 */
	public StringizableReader( String input )   throws IOException
		{
		this( new StringReader( input) );
		}
	
	/**
	 * Create a StringizableReader object with a file as an input 
	 * and the 
	 * given separator. File IO will be buffered for you.
	 * @exception FileNotFoundException thrown if fileName is not 
	 * valid file name. 
	 */
	public static StringizableReader fromFile( String fileName )
		throws FileNotFoundException, IOException
		{
		FileReader inputFile = new FileReader(fileName);
		BufferedReader bufferedInput = 
				new BufferedReader( inputFile, 8000 );
				
		return new StringizableReader( bufferedInput);
		}
	
	public void setSeparator( String newSeparator )
		{
		rawData.setSeparator( newSeparator );
		}

	/**
	 * Return the comment for the current data object
	 */
	public String getDataComment()
		{
		return theData.getComment();
		}
	
	/**
	 * Returns the label for the current data object.
	 * Returns empty string, not null, if no label exists
	 */
	public String getDataLabel()
		{
		return theData.getLabel();
		}
	
    /**
	  * Returns true if there are more blocks of data in the input.
     */
    public boolean hasMoreElements()
    	{
		return rawData.hasMoreElements();
		}
	
	/**
	 * Returns the next block of data.
	 * @exception NoSuchElementException if there are no 
	 * 	blocks of data left.
	 */
	public Object nextElement() 
		{
		try
			{
			String dataBlock = (String) rawData.nextElement();
			theData = LabeledObject.fromString( dataBlock );
			return theData.getObject();
			}
		catch ( Exception error )
			{
			throw new NoSuchElementException( "Error in reading input: " +
				error.getMessage());
			}
		}
	}