/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import sdsu.util.*;
import java.io.*;

/**
 * Writes objects that implement the Stringizable inteface to 
 * a stream(or writer). The object is converted to a string
 * which can be used to recreate the object. A StringizableReader
 * can be used to recreate the objects saved by StringizableWriter.
 * <P><B>Note</B> while StringizableWriter uses the name Writer it
 * is <B>NOT</B> a subclass of Writer. The class acts like a Writer
 * but it does not make sense to allow arbitrary writes to a the stream,
 * as it will corrupt the object format.
 *
 * @see sdsu.util.Stringizable
 * @see sdsu.io.StringizableReader
 * @version 0.8 7 January 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class StringizableWriter
	{
	protected static final String META_SEPARATOR_KEY = 
		StringizableReader.META_SEPARATOR_KEY;

	protected static final String META_COMMENT_CHAR_KEY =
		StringizableReader.META_COMMENT_CHAR_KEY;

	protected static final String META_DATA_CHAR_KEY = 
		StringizableReader.META_DATA_CHAR_KEY;

	protected static final char DEFAULT_COMMENT_CHAR = 
		StringizableReader.DEFAULT_COMMENT_CHAR;

	protected static final char DEFAULT_METADATA_CHAR =
		StringizableReader.DEFAULT_METADATA_CHAR;

	protected static final String DEFAULT_DATA_BLOCK_SEPARATOR = 
		StringizableReader.DEFAULT_DATA_BLOCK_SEPARATOR;

	private static final String lineSeparator = 
		System.getProperty("line.separator");
	
	private String dataBlockSeparator = DEFAULT_DATA_BLOCK_SEPARATOR;
	private char commentChar = DEFAULT_COMMENT_CHAR;
	private char metaDataChar = DEFAULT_METADATA_CHAR;
	
	private ChunkWriter out;
	
	public StringizableWriter( Writer out )
		{
		this.out = new ChunkWriter( out, DEFAULT_DATA_BLOCK_SEPARATOR );
		}
	
	public StringizableWriter( Writer out, String objectSeparator )  
		throws IOException
		{
		this.out = new ChunkWriter( out, objectSeparator );
		dataBlockSeparator = objectSeparator;
		
		// Write out change from default
		// First line contain delta from defaults
		// Sample format #$ DataSeparator= end
		String metaComment = "" + commentChar + 
			"All lines starting with " + commentChar + metaDataChar +
			" are required to reconstruct objects, do not modify them.";
		String metaData = "" + commentChar + metaDataChar + " " +
			META_SEPARATOR_KEY + "=" + dataBlockSeparator;
		
		out.write( metaComment + lineSeparator + metaData );
		}
	
	/**
	 * Close the stream StringizableWriter is writing on.
	 */
	public void close() throws IOException
		{
		out.close();
		}
	
	/**
	 * Flush the stream StringizableWriter is writing on.
	 */
	public void flush() throws IOException
		{
		out.flush();
		}
	
	/**
	 * Write the object on the output stream. Object can be read by 
	 * StringizableReader.
	 */
	public void writeObject( Stringizable object ) throws IOException
		{
		writeLabeledObject( new LabeledObject( object));
		}

	/**
	 * Write the object on the output stream and attach the label
	 * to the object. Object and label can be read by 
	 * StringizableReader.
	 */
	public void writeObject( Stringizable object, String label ) 
		throws IOException
		{
		writeLabeledObject( new LabeledObject( object, label));
		}

	/**
	 * Write the object and attached label and comment on the output  
	 * stream. Object, label, and comment can be read by 
	 * StringizableReader.
	 */
	public void writeObject( Stringizable object, 
									String label, 
									String comment ) throws IOException
		{
		writeLabeledObject( new LabeledObject( object, label, comment));
		}
	
	/**
	 * Perform the actual write
	 */
	void writeLabeledObject( LabeledObject object ) throws IOException
		{
		out.writeChunk( object.toString() );
		} 
	}