/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.FilterReader;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.Reader;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import sdsu.util.*;

/**
 * Deprecated use StringizableReader.
 * @deprecated Use StringizableReader
 * @see sdsu.io.StringizableReader

 *
 * @version 0.5 31 August 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class StringizedDataEnumerator implements Enumeration
	{
	/**
	 * Name used in file meta data line to indicate the data separator string.
	 */
	public static final String META_SEPARATOR_KEY = "DataSeparator";


	/**
	 * Name used in file meta data line to indicate the comment character.
	 */
	public static final String META_COMMENT_CHAR_KEY = "CommentCharacter";


	/**
	 * Name used in file meta data line to indicate the meta data character.
	 */
	public static final String META_DATA_CHAR_KEY = "MetaDataCharacter";

	/**
	 * Default character used at begining of a line to indicate 
	 * a comment, which ends at the end of line.
	 */
	public static final char DEFAULT_COMMENT_CHAR = '#';

	/**
	 * Default character that indicates
	 * a line of metaData when it is the second character of a
	 * comment line.
	 */
	public static final char DEFAULT_METADATA_CHAR = '$';

	/**
	 * Default string that separates data block.
	 */
	public static final String DEFAULT_DATA_BLOCK_SEPARATOR = "@DataBlockSeparator@";
	
	
	protected ChunkEnumerator rawData;
	protected String metaDataComment;
	protected String dataComment;
	protected LabeledData metaData;
	protected Object data;
	
	private char commentChar = DEFAULT_COMMENT_CHAR;
	private char metaDataChar = DEFAULT_METADATA_CHAR;
	 
	private static final String lineSeparator = System.getProperty("line.separator");
	private static final int EOF = -1;
	
	
	/**
	 * Create a StringizedDataEnumerator object with a Reader as an input and the 
	 * given separator. This class does not buffer the reader for you.
	 */
	public StringizedDataEnumerator( Reader in  )   throws IOException
		{
		BufferedReader markableIn;
		String separator = DEFAULT_DATA_BLOCK_SEPARATOR;
		if ( in instanceof BufferedReader )
				markableIn = (BufferedReader) in;
		else
			markableIn = new BufferedReader( in );
	
		// Makes format a bit more flexible
		removeWhiteSpace( markableIn );
	
		// Process file metadata
		if ( startsWith( markableIn, commentChar, commentChar ) )
			{
			//flush metafile indicator characters
			markableIn.read();
			markableIn.read();
			
			LabeledData fileMetaData = new LabeledData();
			fileMetaData.fromString( markableIn.readLine() );
			
			separator = fileMetaData.getData( META_SEPARATOR_KEY , 
																	DEFAULT_DATA_BLOCK_SEPARATOR);
			
			if ( fileMetaData.containsKey( META_COMMENT_CHAR_KEY ) )
				commentChar = fileMetaData.getData( META_COMMENT_CHAR_KEY ).charAt(0);
			if ( fileMetaData.containsKey( META_DATA_CHAR_KEY ) )
				metaDataChar = fileMetaData.getData( META_DATA_CHAR_KEY).charAt(0);
			}
		
		//
		rawData = new ChunkEnumerator( markableIn, separator );
		}

	/**
	 * Create a StringizedDataEnumerator object with a String as an input and the 
	 * given separator.
	 */
	public StringizedDataEnumerator( String input )   throws IOException
		{
		this( new StringReader( input) );
		}
	
	/**
	 * Create a StringizedDataEnumerator object with a file as an input and the 
	 * given separator. File IO will be buffered for you.
	 * @exception FileNotFoundException thrown if fileName is not valid file name. 
	 */
	public static StringizedDataEnumerator fromFile( String fileName )
		throws FileNotFoundException, IOException
		{
		FileReader inputFile = new FileReader(fileName);
		BufferedReader bufferedInput = new BufferedReader( inputFile, 8000 );
		return new StringizedDataEnumerator( bufferedInput);
		}
	
	public void setSeparator( String newSeparator )
		{
		rawData.setSeparator( newSeparator );
		}

	/**
	 * Return the comment for the metaData of the current data object
	 */
	public String getMetaDataComment()
		{
		return metaDataComment;
		}

	/**
	 * Return the comment for the current data object
	 */
	public String getDataComment()
		{
		return dataComment;
		}
	
	/**
	 * Returns the label for the current data object.
	 * Returns empty string, not null, if no label exists
	 */
	public String getDataLabel()
		{
		if (metaData != null)
			return metaData.getData( "Label", "");
		else
			return "";
		}
	
    /**
	  * Returns true if there are more blocks of data in the input.
     */
    public boolean hasMoreElements()
    	{
		return rawData.hasMoreElements();
		}
	
	/**
	 * Returns the next block of data.
	 * @exception NoSuchElementException if there are no blocks of data left.
	 */
	public Object nextElement() 
		{
		try
			{
			String dataBlock = (String) rawData.nextElement();
			BufferedReader dataStream = new BufferedReader( new StringReader( dataBlock));

			// read comments, metaData and data  from input
			removeWhiteSpace( dataStream );
			metaDataComment = extractComment( dataStream);

			removeWhiteSpace( dataStream );
			metaData = extractMetaData( dataStream);

			removeWhiteSpace( dataStream );
			dataComment = extractComment( dataStream);

			String dataString = extractData( dataStream);
			
			// Create data object
			if ( !metaData.containsKey( "class" ))
				return dataString;
				
			String className = metaData.getData( "class");
			
			Stringizable data = (Stringizable) Class.forName( className ).newInstance();
			data.setMetaData( metaData );
			data.fromString( dataString );
			return data;
			}
		catch ( Exception error )
			{
			throw new NoSuchElementException( "Error in reading input: " +
				error.getMessage());
			}
		}
	
	/**
	 * Returns true if first two chars in dataStream are firstChar, secondChar
	 * Does not remove any chars from dataStream
	 */
	protected boolean startsWith( BufferedReader dataStream, char firstChar, char secondChar)
		 throws IOException
		{
		dataStream.mark( 2 );
		
		int peekOne = dataStream.read();
		
		if (peekOne == EOF )
			return false;
			
		int peekTwo = dataStream.read();
		dataStream.reset();
		
		if   ((peekOne == firstChar) && (peekTwo == secondChar ))
			return true;
		else
			return false;
		
		}


	/**
	 * Returns true if first  char in dataStream is firstChar
	 * Does not remove any chars from dataStream
	 */
	protected boolean startsWith( BufferedReader dataStream, char firstChar)  throws IOException
		{
		dataStream.mark( 1 );
		
		int peekOne = dataStream.read();
		
		if (peekOne == EOF )
			return false;
		dataStream.reset();
		
		if   (peekOne == firstChar)
			return true;
		else
			return false;
		}
		
	/**
	 * returns true if reader is at the start of a comment line.
	 * That is starts with # and second char is not $
	 */
	protected boolean startsCommentLine( BufferedReader dataStream ) throws IOException
		{
		return startsWith( dataStream, commentChar ) && !startsMetaDataLine(dataStream);
		}

	/**
	 * returns true if reader is at the start of a line containing metaData.
	 * That is starts with # and second char is $
	 */
	protected boolean startsMetaDataLine( BufferedReader dataStream ) throws IOException
		{
		return startsWith( dataStream, commentChar, metaDataChar);
		}
		
	/**
	 * Assumes dataStream is at the begining of a line.
	 * Returns all consecutinve lines of comments in dataStream
	 * Keeps comment chars in comments
	 * Leaves dataStream at the beginnig of a newline
	 */
	protected String extractComment( BufferedReader dataStream ) throws IOException
		{
		StringBuffer comments = new StringBuffer();
		while ( startsCommentLine( dataStream) )
			{
			comments.append( dataStream.readLine() );
			comments.append( lineSeparator );
			}
		if ( comments.length() >= lineSeparator.length() )
			comments.setLength( comments.length() - lineSeparator.length() );
		return comments.toString();
		}

	/**
	 * Assumes dataStream is at the begining of a line.
	 * Returns all consecutinve lines of metaData in dataStream
	 * Leaves dataStream at the beginnig of a newline
	 */
	protected LabeledData extractMetaData( BufferedReader dataStream ) throws IOException
		{
		LabeledData metaData = new LabeledData();
		StringBuffer metaDataChars = new StringBuffer();
		while ( startsMetaDataLine( dataStream) )
			{
			// flush chars indicating start of metaData line
			dataStream.read();
			dataStream.read();
			metaDataChars.append( dataStream.readLine() );
			}
		
		if ( metaDataChars.length() == 0 )
			return null;
		else
			{
			metaData.fromString( metaDataChars.toString() );
			return metaData;
			}
		}

	/**
	 * Returns the rest of the data in the dataStream
	 */
	protected String extractData( BufferedReader dataStream ) throws IOException
		{
		StringBuffer data = new StringBuffer();
		int nextChar = dataStream.read();
		
		while ( nextChar != EOF )
			{
			data.append( (char) nextChar);
			nextChar = dataStream.read();
			}
		return data.toString();
		}		
		
	/**
	 * Removes leading blanks, tabs, newlines from dataStream
	 */
	protected void removeWhiteSpace(  BufferedReader dataStream ) throws IOException
		{
		dataStream.mark( 1 );
		
		int peekOne = dataStream.read();
		
		while ( (peekOne == ' ') || (peekOne == '\t') || (peekOne == '\n') ||
					(peekOne == '\r'))
			{
			dataStream.mark( 1 );
			peekOne = dataStream.read();
			}

			
		if (peekOne == EOF )
			return;
			
		dataStream.reset();
		}

	}