/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.InputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.io.Reader;

/**
 * Given a substring (pattern), a replacement pattern and an input stream, this 
 * class will replace all occurances of the substring (pattern) with 
 * the replacement pattern in the inputstream and count the number of times
 * the substring occured in the input stream. Any occurance of the seqeunce "_@N" in the
 * replacement pattern will be replaced with number of occurances of the substring up to
 * that point in the stream. Useful for adding page numbers, line numbers, etc.
 * You can give multiple 
 * substring-replacement pattern pairs. Each pattern is counted separately.
 *
 * @version 0.5 31 August 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */


public class SubstringCountReader extends StringReplaceReader
	{
	/**
	 * Sequence of characters used to indicate a special symbol in replacement pattern
	 * strings.
	 */
	protected static final String MARKER = "_@";
	
	/**
	 * Sequence of characters indicating where the current count should
	 * appear in replacement pattern.
	 */
	public static final String COUNT_MARKER =  MARKER + "N";
	
	protected int count = 0;
	protected String patternWithMarkers;
		

	/**
	 * Create a SubstringCountReader on string in where the pattern substringToCount is 
	 * replaced with newPattern. The counting of the pattern starts at value initialCount.
	 */
	public SubstringCountReader( String in, String substringToCount, String newPattern,
									int initialCount )	
		{
		this( new StringReader(in) , substringToCount, newPattern, initialCount );
		}

	/**
	 * Create a SubstringCountReader on inputstream in where the pattern substringToCount is 
	 * replaced with newPattern. The counting of the pattern starts at value initialCount.
	 */
	public SubstringCountReader( InputStream in, String substringToCount, String newPattern,
									int initialCount )	
		{
		this( new BufferedReader(new InputStreamReader(in)) , substringToCount, newPattern, initialCount );
		}

	/**
	 * Create a SubstringCountReader on Reader in where the pattern substringToCount is 
	 * replaced with newPattern. The counting of the pattern starts at value initialCount.
	 */
	public SubstringCountReader( Reader in, String substringToCount, String newPattern,
									int initialCount )	
		{
		super( in , substringToCount, newPattern );
		patternWithMarkers = newPattern;
		count = initialCount;
		}

	/**
	 * Create a SubstringCountReader on Reader in where the pattern substringToCount is 
	 * replaced with newPattern. The counting of the pattern starts at value 1.
	 */
	public SubstringCountReader( Reader in, String substringToCount, String newPattern )	
		{
		this(in, substringToCount, newPattern, 1);
		}
	
	/**
	 * Returns the current count of the number of times the pattern has
	 * occured.
	 */
	public int getCount()
		{
		return count;
		}
	
	/**
	 * This is called by super class when pattern is found in input
	 */
	protected void replacementPatternToBuffer()
		{
		super.replacementPattern = patternWithCount().toCharArray();
		super.replacementPatternToBuffer();
		count++;
		}

	/**
	 * Replace COUNT_MARKER in replacement pattern with current count.
	 */
	protected String patternWithCount()
		{
		try
			{
			StringReplaceReader replace;
			replace = new StringReplaceReader( patternWithMarkers, COUNT_MARKER, String.valueOf( count ) );
			return replace.contents();
			}
		catch ( IOException ioError )
			{
			return patternWithMarkers;
			}
			
		}
		
	}
