/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.*;

/**
 * XorInputStream is an inputstream that Xor all input bytes
 * with the byte mask given in the constructor. Use to decode
 * output of a XorOutputStream by giving it the XorInputStream
 * the same byte mask as XorOutputStream. XorInputStream and 
 * XorOutputStream can be used for a very weak form of encryption.
 *
 * @version 1.0 14 Feb 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class XorInputStream extends FilterInputStream
	{
	byte codeMask = 0;
	
	
	/**
	 * Create an XorInputStream on the given input stream.
	 * @param mask the bit pattern with which all input bytes
	 *  will be Xored
	 */ 
	public XorInputStream(InputStream in, byte mask) 
		{
		super(in);
		codeMask = mask;
		}

	/**
	 * Reads the next byte of data and Xors it with the mask
	 * given in the constructor. If no byte is available 
	 * because the end of the stream has been reached, the value 
	 * <code>-1</code> is returned. This method blocks until input data 
	 * is available, the end of the stream is detected, or an exception 
	 * is thrown. 
	 *
	 * @return	 the next byte of data, or <code>-1</code> if the end of the
	 *			 stream is reached.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public int read() throws IOException 
		{
		int input = in.read();
		if ( input < 0 )
			return input;
		else
			return (byte) input ^ codeMask;
		}


	/**
	 * Reads up to <code>length</code> bytes of data and Xor each byte. 
	 * This method blocks until some input is 
	 * available. 
	 *
	 * @param	  inputBuffer	 the buffer into which the data is read.
	 * @param	  offset   the start offset of the data.
	 * @param	  length   the maximum number of bytes read.
	 * @return	 the total number of bytes read into the buffer, or
	 *			 <code>-1</code> if there is no more data because the end of
	 *			 the stream has been reached.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public int read(byte inputBuffer[], int offset, int length) throws IOException 
		{
		byte[] codedInput = new byte[length];
		int bytesRead = in.read( codedInput );
		
		for (int k = 0; k < bytesRead;k++)
			inputBuffer[offset + k ] = (byte) (codedInput[k] ^ codeMask);
		return bytesRead;
		}

	}
