/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.*;

/**
 * XorOutputStream encodes it output by Xoring all input bytes
 * with the byte mask given in the constructor. 
 * Use a XorInputStream to decode the output of XorOutputStream
 * by givening them both the same mask in their constructors.
 * XorInputStream and 
 * XorOutputStream can be used for a very weak form of encryption.
 *
 * @version 1.0 14 Feb 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class XorOutputStream extends FilterOutputStream
	{
	byte codeMask = 0;
	
	public XorOutputStream(OutputStream out, byte mask) 
		{
		super(out);
		codeMask = mask;
		}

	/**
	 * Writes the specified <code>byte</code> and Xor's it with the
	 * streams mask. 
	 *
	 * @param	  output   the <code>byte</code>.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public void write(int output) throws IOException 
		{
		if ( output < 0 ) //not a byte, 
			{
			out.write(output);
			}
		else
			out.write( ((byte) output) ^ codeMask );
		}

	/**
	 * Writes <code>length</code> bytes from the specified 
	 * <code>byte</code> array starting at offset <code>offset</code> 
	 * and Xor's each byte with the streams mask. 
	 *
	 * @param	  output	 the data.
	 * @param	  offset   the start offset in the data.
	 * @param	  length   the number of bytes to write.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public void write(byte output[], int offset, int length) throws IOException 
		{
		byte[] coded = new byte[length];
		for (int i = 0 ; i < length ; i++) 
			{
			coded[i] = (byte) ( output[offset + i] ^ (byte) codeMask );
			}
		out.write( coded);
		}

	}
