/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.*;

/**
 * XorReader is an inputstream that Xor all input bytes
 * with the byte mask given in the constructor. Use to decode
 * output of a XorWriter by giving it the XorReader
 * the same byte mask as XorWriter. XorReader and 
 * XorWriter can be used for a very weak form of encryption.
 *
 * @version 1.0 14 Feb 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class XorReader extends FilterReader
	{
	char codeMask = 0;
	
	/**
	 * Create an XorReader on the given input stream.
	 * @param mask the bit pattern with which all input bytes
	 *  will be Xored
	 */ 
	public XorReader(Reader in, char mask) 
		{
		super(in);
		codeMask = mask;
		}

	/**
	 * Reads the next char of data and Xors it with the mask
	 * given in the constructor. If no char is available 
	 * because the end of the stream has been reached, the value 
	 * <code>-1</code> is returned. This method blocks until input data 
	 * is available, the end of the stream is detected, or an exception 
	 * is thrown. 
	 *
	 * @return	 the next char of data, or <code>-1</code> if the end of the
	 *			 stream is reached.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public int read() throws IOException 
		{
		int input = in.read();
		if ( input < 0 )
			return input;
		else
			return (char) input ^ codeMask;
		}


	/**
	 * Reads up to <code>length</code> chars of data and Xor each char. 
	 * This method blocks until some input is 
	 * available. 
	 *
	 * @param	  inputBuffer	 the buffer into which the data is read.
	 * @param	  offset   the start offset of the data.
	 * @param	  length   the maximum number of chars read.
	 * @return	 the total number of chars read into the buffer, or
	 *			 <code>-1</code> if there is no more data because the end of
	 *			 the stream has been reached.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public int read(char inputBuffer[], int offset, int length) throws IOException 
		{
		char[] input = new char[length];
		int charsRead = in.read( input );
		
		for (int k = 0; k < charsRead;k++)
			inputBuffer[offset + k ] = (char) (input[k] ^ codeMask);
		return charsRead;
		}

	}
