/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.*;

/**
 * XorWriter encodes it output by Xoring all input chars
 * with the char mask given in the constructor. 
 * Use a XorReader to decode the output of XorWriter
 * by givening them both the same mask in their constructors.
 * XorReader and 
 * XorWriter can be used for a very weak form of encryption.
 *
 * @version 1.0 14 Feb 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class XorWriter extends FilterWriter
	{
	char codeMask = 0;
	
	public XorWriter(Writer out, char mask) 
		{
		super(out);
		codeMask = mask;
		}

	/**
	 * Writes the specified <code>char</code> and Xor's it with the
	 * stream's mask. 
	 *
	 * @param	  output   the <code>char</code>.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public void write(int output) throws IOException 
		{
		if ( output < 0 ) //not a char, 
			{
			out.write(output);
			}
		else
			out.write( ((char) output) ^ codeMask );
		}

	/**
	 * Writes <code>length</code> chars from the specified 
	 * <code>char</code> array starting at offset <code>offset</code> 
	 * and Xor's each char with the stream's mask. 
	 *
	 * @param	  output	 the data.
	 * @param	  offset   the start offset in the data.
	 * @param	  length   the number of chars to write.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public void write(char output[], int offset, int length) throws IOException 
		{
		char[] coded = new char[length];
		for (int i = 0 ; i < length ; i++) 
			{
			coded[i] = (char) ( output[offset + i] ^ (char) codeMask );
			}
		out.write( coded);
		}

	/**
	 * Writes <code>length</code> chars from the specified 
	 * <code>string</code> starting at offset <code>offset</code> 
	 * and Xor's each char with the stream's mask. 
	 *
	 * @param	  output	 the data.
	 * @param	  offset   the start offset in the data.
	 * @param	  length   the number of chars to write.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public void write(String output, int offset, int length) throws IOException 
		{
		write(output.toCharArray(), offset, length);
		}

	}
