/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.logging;

import java.util.Vector;
import java.util.Enumeration;

/**
 * This class is to be used in debuging programs. It has several 
 * advantages over using System.out. First, the debug statements 
 * can be turn off globally. This means you can leave the debug
 * statments in the program, without them producing output. Second,
 * each debug statement prints the line number where is was called from.
 * Third, the debug output can be directed to the screen and/or a file.
 * Fourth, the underlying stream is flushed after each println to 
 * that insure the output is processed before a program crashes.
 * Debug output can be sent to the screen or a file. Default action
 * is to send output to the screen.
 *
 * <P> To use the Debug class, just use the statement:
 *<pre>
 *		Debug.println( "Your debug message here");
 * </pre>
 * where you wish to print out debug messages.
 *
 * @version 1.0 18 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
//34567890123456789012345678901234567890123456789012345678901234567890
public class Debug
	{
	static LoggerImplementation screen = new ScreenLogger();
	static LoggerImplementation file;
	static Vector activeLoggers = new Vector();
	static boolean isDebugOn = true;
	
	static 
		{
		activeLoggers.addElement( screen );
		}
	
	/**
	 * Turn on Debug messages. This means that
	 * calls to println() will be processed.
	 */
	public static void on()
		{
		isDebugOn = true;
		}

	/**
	 * Turn off Debug messages. This means that 
	 * calls to println() will not be processed.
	 */
	public static void off()
		{
		isDebugOn = false;
		}

	/**
	 * Turn output to the screen off. Since output
	 * is sent to the screen by default, there is no
	 * screenOn().
	 */		
	public static void screenOff()
		{
		activeLoggers.removeElement( screen );
		}

	
	/**
	 * Send arguments of println() to the file listed.
	 * If the file can not be opened, a message will
	 * be sent to the screen (standard out). No exceptions are thrown.
	 * @param fileName the file to be used to write
	 * 	the logging messages.
	 * @param append	if true then the file is opened in
	 *		append mode, if false overwrite the file.
	 */		
	public static void fileOn( String fileName, boolean append )
		throws Exception
		{
		try
			{
			file = new FileLogger( fileName, append );
			activeLoggers.addElement( file );
			}
		catch (Exception error )
			{
			DebugMessage errorMessage = new DebugMessage(
				 "The following error occured" +
				" trying to open the debug file: " + fileName +
				" " + error.getMessage(), 1);
				
			screen.log( errorMessage );
			}
		}

	/**
	 * Print the given message.
	 */
	public static void println( String message )
		{
		logMessage( new DebugMessage( message, 1 ));
		}

	/**
	 * Print the given message.
	 */
	public static void println( Object message )
		{
		logMessage( new DebugMessage( message.toString(), 1 ));
		}

	/**
	 * Print the given message.
	 */
	public static void println( int message )
		{
		logMessage( new DebugMessage( "" + message, 1 ));
		}
	
	/**
	 * Print the given message.
	 */
	public static void println( char message )
		{
		logMessage( new DebugMessage( "" + message, 1 ));
		}

	/**
	 * Print the given message.
	 */
	public static void println( char[] message )
		{
		logMessage( new DebugMessage( "" + message, 1 ));
		}

	/**
	 * Print the given message.
	 */
	public static void println( boolean message )
		{
		logMessage( new DebugMessage( "" + message, 1 ));
		}

	/**
	 * Print the given message.
	 */
	public static void println( long message )
		{
		logMessage( new DebugMessage( "" + message, 1 ));
		}

	/**
	 * Print the given message.
	 */
	public static void println( float message )
		{
		logMessage( new DebugMessage( "" + message, 1 ));
		}

	/**
	 * Print the given message.
	 */
	public static void println( double message )
		{
		logMessage( new DebugMessage( "" + message, 1 ));
		}

	private static void logMessage(LoggerMessage aMessage)
		{
		if (isDebugOn )
			{
			Enumeration loggers = activeLoggers.elements();
			while (loggers.hasMoreElements() )
				{
				LoggerImplementation aLog = 
						(LoggerImplementation) loggers.nextElement();
				aLog.log( aMessage );
				}
			}
		}
	}
