/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.logging;

import java.io.StringWriter;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.StringReader;
import java.io.IOException;

/**
 * DebugMessage is used by sdsu.logging.Logger. DebugMessage adds
 * the name of the method  being debuged and the location in the
 * method of the debug statement. 
 *
 * @see sdsu.logging.LogMessage
 * @see sdsu.logging.Logger
 * @version 1.01 30 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class DebugMessage extends LoggerMessage
	{
	
	public DebugMessage( String message )
		{
		super( callersReference( 0 ) + ": " + message);
		}

	/**
	 * Create a new DebugMessage. 
	 * @param callersDistance number of method calls between 
	 *  constructor and code we are debugging.
	 */
	DebugMessage( String message, int callersDistance )
		{
		super( callersReference( callersDistance ) + ": " + message);
		}
	
	public String getMessageType()
		{
		return "Debug";
		}

	/**
	 * Returns the method name and location in source code of the
	 * location where the Debug request was made.
	 * @param	callersDistance the number of method calls from the 
	 *   DebugMessage constructer is from location in code we are trying
	 *   to Debug. So if one creates the Debug message directly in the 
	 *   debug code, this should be zero.
	 */ 
	static String callersReference(int callersDistance)
		{
		// Get a copy of the execution stack
		Throwable stackTrace = new Throwable();
		StringWriter stackOutput = new StringWriter();
		PrintWriter out = new PrintWriter( stackOutput );
		stackTrace.printStackTrace( out );
		out.flush();
		String stackString = stackOutput.toString();

		//Parse for the line of the execution stack of interest
		// method name and source code location in code trying to
		// debug
		
		// callersDistance + 2 => 1 for constuctor, one for current method
		String callersLine = getLine(stackString, callersDistance + 2);
		return stripPackageName( callersLine );
		}
	
	/**
	 * Removes package part of class name from stackLine
	 * reported for location in debug code.
	 */
	static String stripPackageName( String stackLine )
		{
		stackLine = stackLine.trim();
		
		// remove the at
		if (stackLine.startsWith( "at" ))
			stackLine = stackLine.substring( 2);
		stackLine = stackLine.trim();
		
		// Without the package name in error message, there will be 
		// 2 periods. Remove all stackLine that occurs before the last two
		// periods
		int periodCount = occurencesOf( stackLine, '.' );
		if ( periodCount > 2)
			stackLine = removeLeadingText( stackLine, '.', periodCount - 2 );
		return stackLine;
		}
	
	/**
	 * Removes all chars in text up through the count'th occurence
	 * of the char separator
	 */ 
	static String removeLeadingText( String text, char separator, int count)
		{
		//Assert.precondition( occurencesOf( text, separator) >= count );
		
		int searchIndex = 0;
		for ( int k = 0; k < count; k++ )
			searchIndex = text.indexOf( separator, searchIndex + 1);
		
		return text.substring( searchIndex + 1 );	
		}
	
	/**
	 * Returns the number of times seachChar occurs in text
	 */ 
	static int occurencesOf( String text, int searchChar )
		{
		int count = 0;
		int searchIndex = 0;
		while ( (searchIndex < text.length()) && (searchIndex > -1 ) )
			{
			searchIndex = text.indexOf( searchChar, searchIndex + 1);
			count++;
			}
		if (searchIndex == -1 )
			count--;
		return count;
		}
	
	static boolean contains( String text, String pattern )
		{
		int location = text.indexOf( pattern);
		if (location > -1 )
			return true;
		else
			return false;
		}
		
	/**
	 * Returns given line of text 
	 */
	static String getLine( String text, int lineNumberRequested )
		{
		//Assert.precondition( lineNumberRequested >= 0 );
		
		BufferedReader in = new BufferedReader( new StringReader( text));
		
		String currentLine = "";
		
		String referenceLine = "sdsu.logging.DebugMessage.callersReference";
		// lineNumberrequested must be found from call to callersReferece as
		// some versions of java add extra calls on top of stack.
		try
			{
			do
				{
				currentLine = in.readLine();
				
				// readLine returns null on EOF
				if (currentLine == null )  return ""; 
				
				}
			while ( !contains(currentLine, referenceLine ));
				
			for (int line = 0; line < lineNumberRequested; line++)
				currentLine = in.readLine();
			}
		catch ( IOException error)
			{
			currentLine = "";
			}
		if (currentLine == null )  
			return "";
		else 
			return currentLine;
		}
	}
