/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.logging;

import java.io.Writer;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import sdsu.test.Assert;

 /**
 * This logger directs all log messages to a file.
 * As a SelectiveLogger, message types can be turned 
 * off.
 *
 * @see sdsu.logging.Logger
 * @see sdsu.logging.SelectiveLogger
 * @version 1.1 18 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class FileLogger extends SelectiveLogger
	{
	PrintWriter out;
	String logFileName;
	
	/**
	 * Create a FileLogger in append mode on the file indicated
	 * and register it with Logger.
	 * @param fileName ".log" will be appended to fileName if it
	 *		does not already end in ".log"
	 * Returns the current active logger. If a logger has
	 * previously been set, it will be that logger.
	 */
	public static LoggerImplementation register( String fileName ) 
		throws LoggerCreationException
		{
		if ( !Logger.hasLogger() )
			{
			FileLogger instance = new FileLogger( fileName, true ); 
			Logger.register( instance );
			}
		return Logger.getLogger();
		}

	/**
	 * Create a FileLogger on the file indicated
	 * and register it with Logger.
	 * @param append if true open file in append mode, if false
	 * 	overwrite file
	 * @param fileName ".log" will be appended to fileName if it
	 *		does not already end in ".log"
	 */
	public static FileLogger register( String fileName, boolean append ) 
		throws LoggerCreationException
		{
		FileLogger instance = new FileLogger( fileName, append ); 
		Logger.register( instance );
		return instance;
		}

	FileLogger( String fileName, boolean append) 
		throws LoggerCreationException
		{
		Assert.precondition( (fileName != null) && (fileName.length() > 0));
		
		//Insure that Finalize is run to flush and close the file
		System.runFinalizersOnExit( true );
		
		if ( !fileName.endsWith( ".log") )
			fileName = fileName + ".log";
			
		logFileName = fileName;
		try
			{
			out = new PrintWriter( 
						new BufferedWriter( 
							new FileWriter( fileName, append) 
						)
					);
			}
		catch (IOException fileError )
			{
			String errorMessage = "Trouble opening log file:" + fileName;
			throw new LoggerCreationException( errorMessage);
			}
		}

	/**
	 * Resets logger. Erases any permanment storage of the log.
	 * This is to allow students to reset their log file. Not 
	 * normally part of a logging system.
	 */
	public synchronized void reset( )
		{
		out.flush();
		out.close();

		try
			{
			out = new PrintWriter( 
						new BufferedWriter( 
							new FileWriter( logFileName, false) 
						)
					);
			}
		catch (IOException fileError )
			{
			//This a problem what to do here?
			out = new PrintWriter( System.err );
			out.println(
				 "Error in reseting file " + fileError.getMessage() );
			}		
		}

	/**
	 * Insure IO buffers are flushed when FileLogger object dies
	 */
	protected void finalize() throws Throwable
		{
		super.finalize();
		out.flush();
		out.close();
		}
		

	/**
	 * Add the LoggerMessage object to the log. 
	 */
	protected synchronized void processLogRequest( LoggerMessage message )
		{
		out.println( TimeStamp.now() + 
							message.logString() );
		out.flush();
		}

	}