/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.logging;

import java.util.Properties;

 /**
 * This class is the interface for using the logging system in 
 * sdsu.logging. The intent is to provide a static interface to
 * the logging system, so that exact same lines of code are used
 * to log messages. This insures that you can change the type of
 * logger used in a program by changing just one line of code.<P>
 * Other classes (ScreenLogger, FileLogger) actually implement 
 * the logging system. To use the Logger the actual logger must be 
 * registered. This is done using the register method in the 
 * logger class you wish to use. So to use the FileLogger first
 * register it via:
 * <pre>
 *		FileLogger.register( LogFileName );
 * </pre>
 * A FileLogger object that writes to the given file
 * is now registered with the Logger class.
 * Then in your program use the static methods of this class to 
 * add messages to the log. For example:
 * <pre>
 * 	Logger.debug( "Value of x: " + x );
 * </pre>
 * will add a debug message to the log. <B>Note</B> you can only
 * register one logger per program. Also, if you do not register
 * a logger the ScreenLogger will be used.
 * <P>Four types of
 * messages are defined: debug, warning, error, and log. Debug is for
 * recording debug statements, log for logging routine usage of the 
 * program, warnings are for problems in usage (like 
 * someone trying to break into a server), errors are for logging
 * exceptions and errors that occur when the program is run. More
 * types of messages can be created by subclassing 
 * sdsu.logging.LoggerMessage. 
 * <P>
 * Some logger implementations of this 
 * class allow the programmer to turn off logging of one or more 
 * types of messages while still logging the other types. Once a 
 * program is debugged, you can turn off debug messages, but leave
 * them in the program for future use. This does incurr a slight
 * performance penalty, but this should not be a problem for most
 * student programs. These loggers must be accessed directly 
 * to turn logging message off or on. This direct access is done
 * using the register method. For example:
 *<pre>
 *		FileLogger myLog = FileLogger.register( "LogFile");
 *		myLog.debugOff();
 *		myLog.warningOff();
 * </pre>
 * will register the FileLogger and turn off debug and warning 
 * statements. This means the statment:
 * <pre>
 * 	Logger.debug( "Value of x: " + x );
 * </pre>
 * will have no effect later in the program.
 * <P>
 * The logger stores the active logger in a static field.
 * To help insure that this class is not unloaded during 
 * garbage collection an reference to the Logger class is stored
 * in the System properties under the key:
 * <I>_$sdsu.logging.Logger.instance</I>. If you remove this reference
 * by replacing this or all system properties, you need to 
 * store a reference to Logger in your program or turn off 
 * unloading of classes.
 *
 * <P> The logging system is based on the logging pattern by 
 * Neil Harrison in <I>Pattern Languages of Program Design 3</I>
 * Eds Martin, Riehle, Buschman, 1998, pp 277-289.
 *
 * @see sdsu.logging.ScreenLogger
 * @see sdsu.logging.FileLogger
 * @see sdsu.logging.NullLogger
 * @version 1.1 18 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class Logger
	{
	
	// insure that there is always an active logger
	private static boolean hasBeenSet = false;
	private static LoggerImplementation activeLogger = new ScreenLogger();
	
	/**
	 * Insure Logger class in not garbage collected
	 * by creating system reference to logger class
	 */
	private static void preventClassGarbageCollection()
		{
		Properties systemTable = System.getProperties();
		systemTable.put( "_$sdsu.logging.Logger.instance",
							 new Logger());
		}
		
	/**
	 * Sets the active logger. Used by logger implementations
	 * to register them selves with the Logger.
	 */
	public static void register( LoggerImplementation aLogger)
		{
		if ( ! hasBeenSet )
			{
			activeLogger = aLogger;
			hasBeenSet = true;
			preventClassGarbageCollection();
			}
		}
	
	/**
	 * Returns true if a logger has been registered
	 */
	public static boolean hasLogger()
		{
		return hasBeenSet;
		}

	/**
	 * Returns the active logger. 
	 */	
	public static LoggerImplementation getLogger()
		{
		return activeLogger;
		}
		
	/**
	 * Add the message to the log as a debug statement.
	 */
	public static void debug( String message)
		{
		log( new DebugMessage( message, 1 ) );
		}
		
	/**
	 * Add the message to the log as an error statement.
	 */
	public static void error( String message)
		{
		log( new ErrorMessage( message ) );
		}

	/**
	 * Add the message to the log as a debug statement.
	 */
	public static void error( Exception error)
		{
		log( new ErrorMessage( error ));
		}
 
	/**
	 * Add the message to the log as a warning statement.
	 */
	public static void warning( String message )
		{
		log( new WarningMessage( message ) );
		}

	/**
	 * Add the message to the log as a log statement.
	 */
	public static void log( String message )
		{
		log( new LogMessage( message ) );
		}

	/**
	 * Add the LoggerMessage object to the log.
	 */
	public static void log( LoggerMessage message )
		{
		activeLogger.log( message );
		}

	/**
	 * Resets logger. Erases any permanment storage of the log.
	 * This is to allow students to reset their log file. Not 
	 * normally part of a logging system.
	 */
	public static void reset( )
		{
		activeLogger.reset();
		}

	}