/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.logging;

 /**
 * This is the abstract class for all logging classes in sdsu.logging. 
 * It defines a system for logging usage of programs. The only reason
 * to use this class is to create a new logger implementation. To create a new
 * logger implementation you need to: implement the public static method 
 * register() and the protected method processLogRequest( LoggerMessage ).
 * The register method needs to create an instance of the logger, 
 * register it with the class Logger by calling Logger.register(),
 * and return the logger instance. The processLogRequest method 
 * handles the actual processing of the message. You can overload
 * this method based on the type of message. Note that Logger calls
 * the method log( LoggerMessage ), which calls processLogRequest. 
 * This level of indirection is used by SelectiveLogger to filter
 * log requests before they get  to the actual logger.

 * <P> The logging system is based on the logging pattern by 
 * Neil Harrison in <I>Pattern Languages of Program Design 3</I>
 * Eds Martin, Riehle, Buschman, 1998, pp 277-289.
 *
 * @see sdsu.logging.LogMessage
 * @see sdsu.logging.Logger;
 * @see sdsu.logging.SelectiveLogger;
 * @see sdsu.logging.ScreenLogger;
 * @version 1.0 18 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public abstract class LoggerImplementation
	{
	
	/**
	 * All subclass must implement this method. The method should 
	 * create an instance, register it with the Logger class
	 * by using Logger.register( ). <B>Note</B> the compiler
	 * does not force you to implement this method, but still
	 * do so.
	 */
	public static LoggerImplementation register( )
		{
		return null;
		}
	/**
	 * Add the LoggerMessage object to the log.
	 */
	public void log( LoggerMessage message )
		{
		processLogRequest( message);
		}

	/**
	 * Resets logger. Suclasses define what this means.
	 * Not normally part of a logging system. This will allow
	 * log servers to rest log files for students that may not
	 * have direct access to the log file.
	 */
	public void reset( )
		{
		}

	/**
	 * Method used by subclasses to actually log the message.
	 */
	protected abstract void processLogRequest( LoggerMessage message);
	}