/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.logging;

import java.io.Serializable;
import sdsu.util.Stringizable;
import sdsu.util.LabeledData;
import sdsu.util.ConversionException;
import sdsu.util.TokenCharacters;
import sdsu.util.Stringizer;

/**
 * LoggerMessage and its subclasses are used by sdsu.logging.Logger. 
 * You only need to use this class directly when you are adding
 * a new type of message to the logger. To create a new message type
 * create a new subclass of this class. Pass the actual text of the 
 * message as an argument to the constructor of this class. Subclasses must
 * implement the method getMessageType(). You also
 * may wish to override the displayString saveString methods
 * if you wish a different
 * display output for your new message type. 
 * <P> The logging system is based on the logging pattern by 
 * Neil Harrison in <I>Pattern Languages of Program Design 3</I>
 * Eds Martin, Riehle, Buschman, 1998, pp 277-289
 *
 * @see sdsu.logging.Logger
 * @see sdsu.logging.DebugMessage
 * @version 1.0 3 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public abstract class LoggerMessage 
	implements Stringizable, Serializable
	{
	
	// The following are used in converting message to a 
	// string containing key-value pairs
	protected static final String PAIR_SEPARATOR = ";";
	protected static final String KEY_VALUE_SEPARATOR = "=";
	protected static final TokenCharacters parseTable = 
		new TokenCharacters(PAIR_SEPARATOR + KEY_VALUE_SEPARATOR);

	
	protected static final String MESSAGE_KEY = "message";
	protected static final String CLASS_KEY = "class";
	protected static final String TYPE_KEY = "type";
	
	
	String message;

	public LoggerMessage( String message )
		{
		this.message = message;
		}
		
	/**
	 * Returns the type of the message.
	 */
	public abstract String getMessageType();
		
	/**
	 * Converts a string to an object. Used in sending message
	 * to remote loggers.
	 * @exception ConversionException If there is a problem 
	 *    converting to object.
	 */
	public void fromString( String objectString  ) 
		throws ConversionException
		{
		LabeledData data = new LabeledData();
		data.fromString( objectString );
		
		if (! data.containsKey( MESSAGE_KEY ) )
			throw new ConversionException( "Error in converting string" +
				" to Message: Missing message Key");

		message = data.getData( MESSAGE_KEY );
		}
	
	/**
	 * Required by Stringizable. Messages does not require
	 * metadata internally.
	 */		
	public void setMetaData( LabeledData metaData )
		{
		}
	
	/**
	 * MetaData for Message contains key-value pair
	 * "class=objectClassName".
	 * Used to recreate proper message object by remote loggers.
	 */
	public LabeledData getMetaData( )
		{
		LabeledData metaData = new LabeledData();
		metaData.put( CLASS_KEY, this.getClass().getName());
		return metaData;
		}
	
	/**
	 * Used in sending message to remote loggers.
	 */
	public String toString( )
		{
		return toString( null );
		}

	/**
	 * Converts the object to a string with given header information.
	 * Used in sending message to remote loggers.
	 */
	public String toString( String header )
		{
		LabeledData data = new LabeledData();
		data.put( MESSAGE_KEY, message);
		return data.toString( header );
		}

	/**
	 * Displays a string to be displayed on the screen for this message.
	 */
	public String displayString()
		{
		return getMessageType() + " " + message;
		}
		
	/**
	 * Displays a string to be saved in a log for this message.
	 */
	public String logString()
		{
		Stringizer buffer = new Stringizer( LoggerMessage.parseTable );

		buffer.appendToken( TYPE_KEY, KEY_VALUE_SEPARATOR );
		buffer.appendToken( getMessageType(), PAIR_SEPARATOR );
		buffer.appendToken( MESSAGE_KEY, KEY_VALUE_SEPARATOR );
		buffer.appendToken( message, PAIR_SEPARATOR );

		return buffer.toString() + PAIR_SEPARATOR;
		}

	}