/* 
 * Copyright (C) 1996 Andrew Scherpbier <andrew@sdsu.edu>, 1997 Roger Whitney
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.net;

import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.PrintStream;

/**
 * This class is an interface to the Common Gateway Interface protocol used
 * by most Unix based HTTP servers.
 * It will take the values of an HTML form
 * and make them available to the program through the getFormData() method.
 * Supports data submitted via via http post,
 * http put, http get and http isindex. Don't support all isindex queries.  
 * Support queries with 
 * path?name=value
 * pairs, but not path?a+b+c <p>
 *
 * By itself, a java program cannot be used as a CGI program for two reasons:
 * <ol><li>The java program is interpreted by the java interpeter so it
 * cannot be executed as a single program without arguments.</li>
 * <li>The CGI protocol uses environment variables to pass parameters from
 * the HTTP server to the CGI program.  Java doesn't have access to the
 * environment variables.</li></ol>
 * For this reason, a companion program, <a href="jcgi.html">jcgi</a>,
 * needs to be used.<p>
 *
 * The parsing of the CGI protocol stuff is done by the constructor of this
 * class.<p>
 *
 * If this class is used in an interactive (testing) environment, requests
 * for values will cause a interactive prompts.  Once the value has been
 * entered once, it is not asked for again.  This is useful for debugging
 * purposes.
 *
 * @author <a href="http://www.sdsu.edu/~turtle">Andrew Scherpbier</a>
 * @author <a href="http://www.eli.sdsu.edu/">Roger Whitney</a> made some modifications to 
 * Andrew's code
 */
public class CGI
{
	private Properties formData;
	private Properties cgiData;
	private PrintStream out = null;

	private static CGI instance = new CGI();
	
	/**
	 * Returns a CGI object
	 */
	public static CGI getInstance()
		{
		return instance;
		}
	
	/**
	 * This constructor will parse the CGI protocol information.
	 */
	private CGI()
		{
		String		results = null;
		String		method = System.getProperty("REQUEST_METHOD");

		if (method == null || method.length() == 0)
			{
			// Program was not called by web server, assume called directly in test mode
			formData = new sdsu.test.InteractiveProperties();
			cgiData = new sdsu.test.InteractiveProperties();
			return;
			}
			
		cgiData = System.getProperties();
		
		if (method.equalsIgnoreCase("get"))
			{
			results = cgiData.getProperty("QUERY_STRING", "");
			}
		else
			{
			// Assume POST or PUT method.
			// The nvp data needs to be read from STDIN.
			//
			results = readCGIStandardIn();
			}
		formData = tokenizeFormData( results );
		}

	/**
	 * Returns a hashtable containing name-value pairs of the data sent via http post,
	 * http put, http get and http isindex. All name and values are decoded into
	 * normal strings. If class is used in an interactive (testing) environment, the
	 * hashtable returned will prompt for the value associated a key the first time the 
	 * key is accessed.
	 */
	public Properties getFormData()
		{
		return formData;
		}

	/**
	 * Returns a hashtable containing name-value pairs of the cgi environment variables.
	 * The keys are the following strings:
	 *	<TABLE BORDER>
	 *	<TR VALIGN=top><TD>AUTH_TYPE</TD><TD>HTTP_ACCEPT</TD><TD>REMOTE_ADDR</TD><TD>SCRIPT_NAME</TD></TR>
	 *	<TR VALIGN=top><TD>CONTENT_LENGTH</TD><TD>HTTP_USER_AGENT</TD><TD>REMOTE_HOST</TD><TD>SERVER_NAME</TD></TR>
	 *	<TR VALIGN=top><TD>CONTENT_TYPE</TD><TD>PATH_INFO</TD><TD>REMOTE_IDENT</TD><TD>SERVER_PORT</TD></TR>
	 *	<TR VALIGN=top><TD>DOCUMENT_ROOT</TD><TD>PATH_TRANSLATED</TD><TD>REMOTE_USER</TD><TD>SERVER_PROTOCOL</TD></TR>
	 *	<TR VALIGN=top><TD>GATEWAY_INTERFACE</TD><TD>QUERY_STRING</TD><TD>REQUEST_METHOD</TD><TD>SERVER_SOFTWARE</TD></TR>
	 *	</TABLE>
	 * See <A href="http://hoohoo.ncsa.uiuc.edu/cgi/env.html">NCSA CGI</A> description for details
	 */
	public Properties getCGIEnvironmentVariables()
		{
		return cgiData;
		}
		

  /**
   * Returns the Base URL of this script.
   */
	public String getBaseURL()
	{
      StringBuffer url = new StringBuffer();
      url.append("http://");
      url.append(  cgiData.get( "SERVER_NAME" ));
      
      if (! cgiData.getProperty( "SERVER_PORT", "" ).equals("80") )
      	{
      	url.append( ":");
      	url.append( cgiData.getProperty( "SERVER_PORT", "" ) );
      	}
      url.append(  cgiData.getProperty( "SCRIPT_NAME", "" ) );

      return url.toString();
	}

	/**
	 * Returns the full URL of this script.
    */
	public  String getFullURL()
		{
      StringBuffer url = new StringBuffer();
		url.append( getBaseURL() );
		url.append( cgiData.getProperty( "PATH_INFO","" ) );
		
		if ( cgiData.contains( "QUERY_STRING" ) )
      	{
      	url.append( "?" );
      	url.append( cgiData.getProperty( "QUERY_STRING" ) );
      	}
      return url.toString();;
		}

	/**
	 * Returns true if this is a post request
	 */
	public boolean isPostRequest()
		{
		String request = cgiData.getProperty( "REQUEST_METHOD");
		return "POST".equalsIgnoreCase( request);
		}

	/**
	 * Returns true if this is a get request
	 */
	public boolean isGetRequest()
		{
		String request = cgiData.getProperty( "REQUEST_METHOD");
		return "GET".equalsIgnoreCase( request);
		}

	/**
	 * Returns true if this is a put request
	 */
	public boolean isPutRequest()
		{
		String request = cgiData.getProperty( "REQUEST_METHOD");
		return "PUT".equalsIgnoreCase( request);
		}
		
	/**
	 * Send a text message to the client. The text will <strong>not</strong>
	 * be rendered as html by client.
	 * Useful for sending error messages to client. Can send repeated messages to client.
	 */
	public void sendText( String textMessage )
		{
		if ( out == null )	// send header if this is first message sent
			{
			out = System.out;
			out.print("Content-type: text/plain\n\n");
			}
		out.println( textMessage );
		}

	/**
	 * Send an html encoded message to the client. The message will be 
	 * rendered as html by client.
	 * Can send repeated messages to client..
	 */
	public void sendHtml( String htmlMessage )
		{
		if ( out == null )	// send header if this is first message sent
			{
			out = System.out;
			out.print("Content-type: text/html\n\n");
			}
		out.println( htmlMessage );
		}

	/**
	 * Sends a url to client to use unstead of current url.
	 * Url can be a full url or relative to current url.
	 * Does not make sense to send another message to client after calling this method,
	 * as client will drop connection upon getting this message.
	 */	
	public static void sendRedirect( String url )
		{
		System.out.println("Location: " + url + "\n");
		}
	
	/**
	 * Convert an URL-escaped string to a non-escaped string.
	 * The special characters this method knows about are '+' which is
	 * converted to a space, and '%' which is followed by two hex
	 * digits that make up the character.
	 * The reverse action or converting a string into an URL-escaped
	 * string can be accomplished using the java.net.URLEncoder.encode()
	 * method.
	 * @param URL the escaped string
	 * @return the non-escaped string
	 */
	public String decodeURL(String URL)
		{
		StringBuffer	decoded = new StringBuffer(URL.length());
		StringBuffer	digits = null;
		int		state = 0;

		for (int i = 0; i < URL.length(); i++)
			{
			char	c = URL.charAt(i);

			switch (state)
				{
				case 0:
					if (c == '+')
						decoded.append(' ');
					else if (c == '%')
						{
						state = 1;
						}
					else
						decoded.append(c);
					break;
				case 1:		// First digit.
					digits = new StringBuffer();
					digits.append(c);
					state = 2;
					break;
				case 2:		// Second digit
					digits.append(c);
					state = 0;
					decoded.append((char) Integer.parseInt(digits.toString(),
									   16));
					break;
				}
			}
		return decoded.toString();
		}

	/** 
	 * Convert form data sent to program into name value pairs stored in a 
	 * hashtable
	 * @return hashtable containing form data
	 */
	private Properties tokenizeFormData( String formDataString )
		{
		Properties dataPairs = new Properties();
		//if ( ( formDataString == null ) || ( formDataString.length() < 1 ) )
		//	return dataPairs;
			
		StringTokenizer	tokens = new StringTokenizer(formDataString, "&");

		String		name;
		String		value;
		StringTokenizer	nameValuePair;
		while (tokens.hasMoreTokens())
			{
			nameValuePair = new StringTokenizer(tokens.nextToken(),  "=");
			name = nameValuePair.nextToken();
			if (nameValuePair.hasMoreTokens())
				{
				value = nameValuePair.nextToken();
				dataPairs.put(name, decodeURL(value));
				}
			else
				{
				dataPairs.put(name, "");
				}
			}
		return dataPairs;
		}
		
	/**
	 * Read data sent to cgi program via standard input. This happens with post or put
	 * command from client
	 * @return string read from standard in
	 */
	private String readCGIStandardIn()
		{
		int	dataSize;
		try
			{
			dataSize = Integer.parseInt(System.getProperty("CONTENT_LENGTH"));
			byte		data[] = new byte[dataSize];
			DataInputStream	in = new DataInputStream(System.in);
		
			in.readFully(data, 0, dataSize);
			return new String(data, 0);
			}
		catch (Exception e)
			{
			sendText(e.toString());
			return "";
			}
	
		}

}


