/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.net;

import java.io.*;
import java.net.*;

/*
 * This class implements server sockets. A server socket waits for 
 * requests to come in over the network. It performs some operation 
 * based on that request, and then possibly returns a result to the requester.
 * between two machines. Each input and output byte is
 * Xor'ed with the byte mask provided in the constructor.
 * This provides a very weak form of encryption.
 *
 * @see java.net.ServerSocket
 * @version 1.0 14 Feb 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
//12345678901234567890123456789012345678901234567890123456789012345678
public class XorServerSocket extends ServerSocket 
	{

	private byte xorMask = 0;
	
	/**
	 * Creates a server socket on a specified port. A port of 
	 * <code>0</code> creates a socket on any free port. 
	 * <p>
	 * The maximum queue length for incoming connection indications (a 
	 * request to connect) is set to <code>50</code>. If a connection 
	 * indication arrives when the queue is full, the connection is refused.
	 *
	 * @param	  port  the port number, or <code>0</code> to use any
	 *				   free port.
	 * @param	  mask  byte mask with which all input/output bytes are 
	 *		Xor'ed
	 * @exception  IOException  if an I/O error occurs when opening the 
	 *		socket.
	 */
	public XorServerSocket(int port, byte mask) 
		throws IOException 
		{
		super(port);
		xorMask = mask;
		}

	/**
	 * Creates a server socket on a specified port. A port of 
	 * <code>0</code> creates a socket on any free port. 
	 * <p>
	 * The maximum queue length for incoming connection indications (a 
	 * request to connect) is set to the <code>count</code> parameter. If 
	 * a connection indication arrives when the queue is full, the 
	 * connection is refused. 
	 *
	 * @param	  port  the port number, or <code>0</code> to use any
	 *				   free port.
	 * @param	  backlog  the maximum length of the queue.
	 * @param	  mask  byte mask with which all input/output bytes 
	 *		are Xor'ed
	 * @exception  IOException  if an I/O error occurs when opening 
	 *		the socket.
	 */
	public XorServerSocket(int port, int backlog, byte mask) 
		throws IOException 
		{
		super(port, backlog);
		xorMask = mask;
		}

	/** 
	 * Create a server with the specified port, listen backlog, and 
	 * local IP address to bind to.  The <i>bindAddr</i> argument
	 * can be used on a multi-homed host for a ServerSocket that
	 * will only accept connect requests to one of its addresses.
	 * If <i>bindAddr</i> is null, it will default accepting
	 * connections on any/all local addresses.
	 * The port must be between 0 and 65535, inclusive.
	 * <P>
	 * @param port the local TCP port
	 * @param backlog the listen backlog
	 * @param bindAddr the local InetAddress the server will bind to
	 * @param	  mask  byte mask with which all input/output bytes 
	 *		are Xor'ed
	 */
	public XorServerSocket(int port, 
									int backlog,  
									InetAddress bindAddress, 
									byte mask) 
		throws IOException 
		{
		super(port, backlog, bindAddress);
		xorMask = mask;
		}

	/**
	 * Listens for a connection to be made to this socket and accepts 
	 * it. The method blocks until a connection is made. 
	 *
	 * @exception  IOException  if an I/O error occurs when waiting for a
	 *			   connection.
	 */
	public Socket accept() throws IOException 
		{
		Socket socket = new XorSocket( xorMask );
		implAccept(socket);
		return socket;
		}
	}