/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.net;

import java.io.*;
import java.net.*;
import sdsu.io.*;
 
/**
 * This class implements client sockets (also called just 
 * "sockets"). A socket is an endpoint for communication 
 * between two machines. Each input and output byte is
 * Xor'ed with the byte mask provided in the constructor.
 * This provides a very weak form of encryption.
 * <p>
 *
 * @see java.net.Socket
 * @version 1.0 14 Feb 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class XorSocket extends Socket {

	/* Streams used by socket */
	private InputStream in;
	private OutputStream out;

	private byte xorMask = 0;

	/** 
	 * Creates an unconnected socket. Used in Socket factories. 
	 * @param	  mask  byte mask with which all input/output bytes 
	 *		are Xor'ed
	 * @exception  IOException  if an I/O error occurs when creating 
	 * 	the socket.
	 */
	public XorSocket( byte mask) 
		throws IOException 
		{
		super();
		xorMask = mask;		
		}

	/** 
	 * Constructor for class XorSocket. 
	 * @param host the remote machine to connect to
	 * @param port the remote TCP port
	 * @param	  mask  byte mask with which all input/output bytes 
	 *		are Xor'ed
	 * @exception  IOException  if an I/O error occurs when creating 
	 * 	the socket.
	 */
	public XorSocket(String host, int port, byte mask) 
		throws IOException 
		{
		super(host, port);
		xorMask = mask;		
		}

	/** 
	 * Creates a stream socket and connects it to the specified port 
	 * number at the specified IP address. 
	 *
	 * @param	  address   the IP address.
	 * @param	  port	  the port number.
	 * @param	  mask  byte mask with which all input/output bytes 
	 *		are Xor'ed
	 * @exception  IOException  if an I/O error occurs when creating 
	 * 	the socket.
	 */
	public XorSocket(InetAddress address, int port, byte mask) 
		throws IOException 
		{
		super(address, port);
		xorMask = mask;		
		}

	/** 
	 * Creates a socket and connects it to the specified remote host on
	 * the specified remote port. The Socket will also bind() to the local
	 * address and port supplied.
	 * @param host the name of the remote host
	 * @param port the remote port
	 * @param localAddr the local address the socket is bound to
	 * @param localPort the local port the socket is bound to
	 * @param	  mask  byte mask with which all input/output bytes 
	 *		are Xor'ed
	 * @exception  IOException  if an I/O error occurs when creating 
	 * 	the socket.
	 */
	public XorSocket(String host, 
							int port, 
							InetAddress localAddr, 
		  					int localPort,
		  					byte mask) throws IOException 
		{
		super( host, port, localAddr, localPort);
		xorMask = mask;		
		}

	/** 
	 * Creates a socket and connects it to the specified remote address on
	 * the specified remote port. The Socket will also bind() to the local
	 * address and port supplied.
	 * @param address the remote address
	 * @param port the remote port
	 * @param localAddr the local address the socket is bound to
	 * @param localPort the local port the socket is bound to
	 * @param	  mask  byte mask with which all input/output bytes 
	 *		are Xor'ed
	 * @exception  IOException  if an I/O error occurs when creating 
	 * 	the socket.
	 */
	public XorSocket(InetAddress address, 
							int port, 
							InetAddress localAddr, 
		  					int localPort,
		  					byte mask) 
			throws IOException 
		{
		super(address, port, localAddr, localPort );
		xorMask = mask;		
		}		 

	/** 
	 * Returns an inputstream on the socket.
	 * All bytes in the stream are Xor'ed with the sockets
	 * Xor mask; 
	 * @exception  IOException  if an I/O error occurs.
	 */
	public InputStream getInputStream() 
		throws IOException 
		{
		if (in == null) 
			{
			BufferedInputStream buffered = 
				new BufferedInputStream( super.getInputStream() );
			in = new XorInputStream( buffered , xorMask);
			}
		return in;
		}

	/** 
	 * Returns an outputstream on the socket.
	 * All bytes in the stream are Xor'ed with the sockets
	 * Xor mask; 
	 * @exception  IOException  if an I/O error occurs.
	 */
	public OutputStream getOutputStream() 
		throws IOException 
		{
		if (out == null) 
			{
			BufferedOutputStream buffered = 
				new BufferedOutputStream( super.getOutputStream() );
			out = new XorOutputStream(buffered, xorMask);
			}
		return out;
		}

	/*
	 * Close the socket, flushing output stream first.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public synchronized void close() throws IOException 
		{
		OutputStream o = getOutputStream();
		o.flush();
		super.close();
		}

	}