/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.rmi.registry;

import java.rmi.*;
import java.rmi.registry.*;
import java.io.StringWriter;
import java.io.PrintWriter;

/**
 * Registrar is a client that can be used to register 
 * remote objects with a UniVMRegistry. Create a
 * Registrar with host and port of the machine that
 * is running the UniVMRegistry. You can bind objects to
 * a UniVMRegistry on a remote host. Object bound to the 
 * UniVMRegistry are run in the VM running the UniVMRegistry,
 * not the VM running the Registrar.
 *
 * @see sdsu.rmi.registry.UniVMRegistry
 * @version 0.9 21 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class Registrar implements java.rmi.registry.Registry
	{
	Registry rmiRegistry;
	RemoteRegistrar uniRegistry;
	
	/**
	 * Create a Registrar for the UniVMRegistry on current host
	 * using the default rmi registry port.
	 */	
	public Registrar( ) throws RemoteException 
		{
		this( Registry.REGISTRY_PORT  );
		}

	/**
	 * Create a Registrar for the UniVMRegistry on current host
	 * using the given port.
	 */	
	public Registrar( int port ) throws RemoteException 
		{
		rmiRegistry = LocateRegistry.getRegistry( port );
		}

	/**
	 * Create a Registrar for the UniVMRegistry on given host
	 * using the default rmi registry port.
	 */	
	public Registrar( String host) throws RemoteException 
		{
		this( host, Registry.REGISTRY_PORT  );
		}

	/**
	 * Create a Registrar for the UniVMRegistry on given host
	 * using the given port.
	 */	
	public Registrar( String host, int port ) throws RemoteException 
		{
		rmiRegistry = LocateRegistry.getRegistry(host, port );
		}
	
	/**
	 * Returns an array of the names in the registry.
	 * @exception RemoteException If remote operation failed.
	 */
	public String[] list( ) throws RemoteException
		{
		return rmiRegistry.list();
		}
	
	/** 
	 * Rebind the name to a new object, created from the 
	 * class indicated by className, replacing any existing binding.
	 * The remote registry's classpath <B>must</B> include the className.
	 * @param className <B>Full</B> class name, used to create remote object.
	 *		Class must implement java.rmi.Remote
	 * @exception RemoteException If remote operation failed.
	 * @exception AccessException If this operation is not permitted.
	 */
	public void rebind(  String name, String className ) throws RemoteException
		{
		try
			{
			RemoteRegistrar uniRegistry = 
				(RemoteRegistrar) rmiRegistry.lookup( UniVMRegistry.RMI_NAME);

			uniRegistry.rebind( name, className );
			}
		catch (NotBoundException error )
			{
			String errorMessage = "UniVMRegistry not installed " +
				error.toString();
			throw new RemoteException( errorMessage );
			}	
		}
		

	/** 
	 * Rebind the name to a new object, replacing any existing binding.
	 * @exception RemoteException If remote operation failed.
	 * @exception AccessException If this operation is not permitted.
	 */
	public void rebind(  String name, Remote object ) throws RemoteException
		{
		try
			{
			RemoteRegistrar uniRegistry = 
				(RemoteRegistrar) rmiRegistry.lookup( UniVMRegistry.RMI_NAME);

			uniRegistry.rebind( name, object );
			}
		catch (NotBoundException error )
			{
			String errorMessage = "UniVMRegistry not installed " +
				error.toString();
			throw new RemoteException( errorMessage );
			}
		} 

	/**
	 * Binds the name to the specified remote object.
	 * @exception RemoteException If remote operation failed.
	 * @exception AlreadyBoundException If name is already bound.
	 */
	public void bind(String name, Remote object)
		throws RemoteException, AlreadyBoundException
		{
		try
			{
			RemoteRegistrar uniRegistry = 
				(RemoteRegistrar) rmiRegistry.lookup( UniVMRegistry.RMI_NAME);

			uniRegistry.bind( name, object );
			}
		catch (NotBoundException error )
			{
			String errorMessage = "UniVMRegistry not installed " +
				error.toString();
			throw new RemoteException( errorMessage );
			}
		}
		
	/**
	 * Unbind the name.
	 * @exception RemoteException If remote operation failed.
	 * @exception NotBoundException if there is no object with this name in the
	 *			  registry.
	 * @exception AccessException If this operation is not permitted.
	 */
	public void unbind( String name ) 
		throws RemoteException, NotBoundException, AccessException
		{
		rmiRegistry.unbind( name );
		}

	/**
	 * Returns the remote object associated with the specified name in the 
	 * registry.
	 *
	 * @exception RemoteException If remote operation failed.
	 * @exception NotBoundException if there is no object with this name in the
	 *			  registry.
	 * @exception AccessException If this operation is not permitted.
	 */
	public Remote lookup(String name)
		throws RemoteException, NotBoundException, AccessException
		{
		return rmiRegistry.lookup( name );
		}

	/**
	 * Binds the name to the specified remote object on
	 * locat registry on given port.
	 * Returns a list names which objects are bound to in
	 * the registry.
	 * @exception RemoteException If remote operation failed.
	 */
	public static String verboseRebind( int port, String name, String className ) 
		throws RemoteException
		{
		StringWriter outputString = new StringWriter();
		PrintWriter out = new PrintWriter( outputString );
		
		Registrar installer = new Registrar( port );
		installer.rebind( name, className);
		
		String [] servers = installer.list();

		out.println( "The registry now contains object bound to:");
		for ( int k = 0 ; k < servers.length; k++ )
			out.println( servers[k] );
		
		return outputString.toString();

		} 

	}
