/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.rmi.registry;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.registry.*;
import sdsu.util.ProgramProperties;

/**
 * UniVMRegistry allows server objects created in the VM that 
 * is running the rmi registy. This means you do not have to 
 * create a new VM and an new process for each object in the
 * registry. The ".class" files, including the stub and skeletons
 * for an object added to the registry <B>must</B> be accessable
 * using the classpath of the process that created the UniVMRegistry
 * and the rmiregistry.
 * <P>
 * The UniVMRegistry is a normal rmi registry with an 
 * object of type Registrant bound at the name "RemoteRegistrar".
 * Use Registrar to register object to on a UniVMRegistry.
 * <P>
 * The easiest way to run a UniVMRegistry is using the command
 * <pre>
 * 	java sdsu.rmi.registry.UniVMRegistry -p=5555 &
 * </pre>
 * after you have an rmiregistry running on the given port.
 * If the "-p" flag is not given, the default rmiregistry port
 * 1099 is used. 
 *
 * @see sdsu.rmi.registry.Registrar
 * @version 0.9.1 2 Feb. 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class UniVMRegistry 
		extends UnicastRemoteObject
		implements RemoteRegistrar
	{
	
	Registry installee;
	
	// Insure there is an active security manager
	{
	SecurityManager currentManager = System.getSecurityManager();
	if (currentManager == null )
		System.setSecurityManager(new RMISecurityManager());
	}
	
	public static final String RMI_NAME = "RemoteRegistrar";
			
	/**
	 * Install a UniVMRegistry on the default rmiregistry port.
	 * If a registry is active on that port make it a 
	 * UniVMRegistry.
	 */	
	public UniVMRegistry( ) throws RemoteException 
		{
		this( Registry.REGISTRY_PORT  );
		}

	/**
	 * Install a UniVMRegistry on the given port.
	 * If a registry is active on that port make it a 
	 * UniVMRegistry.
	 */	
	public UniVMRegistry( int port) throws RemoteException 
		{
		installee = getLocalRegistry( port );
		}

	/**
	 * Return the local registry on the given port. 
	 * which aready has an rmiregistry running.
	 */
	protected Registry getLocalRegistry( int port ) 
		throws RemoteException 
		{
		try
			{
			return LocateRegistry.getRegistry( port );
			}
		catch ( RemoteException noRegistry )
			{
			return LocateRegistry.createRegistry( port );
			}
		}
	
	/**
	 * Binds a Remote object created from the class className in the
	 * rmiregistry. Replaces and object currently bound at same name.
	 * The registry's classpath <B>must</B> include the className.
	 * @param className Full class name, used to create remote object.
	 * @param name Name to which object is bound
	 * @returns Name to which object is bound to.  
	 * @exception RemoteException thrown if there is any problem
	 * 	binding object.
	 */
	public void rebind( String name, String className) 
		throws RemoteException
		{
		try
			{
			Remote serverObject = 
				(Remote) Class.forName( className ).newInstance();
			
			rebind( name, serverObject);
			}
		catch ( Exception error )
			{
			String errorMessage = "Installing " + className + " failed. " +
				"Exception: " + error.getMessage();
				
			throw new RemoteException( errorMessage );
			}		
		}
	
	/**
	 * Binds a Remote object created from the class className in the
	 * rmiregistry. Replaces and object currently bound at same name.
	 * @param object  Remote object which is bound.
	 * @param name Name to which object is bound.
	 * @returns Name to which object is bound to.  
	 * @exception RemoteException thrown if there is any problem
	 * 	binding object.
	 */
	public void rebind( String name, Remote object) 
		throws RemoteException
		{
		installee.rebind( name, object);

		}

    /**
     * Binds the name to the specified remote object.
     * @exception RemoteException If remote operation failed.
     * @exception AlreadyBoundException If name is already bound.
     */
    public void bind(String name, String remoteObjectClassName)
		throws RemoteException, AlreadyBoundException
		{
		try
			{
			Remote serverObject = 
				(Remote) Class.forName( remoteObjectClassName ).newInstance();
			
			bind( name, serverObject);
			}
		catch ( ClassNotFoundException error )
			{
			String errorMessage = "Could not find " + remoteObjectClassName +
				 error.getMessage();
				
			throw new RemoteException( errorMessage );
			}		
		catch ( IllegalAccessException error )
			{
			String errorMessage = "Could not access " + remoteObjectClassName +
				 error.getMessage();
				
			throw new RemoteException( errorMessage );
			}		
		catch ( InstantiationException error )
			{
			String errorMessage = "Could not instantiate " + remoteObjectClassName +
				 error.getMessage();
				
			throw new RemoteException( errorMessage );
			}		

		}

    /**
     * Binds the name to the specified remote object.
     * @exception RemoteException If remote operation failed.
     * @exception AlreadyBoundException If name is already bound.
     */
    public void bind(String name, Remote object)
		throws RemoteException, AlreadyBoundException
		{
		installee.bind( name, object);
		}

	/**
	 * Returns an array of the names in the registry.
	 * @exception RemoteException If remote operation failed.
	 * @exception AccessException If this operation is not permitted.
	 */
	public String[] list() throws  RemoteException, AccessException
		{
		return installee.list();
		}
		
	/**
	 * Install a UniVMRegistry. To specify the port use the
	 * "-p" flag to specify a port other than the default 
	 * rmiregistry port. If registry already exists on that
	 * port make it a UniVMRegistry. Prints out all names
	 * that are currently bound to objects in the 
	 * registry. 
	 */
	public static void main(String args[]) 
		{
		try 
			{
			ProgramProperties flags = new ProgramProperties( args );
			
			int port = flags.getInt( "p", Registry.REGISTRY_PORT);			
			UniVMRegistry serverObject = new UniVMRegistry( port );
					
			serverObject.rebind( RMI_NAME, serverObject );
			System.out.println("UniVMRegistry bound in registry");
			
			String [] servers = serverObject.list();

			if ( servers.length > 0)
				System.out.println( "The server on port " + port + 
					" now contains objects bound to the names: ");

			for ( int k = 0 ; k < servers.length; k++ )
				System.out.println( servers[k] );
			} 
		catch (Exception e) 
			{
			System.out.println("Error in installing Registry: ");
			e.printStackTrace();
			}
		}
	
	}