/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package sdsu.rmi.server;


import java.io.*;
import java.net.*;
import java.rmi.server.*;
import sdsu.net.*;

/**
 * The RMISocketFactory is used by the RMI runtime in order to obtain
 * client and server sockets for RMI calls. This factory creates 
 * XorSockets and XorServerSocket, which use the byte mask given
 * the constructor. To use Xor Sockets in an RMI application
 * execute the following code before creating a remote object
 * on both the client and server side:
 *<pre>
 *	try
 		{
 		RMISocketFactory.setSocketFactory(
 			new XorRMISocketFactory( (byte) 0x1A ) );
 		}
 	catch (IOException factoryError )
 		{
 		System.err.println( "setSocketFactory failed: " +
 			factoryError );
 		}
 *</PRE>
 * Based on example found at: 
 * <A href="http://java.sun.com/products/jdk/1.2/docs/guide/rmi/copy/rmisocketfactory.doc.html">
 * http://java.sun.com/products/jdk/1.2/docs/guide/rmi/copy/rmisocketfactory.doc.html</A>
 *
 * @version 1.0 14 Feb 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class XorRMISocketFactory extends RMISocketFactory 
	{

	private byte xorMask = 0;
	
	/**
	 * Create a new socket factory with the given
	 * byte mask to be used in the XorSockets.
	 */
	public XorRMISocketFactory( byte mask )
		{
		xorMask = mask;
		}
		
	/**
	 * Create a client socket connected to the specified host and port.
	 * @exception  IOException  if an I/O error occurs.
	 */
	public Socket createSocket(String host, int port)
		throws IOException
		{
		XorSocket socket = 
			new XorSocket(host, port, xorMask);
		return socket;
		}

	/**
	 * Create a server socket on the specified port (port 0 represents
	 * an anonymous port).
	 * @exception  IOException  if an I/O error occurs.
	 */
	public ServerSocket createServerSocket(int port)
		 throws IOException
		{
		XorServerSocket server = 
		 	new XorServerSocket(port, xorMask);
		return server;
		}
	}