/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

 
package sdsu.sql;

import java.sql.*;
import sdsu.util.Table;
import sdsu.util.LabeledTable;
import java.util.*;

/**
 * A class defines an interface for transfering tables between Java and 
 * a database. This class is easy to misuse. You still should use SQL to
 * have the database perform the query, don't just download all the
 * tables and then perform queries.
 * @version 0.5 12 July 1997 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */


public class DatabaseTable
	{
	Table dataElements;
	Hashtable columnIndex;
	
	/**
	 * Types are stored just to know if item needs to be quoted in sql
	 * statement
	 */
	int[] columnTypes;
	
	String tableName;
	
	
	/**
	 * Create a DatabaseTable object on the result of the sqlSelect query.
	 * @param sqlSelect sql query used to select data.
	 * @param toDatabase connection to the database that evaluates the sql query.
	 * @exception SQLException thrown on any error connecting to database or 
	 * satisfying the query
	 */
	public static DatabaseTable fromSQL( String sqlSelect,  Connection toDataBase )
		throws SQLException
		{
		Statement query = toDataBase.createStatement(); 
		ResultSet data = query.executeQuery( sqlSelect );
		DatabaseTable tableData = new DatabaseTable( data );
		query.close();
		return tableData;  
		}

	/**
	 * Create a DatabaseTable object containing a table from a database.
	 * @param tableName the name of the table in the database
	 * @param toDatabase connection to the database which contains the table to be 
	 * downloaded into DatabaseTable object.
	 * @exception SQLException thrown on any error connecting to database or obtaining 
	 * the table.
	 */
	public static DatabaseTable getTable( String tableName,  Connection toDataBase )
		throws SQLException
		{
		String selectTable = "SELECT * FROM " + tableName;
		return DatabaseTable.fromSQL( selectTable, toDataBase );
		}

	/**
	 * Create a DatabaseTable object containing selected rows a table from a database.
	 * @param tableName the name of the table in the database.
	 * @param column a column label in the table.
	 * @param columnValue select all rows that have this value in the given column
	 * @param toDatabase connection to the database which contains the table to be 
	 * downloaded into DatabaseTable object.
	 * @exception SQLException thrown on any error connecting to database or obtaining 
	 * the table.
	 */
	public static DatabaseTable getRow( String tableName, 
													String column, 
													String columnValue, 
													Connection toDataBase )
		throws SQLException
		{
		String selectTable = "SELECT * FROM " + tableName + 
										" WHERE " + column + " = " + columnValue;
		return DatabaseTable.fromSQL( selectTable, toDataBase );
		}

	/**
	 * Creates a DatabaseTable with rows equal to the table data.
	 */
	public DatabaseTable( Table data )
		{
		dataElements = (Table) data.clone();
		setColumnLabels();
		setTypesAsQuotable();	
		}

	public DatabaseTable( ResultSet dateBaseContents ) throws SQLException
		{
		ResultSetMetaData tableData = dateBaseContents.getMetaData();
		int numberOfColumns = tableData.getColumnCount();
		dataElements = new Table( 1, numberOfColumns );
		
		// getColumn Labels
		for ( int k = 0; k < numberOfColumns; k++ )
			dataElements.setElementAt(tableData.getColumnName(k+1), 0, k );
		
		setColumnLabels();

		columnTypes = new int[numberOfColumns];
		for ( int k = 0; k < numberOfColumns; k++ )
			columnTypes[k] = tableData.getColumnType(k+1);
		
		while ( dateBaseContents.next() )
			{
			Vector rowElements = new Vector( numberOfColumns );
			for ( int k = 1;  k <= numberOfColumns; k++ )
				rowElements.addElement( dateBaseContents.getString( k ) );
			dataElements.addRow( rowElements );
			}
		dateBaseContents.close();
		}
	
	public String toString()
		{
		return dataElements.toString();
		}
		
	/**
	 * Convert DatabaseTable to a LabeledTable. Row zero contains the 
	 * column vectors.
	 */
	public LabeledTable toLabeledTable()
		{
		return new LabeledTable( dataElements );
		}
		
	/**
	 * Set the type of the elements of a column
	 * @param type	the type of the elements as defined in java.sql.Types.
	 * @param columnLabel indicates the column that we are setting the type
	 */
	public void setColumnType( int type, String columnLabel )
		{
		columnTypes[ columnIndex( columnLabel ) ] = type;
		}

	/**
	 * Set the type of the elements of a column
	 */
	public int getColumnType( String columnLabel )
		{
		return columnTypes[ columnIndex( columnLabel ) ];
		}

	/**
	 * Return the element in a given row and column. Rows start at
	 * one.
	 * @param row index of the row of the element
	 * @param columnLabel label of the column of the element. Labels are 
	 * converted to lower case to void case problems.
	 */
	public Object elementAt( int row, String columnLabel )
		{
		return dataElements.elementAt( row, columnIndex( columnLabel ) );
		}

	/**
	 * Return the element in a given row and column. Rows start at
	 * one.
	 * @param row index of the row of the element
	 * @param columnLabel label of the column of the element
	 */
	public void setElementAt( Object element, int row, String columnLabel )
		{
		 dataElements.setElementAt(element, row, columnIndex( columnLabel ) );
		}
	
	/**
	 * Returns the column of data indicated by the columnLabel.
	 * Column label is not in the returned vector.
	 */
	public Vector columnAt( String columnLabel )
		{
		Vector column = dataElements.columnAt( columnIndex( columnLabel ) );
		column.removeElementAt( 0 );
		return column;
		}
		
	/**
	 * Returns the number of rows of data in the table.
	 */
	public int numberOfRows()
		{
		return dataElements.numberOfRows() - 1;
		}

	/**
	 * Returns the number of columns of data in the table.
	 */
	public int numberOfColumns()
		{
		return dataElements.numberOfColumns();
		}
		
	/**
	 * Return all the column labels
	 */
	public Enumeration getColumnLabels()
		{
		return columnIndex.keys();
		}

	/** 
	 * Remove leading and trailing spaces from all data elements
	 */
	public void trimAll()
		{
		for ( int row = 1; row <= numberOfRows(); row++ )
			for ( int column = 0; column < numberOfColumns(); column++ )
				{
				String trimedElement = 
						dataElements.elementAt( row, column).toString().trim();
				dataElements.setElementAt( trimedElement, row, column );
				}
		} 
		
	/**
	 * Add rows of this table to a table in a database.
	 * @param dbTableName name of the table in the database which rows
	 * will be inserted
	 * @param toDataBase a connection to the database which contains the table
	 * dbTableName
	 * @return total number of rows that were added to database table
	 * @exception SQLException
	 */
	public int databaseInsert( String dbTableName, Connection toDataBase ) 
		throws SQLException
		{
		int rowCount = 0;
		Statement insert = toDataBase.createStatement();  
		for ( int row = 1; row <= numberOfRows(); row++)
			{
			rowCount += insert.executeUpdate( sqlInsertRow( dbTableName, row ));
			}
		insert.close();
		return rowCount;
		}
	public int databaseInsertTest( String dbTableName ) throws SQLException
		{
		int rowCount = 0;
		for ( int row = 1; row <= numberOfRows(); row++)
			{
			System.out.println( sqlInsertRow( dbTableName, row ) );
			}
		return rowCount;
		}

	/**
	 * Deletes rows of this table from a table in a database. Rows in 
	 * the database table with are identical to the rows in this table 
	 * will be deleted.
	 * @param dbTableName name of the table in the database which rows
	 * will be deleted
	 * @param toDataBase a connection to the database which contains the table
	 * dbTableName
	 * @exception SQLException
	 */
	public int databaseDelete( String dbTableName, Connection toDataBase ) 
		throws SQLException
		{
		int rowCount = 0;
		Statement delete = toDataBase.createStatement();  
		for ( int row = 1; row <= numberOfRows(); row++)
			{
			rowCount += delete.executeUpdate( sqlDeleteRow( dbTableName, row ));
			}
		delete.close();
		return rowCount;
		}
	
	/**
	 * Updates rows of this table from a table in a database. Rows in 
	 * the database table with are identical to the rows in this table 
	 * will be deleted.
	 * @param dbTableName name of the table in the database which
	 * will be updated.
	 * @param key column label used to select which rows in the database 
	 * table to update. Any row in the database table that has the same value 
	 * in the column indicated by key as a row in this table will be updated.
	 * @param toDataBase a connection to the database which contains the table
	 * dbTableName.
	 * @exception SQLException
	 */
	public int databaseUpdate( String dbTableName, 
								String key, Connection toDataBase ) throws SQLException
		{
		Vector keys = new Vector();
		keys.addElement( key );
		return databaseUpdate( dbTableName, keys, toDataBase );
		}

	/**
	 * Updates rows of this table from a table in a database. Rows in 
	 * the database table with are identical to the rows in this table 
	 * will be deleted.
	 * @param dbTableName name of the table in the database which
	 * will be updated.
	 * @param keys Vector of column labela used to select which rows in the database 
	 * table to update. Any row in the database table that has the same value 
	 * in the columns indicated by keys as a row in this table will be updated.
	 * @param toDataBase a connection to the database which contains the table
	 * dbTableName.
	 * @exception SQLException
	 */
	public int databaseUpdate( String dbTableName, 
								Vector keys, Connection toDataBase ) throws SQLException
		{
		int rowCount = 0;

		Statement update = toDataBase.createStatement();  
		for ( int row = 1; row <= numberOfRows(); row++)
			{
			rowCount += update.executeUpdate( sqlUpdateRow( dbTableName, row, keys ));
			}
		update.close();
		return rowCount;
		}

	/**
	 * Returns true if dataType is a numeric type
	 */
	protected static boolean isNumericType( int dataType )
		{
		if ( ( dataType == Types.BIT ) ||
			  ( dataType == Types.TINYINT) ||
			  ( dataType == Types.SMALLINT) ||
			  ( dataType == Types.INTEGER) ||
			  ( dataType == Types.BIGINT) ||
			  ( dataType == Types.FLOAT) ||
			  ( dataType == Types.REAL) ||
			  ( dataType == Types.DOUBLE) ||
			  ( dataType == Types.NUMERIC) ||
			  ( dataType == Types.DECIMAL) 
			)
			return true;
		else
			return false;
		}
		
	/**
	 * Returns the index of the colunm columnLabel
	 */
	protected int columnIndex( String columnLabel )
		{
		return columnIndex.get( columnLabel.toLowerCase() ).hashCode();
		}

	/**
	 * Returns an sql statement that will delete the indicated
	 * row of this table from a database
	 * @param tableName name of table in database to update
	 * @param row index of row of this table to be updated in database table
	 * @param keyColumns labels of columns in this table use to determine
	 * which rows in database table to update.
	 */
	protected String sqlUpdateRow( String tableName,  int row, Vector keyColumns )
		{
		String and = " AND";
		String comma = ", ";
		StringBuffer set = new StringBuffer();
		StringBuffer where = new StringBuffer();

		Enumeration columnLabels = getColumnLabels();
		String currentColumn;
		while ( columnLabels.hasMoreElements() )
			{
			currentColumn = (String) columnLabels.nextElement();
			if ( keyColumns.contains( currentColumn ) )
				{
				where.append( currentColumn );
				where.append( " = " );
				where.append( quotedElementAt( row, currentColumn ) );
				where.append( and );
				}
			else
				{
				set.append( currentColumn );
				set.append( " = " );
				set.append( quotedElementAt( row, currentColumn ) );
				set.append( comma );
				}
			}
		// remove last seperators
		removeFromBuffer( set, comma.length() ); 		
		removeFromBuffer( where, and.length() ); 	
		return "UPDATE 	" + tableName + " SET " + set.toString() +
			" WHERE " + where.toString();	
		}

	/**
	 *
	 */
	protected static void removeFromBuffer( StringBuffer toShorten, 
														int numberOfCharsToRemove )
		{
		toShorten.setLength( toShorten.length() - numberOfCharsToRemove );
		}	
		
	/**
	 * Returns an sql statement that will delete the indicated
	 * row of this table from a database.
	 * @param tableName name of table in database to delete rows
	 * @param row index of row of this table to be removed from database table
	 */
	protected String sqlDeleteRow( String tableName,  int row )
		{
		String and = " AND ";
		StringBuffer deleteClause = new StringBuffer();
		deleteClause.append( "DELETE FROM "	);
		deleteClause.append( tableName);
		deleteClause.append( " WHERE ");
		
		Enumeration columnLabels = getColumnLabels();
		
		while ( columnLabels.hasMoreElements() )
			{
			String currentColumn = (String) columnLabels.nextElement();
			deleteClause.append( currentColumn );
			deleteClause.append( " = " );
			deleteClause.append( quotedElementAt( row, currentColumn ) );
			deleteClause.append( and );
			}
		removeFromBuffer( deleteClause, and.length() ); 	
		return deleteClause.toString();
		}
	
	/**
	 * Returns an sql statement that will insert the indicated
	 * row into a database.
	 * @param tableName name of table in database to insert rows
	 * @param row index of row of this table to be added to database table
	 */
	protected String sqlInsertRow( String tableName,  int row )
		{
		String comma = ", ";
		StringBuffer columns = new StringBuffer();
		StringBuffer values = new StringBuffer();
		columns.append( " ( " );
		values.append( " ( " );
		
		Enumeration columnLabels = getColumnLabels();
			
		while ( columnLabels.hasMoreElements() )
			{
			String currentColumn = (String) columnLabels.nextElement();
			columns.append( currentColumn );
			columns.append( comma );
			values.append( quotedElementAt( row, currentColumn ) );
			values.append( comma );
			}
		removeFromBuffer( columns, comma.length() ); 	
		removeFromBuffer( values, comma.length() ); 	

		columns.append( " ) " );
		values.append( " ) " );
		
		return "INSERT INTO " + tableName + columns.toString() + 
			" VALUES " + values.toString();
		}
		
	/**
	 * Adds quotes to given element if needed in sql statement
	 */
	protected String quotedElementAt( int row, String label )
		{
		int elementType = columnTypes[ columnIndex( label ) ];
		if ( isNumericType( elementType ) )
			return elementAt( row, label ).toString();
		else
			{
			String element = elementAt( row, label ).toString(); 
			return "'" + escapeQuote( element ) + "'";
			}
		}
		
	/**
	 * Escapes any occurance of a quote in a string with a quote.
	 * static final allows the method to be inlined.
	 */
	protected static final String escapeQuote( String toBeEscaped )
		{
		char quote = '\'';
		int quoteIndex = toBeEscaped.indexOf( quote );

		//No quote's in string
		if ( quoteIndex == -1 ) 
			return toBeEscaped;
		
		// Assume that 90+% of strings will have 3 or fewer quotes
		StringBuffer quotedString = new StringBuffer( toBeEscaped.length() + 3 );
		
		// divid the string into tokens. Quotes in the string separate tokens
		int tokenStartIndex = 0;
		while ( ( quoteIndex != -1 ) )
			{
			quotedString.append( toBeEscaped.substring(tokenStartIndex, quoteIndex ) );
			quotedString.append( quote );
			tokenStartIndex = quoteIndex;
			quoteIndex = toBeEscaped.indexOf( quote, quoteIndex + 1 );
			}
			
		quotedString.append( toBeEscaped.substring( tokenStartIndex ) );
		return quotedString.toString();
		}	
		
	/**
	 * Sets all columns as char to make them quoted in sql statements
	 */
	protected void setTypesAsQuotable()
		{
		columnTypes = new int[ dataElements.numberOfColumns() ];
		for ( int column = 0; column < dataElements.numberOfColumns(); column++ )
			columnTypes[ column ] = Types.CHAR;
		}	

	/**
	 * Creates the column label index.
	 */
	protected void setColumnLabels()
		{
		columnIndex = new Hashtable();
		for ( int column = 0; column < dataElements.numberOfColumns(); column++ )
			{
			String label = dataElements.elementAt( 0, column ).toString();
			Integer index = new Integer( column );
			columnIndex.put( label.toLowerCase(), index);
			}
		}

	}