/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package sdsu.util;

/**
 * This class implements a characater stack. Yes the JKD does contain
 * a general stack. However that stack operates on objects. This stack
 * just handles char elements. Use in IO operations where converting 
 * chars to objects will be too expensive.
 * @version 1.0 21 August 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
 
final public class CharStack
	{
	private char[] stackElements;
	private int topStackElement = -1;

	public static final int DEFAULT_QUEUE_SIZE = 256;

	/**
	 * Create a stack of the default size.
	 */	
	public CharStack()
			{
			this( DEFAULT_QUEUE_SIZE );
			}
	
	/**
	 * Create a stack with the given initial size.
	 */	
	public CharStack( int initialSize)
			{
			stackElements = new char[ initialSize ];
			}

	public String toString()
			{
			return "Stack" + capacity() + "[" + String.valueOf( stackElements ) + "]";
			}
	/**
	 * Returns true is stack is empty.
	 */
	public boolean isEmpty()
		{
		if ( topStackElement < 0 )
			return true;
		else
			return false;
		}
		
	/**
	 * Remove and return the top char on the stack.
	 * Assumes stack is not empty.
	 */
	public char pop()
		{
		return stackElements[ topStackElement-- ];
		}

	/**
	 * Returns the top char on the stack. Does <b>not</b> remove any elements.
	 * Assumes stack is not empty.
	 */
	public char peek()
		{
		return stackElements[ topStackElement];
		}

	/**
	 * Returns the total number of locations,, used or not, for chars in the stack 
	 */
	public int capacity()
		{
		return stackElements.length;
		}

	/**
	 * Adds a char to the top of the stack.
	 */
	public void push( char addMe )
		{
		if ( topStackElement >= capacity() - 1 )	// array index starts at zero
			grow();
			
		topStackElement++;
		stackElements[ topStackElement ] = addMe;
		}

	/**
	 * Adds an array of characters to the stack.
	 * First element of the array is pushed first.
	 */
	public void push( char[] addMe )
		{
		for ( int index = 0; index < addMe.length; index++ )
			push( addMe[ index ] );
		}

	/**
	 * Adds an array of characters to the stack.
	 * First element of the array is pushed first.
	 */
	public void push( String addMe )
		{
		push( addMe.toCharArray());
		}

	/**
	 * Removes all elements from the stack.
	 */
	public void clear()
		{
		topStackElement = -1;
		}
					
	/** 
	 * Grows the stack. Growth policy insures amortized cost per insert is O(1)
	 */
	private void grow()
		{
		// Doubling queue insures that amortized cost per insert is O(1)
		if ( capacity() <= 16 )	
			grow( 32 );
		else if ( capacity() <= 2048 )
			grow( capacity() * 2 );
		else 
			grow( (int) (capacity() * 1.5) );
		}	

	/** 
	 * Grows the stack to the given new size
	 */
	private void grow( int newSize )
		{		
		char[] newStack = new char[ newSize ];
		
		System.arraycopy( stackElements, 0,  newStack, 0, capacity());	
		stackElements = newStack;	
		}	
	
	}