/* 
 * Copyright (C) 1999 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
package sdsu.util;

import java.util.BitSet;

/**
 * This class extends the functionality of java.util.BitSet. Since java.util.BitSet is a 
 * final class, ConvertableBitSet is <B>not</B> a subclass of java.util.BitSet.
 * 	It adds the ablity to convert between
 * basic types (ints, float) and BitSet objects.
 *
 * @version 1.0 5 January 1999 
 * @author Roger Whitney 
 *	(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public final class ConvertableBitSet implements Cloneable, java.io.Serializable
	{
	private BitSet theBits;

	/**
	 * @param bits a string representation of the bits. All characters
	 *		in the string must be '0' or '1'. The low order bit is on the right end 
	 *		of the string
	 */
	public static ConvertableBitSet fromString( String bits )
		{
		ConvertableBitSet theBits = new ConvertableBitSet( bits.length() );
		
		for ( int k = 0; k < bits.length(); k++ )
			if ( bits.charAt( bits.length() - k - 1  ) == '1' )
				theBits.set( k );
		
		return theBits;
		}

	/**
	 * @return A bitset that represents the bit pattern of the int "value"
	 * The low order bit in the int is bit zero in the bitset.
	 */
	public static ConvertableBitSet fromInt( int value )
		{
		return fromString( Integer.toBinaryString( value ) );
		}

	/**
	 * @return A bitset that represents the bit pattern of the long "value". 
	 * The low order bit in the long is bit zero in the bitset.
	 */
	public static ConvertableBitSet fromLong( long value )
		{
		return fromString( Long.toBinaryString( value ) );
		}

	/**
	 * Returns the bitset represention of a single-float value. The result is a 
	 * representation of the floating-point argument according to the IEEE 754 
	 *  floating-point "single precision" bit layout. <P>

     * Bit 31 represents the sign of the floating-point number. Bits 30-23 
     * represent the exponent. Bits 22-0 represent the mantissa 
     *  (or significand) of the floating-point number. <P>
     * If the argument is positive infinity, the result is 0x7f800000. <BR>
     * If the argument is negative infinity, the result is 0xff800000. <BR>
     * If the argument is NaN, the result is 0x7fc00000. <BR>

	 * @return A bitset that represents the bit pattern of the float "value".
	 */
	public static ConvertableBitSet fromFloat( float value )
		{
		return fromInt( Float.floatToIntBits( value ) );
		}

	/**
	 * Returns the bitset representation of the specified floating-point value according 
	 * to the IEEE 754 floating-point "double format" bit layout. <P>

     * Bit 63 represents the sign of the floating-point number. Bits 62-52 
     * represent the exponent. Bits 51-0 represent the mantissa or significand 
     *  of the floating-point number. <P>
     
     * If the argument is positive infinity, the result is 0x7ff0000000000000L. <BR>
	 * If the argument is negative infinity, the result is 0xfff0000000000000L. <BR>
	 * If the argument is NaN, the result is 0x7ff8000000000000L. <BR>
     */
	public static ConvertableBitSet fromDouble( double value )
		{
		return fromLong( Double.doubleToLongBits( value ) );
		}

	public ConvertableBitSet()
		{
		theBits = new BitSet();
		}

	/**
	 * Contructs a BitSet object with given number of bits, all set to
	 * zero.
	 */
	public ConvertableBitSet( int size)
		{
		theBits = new BitSet( size );
		}
		

	/**
	 * Constucts a float from the bitset. Uses the IEEE 754 floating-point
	 * "single precision" bit layout to construct the float. Only the first 32 bits
	 * in the BitSet are used.
	 */
	public float toFloat()
		{
		return Float.intBitsToFloat( toInt() );
		}

	/**
	 * Constucts a double from the bitset. Uses the IEEE 754 floating-point
	 * "double format" bit layout to construct the float. Only the first 64 bits
	 * in the BitSet are used.
	 */
	public double toDouble()
		{
		return Double.longBitsToDouble( toLong() );
		}

	/**
	 * Constucts an int from the bitset. Only the first 32 bits
	 * in the BitSet are used.
	 */
	public int toInt( )
		{
		int intValue = 0;
		int bitValue = 1;
		
		for (int bit = 0; bit <32; bit++ )
			{
			if ( theBits.get( bit ) )
				intValue = intValue + bitValue;
			bitValue =  bitValue << 1;
			}
		return intValue;
		}

	/**
	 * Constucts an int from the bitset. Only the first 64 bits
	 * in the BitSet are used.
	 */
	public int toLong( )
		{
		int intValue = 0;
		int bitValue = 1;
		
		for (int bit = 0; bit <64; bit++ )
			{
			if ( theBits.get( bit ) )
				intValue = intValue + bitValue;
			bitValue =  bitValue << 1;
			}
		return intValue;
		}
	
	/**
	 * Logically ANDs current ConvertableBitSet with the specified 
	 * ConvertableBitSet.
	 */
	public void and( ConvertableBitSet set )
		{
		theBits.and( set.theBits);
		}
	
	/**
	 * Logically ANDs current ConvertableBitSet with the specified 
	 * BitSet.
	 */
	public void and(  BitSet set )
		{
		theBits.and( set);
		}
	
	/**
	 * Logically ORs current ConvertableBitSet with the specified 
	 * BitSet.
	 */
	public void or(BitSet set)
		{
		theBits.or( set);
		}

	/**
	 * Logically ORs current ConvertableBitSet with the specified 
	 * ConvertableBitSet.
	 */
	public void or(ConvertableBitSet set)
		{
		theBits.or( set.theBits);
		}

	/**
	 * Logically XORs current ConvertableBitSet with the specified 
	 * BitSet.
	 */
	public void xor(BitSet set)
		{
		theBits.xor( set);
		}

	/**
	 * Logically XORs current ConvertableBitSet with the specified 
	 * ConvertableBitSet.
	 */
	public void xor(ConvertableBitSet set)
		{
		theBits.xor( set.theBits);
		}
	
	/**
	 * Sets the given bit to zero (or false).
	 */
	public void clear( int bit )
		{
		theBits.clear( bit );
		}

	/**
	 * Returns the value of the given bit.
	 */
	public boolean get( int bit )
		{
		return theBits.get( bit );
		}

	/**
	 * Sets the given bit to one (or true).
	 */
	public void set( int bit )
		{
		theBits.set( bit );
		}

	public int hashCode(  )
		{
		return theBits.hashCode( );
		}
	
	/**
	 * Returns the number of bits in the ConvertableBitSet.
	 */
	public int size()
		{
		return theBits.size();
		}	

	public boolean equals( Object anObject )
		{
		if ( anObject instanceof ConvertableBitSet )
			return theBits.equals( ((ConvertableBitSet) anObject).theBits );
		else
			return theBits.equals( anObject );
		}
	
	public Object clone()
		{
    	try 
    		{
			ConvertableBitSet aClone = (ConvertableBitSet) super.clone();
			aClone.theBits =(BitSet) theBits.clone();
			return aClone;
      		} 
		catch (CloneNotSupportedException e)
			{ 
			// this shouldn't happen, since we are Cloneable
			throw new InternalError();		
			}
		}
		
	/**
	 * Returns a string representation of the bitset. Includes one bit beyond
	 * the highest order one (true) bit.
	 */
	public String toString(  )
		{	
		for (int bit = theBits.size(); bit >= 0; bit-- )
			if ( theBits.get( bit ) )
				return toString( bit + 2 ); //show one zero beyond high "one" bit
		return "00";
		}
	
	/**
	 * Returns a String representation of the bitset using the first
	 * numberOfBits of the bitset.
	 */
	public String toString( int numberOfBits )
		{
		StringBuffer stringValue = new StringBuffer();
		
		for (int bit = numberOfBits - 1; bit >= 0; bit-- )
			{
			if ( theBits.get( bit ) )
				stringValue.append( '1');
			else
				stringValue.append( '0');
			}
		return stringValue.toString();
		}

	/**
	 * Return a string representation of the ConvertableBitSet as if it represented
	 * a float. A "," separates the bit that represent the sign bit (bit 31), 
	 * the bits that represent the exponent (bits 30-23) and the bits that 
	 * represent the mantissa (or significand) (bits 22-0).
	 */
	public String toFloatString()
		{
		String theFloat = toString( 32);
		final String bitSeparator = ",";
		
		// bits are numbered right to left, strings left to right
		return theFloat.substring( 0, 1) + bitSeparator +
					theFloat.substring( 1, 9) + bitSeparator +
					theFloat.substring( 9);
		}
	/**
	 * Return a string representation of the ConvertableBitSet as if it represented
	 * a double. A "," separates the bit that represent the sign bit (bit 63), 
	 * the bits that represent the exponent (bits 62-52) and the bits that 
	 * represent the mantissa (or significand) (bits 51-0).
	 */
	public String toDoubleString()
		{
		String theDouble = toString( 64);
		final String bitSeparator = ",";
		
		// bits are numbered right to left, strings left to right
		return theDouble.substring( 0, 1) + bitSeparator +
					theDouble.substring( 1, 12) + bitSeparator +
					theDouble.substring( 12);
		}
	}