/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.util;

import java.util.Properties;
import java.io.File;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import sdsu.util.LabeledData;
import sdsu.io.LocalRepository;

/**
 * ProgramProperties obtains program options from 
 * command line arguments and configuration files. Program options are
 * given in key-value (or name-value) pairs as in a hastable. Given a
 * key (in a get method), the ProgramProperties first checks the 
 * command line flags for a pair
 * with that key. If the key is found in the command line the 
 * corresponding value is returned. If the key is not found the 
 * configuration file is checked. One can give a default value in the 
 * program for cases where the key is not in the command line or the
 * configuration file. <P>
 * Flags are indicated in command line via 
 * "-" before the flag. Supported syntax:<br>
 * <B> -flag=value</B> 
 * <br><B> -flag value</B> The string "flag" 
 * (without the '-' and without the "'s)  
 * is stored as key. The string "value" is the 
 * associated value in the hashtable. If 
 * -flag is the last argument or the argument 
 * after -flag starts with a '-', 
 * the string "NO_VALUE" is stored as value for the flag.
 * <br><B> --flagChars</B> Individual characters 
 * after the -- are treated as separate
 * flags. The string "NO_VALUE" is stored as value for each flag.
 * <br><B>--</B> ignore rest of the command line arguments. <P>
 * Configuration files
 * can either contain ASCII versions of java.util.Properties or
 * sdsu.util.LabeledData objects. The basic format for a Properties 
 * file is:
 * <pre>
 *		#A comment to the end of the line, they go between lines of data
 *		key1=value1
 *		key2=value2	with spaces
 *		key3 with spaces=value3	#part of the value, not a comment
 * </pre> 
 * The basic format for a LabeledData file is:
 * <pre>
 *		#A comment to the end of the line, they go between lines of data
 *		key1 = value1;
 *		key2='value2	with spaces';
 *		'key3 with spaces'=value3;		# a comment
 * </pre>  
 * The actual name of a Properties file must end in ".properties".
 * The actual name of a LabeledData file must end in ".labeledData".
 * The file extension comparison used is not case sensitive.
 * The name of the configuration file is given in the constructor.
 * The name does not have to include the file extension 
 * ".properties" or ".labeledData", the extensions will be added.
 * This means you should not have files with the same name different
 * extensions in the same directory. The command line flag 
 * "-conf=fileName" overrides the file name given in the 
 * constructor.<P>
 * If you need many configuration files for a program 
 * you may wish to use a Respository instead of this class.
 *
 * @see java.util.Properties
 * @see sdsu.util.LabeledData
 * @see sdsu.io.LocalRepository
 * @version 1.0 13 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class ProgramProperties
	{
	private static final String FILE_OPTIONS_KEY = "conf";
	
	LabeledData flags = new LabeledData();
	Properties fileOptions = new Properties();
	
	/**
	 * Create a new ProgramProperties from data in command
	 * line arguments.
	 * @param  mainArguments arguments passed to main.
	 *	@exception IOException if there is a problem reading or parsing
	 *		the configuration file for program, which can be 
	 *		specified in the command line.
	 * @exeption FileNotFoundException if can not find requested 
	 *		configuration file (requested in command line). 
	 *		FileNotFoundException is subclass of IOException.
	 */
	public ProgramProperties( String[] mainArguments )
		throws IOException
		{
		this( mainArguments, null );
		}

	/**
	 * Create a new ProgramProperties from data in command
	 * line arguments.
	 * @param configurationFileName name of configuration file for program.
	 * 	File must contain either a java.util.Properties or
	 *		sdsu.util.LabeledData. Actual file name must end in either
	 *		".properties" or ".labeledData". The suffex can be omitted from 
	 *		the file name given here. 
	 *	@exception IOException if there is a problem reading or parsing
	 *		the configuration file for program.
	 * @exeption FileNotFoundException if can not find requested 
	 *		configuration file. 
	 *		FileNotFoundException is subclass of IOException.
	 */
	public ProgramProperties( String configurationFileName ) 
		throws IOException
		{
		fileOptions = getFileOptions( configurationFileName );
		}

	/**
	 * Create a new ProgramProperties from data in command
	 * line arguments.
	 * @param  mainArguments arguments passed to main.
	 * @param configurationFileName name of configuration file for program.
	 * 	File must contain either a java.util.Properties or
	 *		sdsu.util.LabeledData. Actual file name must end in either
	 *		".properties" or ".labeledData". The suffex can be omitted from 
	 *		the file name given here. 
	 *	@exception IOException if there is a problem reading or parsing
	 *		the configuration file for program.
	 * @exeption FileNotFoundException if can not find requested 
	 *		configuration file. 
	 *		FileNotFoundException is subclass of IOException.
	 */
	public ProgramProperties( String[] mainArguments, 
										String configurationFileName )
		throws IOException
		{
		flags.fromCommandLine( mainArguments );
		fileOptions = getFileOptions( configurationFileName );
		}

	/**
	 * Create a new ProgramProperties from data in command
	 * line arguments where configuration file existance is optional.
	 * @param  mainArguments arguments passed to main.
	 * @param configurationFileName name of configuration file for program.
	 * 	File must contain either a java.util.Properties or
	 *		sdsu.util.LabeledData. Actual file name must end in either
	 *		".properties" or ".labeledData". The suffex can be omitted from 
	 *		the file name given here. 
	 *	@exception IOException if there is a problem reading or parsing
	 *		the configuration file for program.
	 * @exeption FileNotFoundException if can not find requested 
	 *		configuration file. 
	 *		FileNotFoundException is subclass of IOException.
	 */
	protected ProgramProperties( String[] mainArguments, 
										String configurationFileName,
										boolean isFileOptional )
		throws IOException
		{
		flags.fromCommandLine( mainArguments );
		try
			{
			fileOptions = getFileOptions( configurationFileName );
			}
		catch (FileNotFoundException  ignore)
			{
			// Internal state if ok, just could not find file
			// which user claims is ok. 
			fileOptions = new Properties(); 
			}
		}

	/**
	 * Create a new ProgramProperties, where the configuration
	 * file is optional. That is if the listed file can not be found
	 * no exception is thrown. Only those key-value pairs given in the 
	 * command line or those given default values will exist.
	 * @param  mainArguments arguments passed to main.
	 * @param configurationFileName name of configuration file for program.
	 * 	File must contain either a java.util.Properties or
	 *		sdsu.util.LabeledData. Actual file name must end in either
	 *		".properties" or ".labeledData". The suffex can be omitted from 
	 *		the file name given here. 
	 *	@exception IOException if there is a problem reading
	 *		the configuration file for program.
	 */
	public static ProgramProperties fileOptional( String[] mainArguments, 
										String configurationFileName)
		throws IOException
		{
		return  new ProgramProperties( mainArguments, 
											configurationFileName, true );
		}
	
	/**
	 * Returns the string value associated with the 
	 * given key. Returns null if key does not exist.
	 */
	public String getString( String key )
		{
		return getString( key, null );
		}

	/**
	 * Returns the string value associated with the 
	 * given key. Returns defaultValue if key does not exist.
	 */
	public String getString( String key, String defaultValue )
		{
		if ( flags.containsKey( key ))
			return flags.getData( key );
		else if ( fileOptions.containsKey( key ))
			return fileOptions.getProperty( key );
		else 
			return defaultValue;
		}

	/**
	 * Returns the int value associated with the given key. 
	 * @exception NumberFormatException if key does not exist.
	 * 	or the value at the key can not be converted to an int.
	 */
	public int getInt( String key ) throws NumberFormatException
		{
		String intString = getString( key, null );
		return Integer.valueOf( intString ).intValue();
		}

	/**
	 * Returns the int value associated with the 
	 * given key. Returns defaultValue if the key does not exist.
	 * @exception NumberFormatException if the value at the key can 
	 *		not be converted to an int.
	 */
	public int getInt( String key, int defaultValue )
		throws NumberFormatException
		{
		String intString = getString( key, null );
		if (intString == null )
			return defaultValue;
		else
			return Integer.valueOf( intString ).intValue();
		}

	/**
	 * Returns the float value associated with the given key. 
	 * @exception NumberFormatException if key does not exist.
	 * 	or the value at the key can not be converted to an float.
	 */
	public float getFloat( String key ) throws NumberFormatException
		{
		String floatString = getString( key, null );
		return Float.valueOf( floatString ).floatValue();
		}

	/**
	 * Returns the float value associated with the 
	 * given key. Returns defaultValue if the key does not exist.
	 * @exception NumberFormatException if the value at the key can 
	 *		not be converted to an float.
	 */
	public float getFloat( String key, float defaultValue )
		throws NumberFormatException
		{
		String floatString = getString( key, null );
		if (floatString == null )
			return defaultValue;
		else
			return Float.valueOf( floatString ).floatValue();
		}

	/**
	 * Returns true if ProgramProperties contains the 
	 * given key. Returns false otherwise.
	 */
	public boolean containsKey( String key )
		{
		return flags.containsKey( key ) || 
				fileOptions.containsKey( key );
		}
	
	/**
	 * Opens the proper configuration file and returns the 
	 * contents as a Properties. File must contain a 
	 * Property or LabeledData
	 * @exception	IOException if problems reading the file
	 */
	private Properties getFileOptions( String fileName ) 
		throws IOException
		{
		// check to see if command line specifies different 
		// config file
		fileName = flags.getData( FILE_OPTIONS_KEY, fileName );
		
		Properties options;

		// handle degenerate cases
		if ( fileName == null ) 
			return new  Properties();
		else if ( fileName.length() <= 0 )
			return new  Properties();
		
		//explicit file endings		
		if ( 	fileName.toLowerCase().endsWith( ".properties") )
			{
			options = new Properties();
			options.load( new BufferedInputStream( new FileInputStream(fileName)));
			return options;
			}
		else if ( fileName.toLowerCase().endsWith( ".labeleddata") )
			{
			options = new LabeledData();
			options.load( new BufferedInputStream( new FileInputStream(fileName)));
			return options;
			}
		
		//implicit file endings - have Repository get file
		//Get directory holding file
		File partialName = new File( fileName );
		String parent = partialName.getParent();
		
		if ( parent == null )
			parent = ".";	//"" = current directory
			
		String filePrefix = partialName.getName();
	
		LocalRepository programData = new LocalRepository( parent );
		Properties configFile = (Properties)  programData.get(filePrefix);
		
		if (configFile == null )
			throw new FileNotFoundException( "Could not find file: "
									+ fileName );
		else
			return configFile;

		}

	}